
# line 2 "./ascParse.y"
/*
 *  Ascend Grammar file
 *  by Tom Epperly
 *  Version: $Revision: 1.1 $
 *  Version control file: $RCSfile: ascParse.c.no.yacc,v $
 *  Date last modified: $Date: 97/09/04 13:09:55 $
 *  Last modified by: $Author: kt2g $
 *
 *  This file is part of the Ascend Language Interpreter.
 *
 *  Copyright (C) 1990, 1993, 1994 Thomas Guthrie Epperly
 *  Copyright (C) 1997 Benjamin Andrew Allan & Vicente Rico-Ramirez
 *
 *  The Ascend Language Interpreter is free software; you can redistribute
 *  it and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  The Ascend Language Interpreter is distributed in hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the program; if not, write to the Free Software Foundation,
 *  Inc., 675 Mass Ave, Cambridge, MA 02139 USA.  Check the file named
 *  COPYING.
 *
 */
#include<stdio.h>
#include<limits.h>		/* need DBL_MAX and LONG_MAX */
#include<float.h>		/* on a NeXT they are in here */
#include"utilities/ascConfig.h"
#include"utilities/ascMalloc.h"
#include"general/list.h"
#include"general/dstring.h"
#include"compiler/compiler.h"
#include"compiler/scanner.h"
#include"compiler/symtab.h"		/* the global string/symbol table */
#include"compiler/fractions.h"
#include"compiler/dimen.h"
#include"compiler/functype.h"
#include"compiler/func.h"
#include"compiler/types.h"
#include"compiler/name.h"
#include"compiler/nameio.h"
#include"compiler/instance_enum.h"
#include"compiler/extfunc.h"
#include"compiler/packages.h"
#include"compiler/sets.h"
#include"compiler/exprs.h"
#include"compiler/exprio.h"
#include"compiler/vlist.h"
#include"compiler/vlistio.h"		/* for debugging only */
#include"compiler/stattypes.h"
#include"compiler/slist.h"
#include"compiler/statement.h"
#include"compiler/statio.h"
#include"compiler/units.h"
#include"compiler/when.h"
#include"compiler/select.h"
#include"compiler/switch.h"
#include"compiler/proc.h"
#include"compiler/procio.h"
#include"compiler/module.h"
#include"compiler/child.h"
#include"compiler/type_desc.h"
#include"compiler/type_descio.h"
#include"compiler/typedef.h"
#include"compiler/library.h"
#include"compiler/syntax.h"

#ifndef lint
static CONST char ParserID[] = "$Id: ascParse.c.no.yacc,v 1.1 97/09/04 13:09:55 kt2g Exp $";
#endif

int g_compiler_warnings = 1;		/* level of whine to allow */

FILE *g_ascend_errors = stderr;		/* error file */
FILE *g_ascend_warnings = stderr;	/* whine file */
FILE *g_ascend_information = stderr;	/* note file */
#ifndef ASCERR
#define ASCERR g_ascend_errors
#define ASCWAR g_ascend_warnings
#define ASCINF g_ascend_information
#endif

extern int zz_error(char *);
/*  provided at the end of this file.
 *  it increments g_untrapped_error, the global defined below
 */

static unsigned long g_header_linenum = 0;
static unsigned int g_untrapped_error = 0;
/* if g_untrapped_error is set to 1, the current definition
 * should be abandoned even if nothing else detectable by
 * typedef.c exists. basically any syntax error should cause a type
 * to be abandoned, but not all cause the parser to barf.
 */

static CONST char *g_type_name=NULL;
static CONST char *g_refines_name=NULL;
/*********************************************************************\
 * g_type_name is used by the scanner when closing a module to check if
 * the parser is in the middle of parsing a MODEL or ATOM type
 * definition.
 *      g_type_name == NULL implies that the parser is in between
 *                          definitions
 *      g_type_name != NULL implies that the parser is in the middle
 *                          of a type definition and gives the name
 *                          of that type.
 *      g_tail_name should shadow g_type_name, but we don't guarantee
 *      its sanity between types, only during a production.
\*********************************************************************/

static CONST char *g_end_identifier = NULL;
/*  This variable gets reset every time we see an ``END_T''.  If the
 *  token after END_T is missing (i.e., ``END;'') or if it is recognized
 *  (e.g., FOR_T), set this variable to NULL.  When we see an
 *  ``END_T IDENTIFIER_T'', set this variable to the pointer into the
 *  symbol table returned by the scanner.
 */

static CONST char *g_type_notes = NULL;
/*  The text of notes.
 */

static struct StatementList *g_model_parameters=NULL;
/* this is the statementlist of the parameterized type
 */

static struct StatementList *g_parameter_wheres=NULL;
/* this is the where list of a parameterized type. restrictions on args
 */

static struct StatementList *g_parameter_reduction=NULL;
/* this is the statementlist of the REFINES clause
 * at present both might be null, which is bad.
 */

static struct Set *g_typeargs=NULL;
/* optional args to a typename in part declaration.
 * it is set in the production type_identifier. in
 * contexts where args are not allowed, use IDENTIFIER_T instead of the
 * type_identifier production.
 */

static struct Set *g_callargs=NULL;
/* optional args to a user defined method.
 * it is set in the production call_identifier. in
 * contexts where args are not allowed, use IDENTIFIER_T instead of the
 * type_identifier production.
 */

static int g_defaulted;			/* used for atoms,constants */
struct StatementList *g_statementlist_ptr=NULL;	/* for interactive mode */

static CONST dim_type *g_dim_ptr;	  /* dim of last units parsed, or so */
static CONST dim_type *g_atom_dim_ptr;	  /* dim of DIMENSION decl */
static CONST dim_type *g_default_dim_ptr; /* dim of default value parsed */

static double g_default_double;
static long g_default_long;
CONST char *g_default_symbol;
#define DOUBLECONSTANT 0
#define LONGCONSTANT 1
#define BOOLEANCONSTANT 2
#define SYMBOLCONSTANT 3
static int g_constant_type = DOUBLECONSTANT;
static CONST struct Units *g_units_ptr;

int g_parse_relns = 1;
/*  Should we parse relations?
 *      0 indicates don't parse relations
 *      1 indicates process them
 */

/*  Forward declaration of error message reporting
 *  functions provided at the end of this file.
 */
static void ErrMsg_Generic(CONST char *);
static void ErrMsg_NullDefPointer(CONST char *);
static void ErrMsg_ParensBrackets(CONST char *);
static void WarnMsg_MismatchEnd(CONST char *, CONST char *,
                                unsigned long, CONST char *);
static CONST char *TokenAsString(unsigned long);

/*
 *  NOTES  Explanations  NOTES  Explanations  NOTES  Explanations  NOTES
 *
 *  Mark Thomas  Thursday, 13 March 1997
 *
 *  The parser will now parse a NOTES block as well as in-lined NOTES.  As
 *  a matter of fact, the parser would probably parse FORTRAN now since it
 *  is very lenient.  For the in-lined NOTES, I accept any "doubly-quoted"
 *  string after an `fname'.  I am currently dropping the NOTE on the
 *  floor.  Ideally, we should only accept an inline NOTE when we are
 *  creating a new thing, like in an IS_A statement or a labeled relation.
 *  That means either adding the optional_note to fname and whining at the
 *  user when he/she uses the in-lined notes incorrectly, or coming up
 *  with a fvarnotelist and fnamenote that accepts fnames and
 *  optional_notes in a list or a stand-alone form.
 *
 *  For the block NOTES statement, the symtax is
 *
 *      NOTES ( SYMBOL_T ( fvarlist BRACEDTEXT_T )+ )+ END NOTES ';'
 *
 *  Here I am using () as part of the meta-language to describe the syntax
 *  to you, they are not part of the formal grammer.  An example is
 *
 *      NOTES
 *          'text'  a,b,c  { Here is some text }
 *                  d      { Here is more text }
 *          'html'  SELF   { <bold>html sucks</bold> }
 *      END NOTES;
 *
 *  Note that the only punctuation is the `,' between the members of the
 *  fvarlist and the closing `;'.  Right now, the term `SELF' would be
 *  eaten in the fvarlist production.  I'm not sure if this is what we
 *  should do (which requires having the notes processing do something
 *  special when it sees SELF in the fvarlist), or if we should create
 *  a SELF_T token.  The latter is certainly easier to implement from
 *  the parser's perspective, which is why I did it that way.
 *
 *  The block NOTES statement doesn't do anything with its stuff either,
 *  the symbols and {bracedText} get dropped on the floor and I destroy
 *  the fvarlist, but that all that happens.
 *
 *  The `notes_body' and `noteslist' productions return `notes_ptr', which
 *  right now is a `void*' until we can decide what type of data structure
 *  we want to handle NOTES.
 *
 *  As an amusing side note, the parser will currently eat the following:
 *
 *      NOTES
 *          'fun' name  "put some text here"  {put more text here}
 *      END NOTES;
 *
 *  Like I said, this is so the parser will eat them; it's not being real
 *  smart about what it does with them.
 *
 *  For debugging the NOTES stuff, set the DEBUG_NOTES macro to the
 *  following:
 *
 *  #define DEBUG_NOTES(s) FPRINTF(stderr,"****DISCARDED NOTES:\n%s****\n",(s))
 *
 *  To completely ignore the NOTES, set DEBUG_NOTES to the following:
 *
 *  #define DEBUG_NOTES(s) 0
 *
 *  Note that if you do the latter, you'll get `statement with no effect'
 *  warnings from gcc -Wall.
 */
#define DEBUG_NOTES(s) 0

# line 258 "./ascParse.y"
typedef union  {
  double real_value;
  long   int_value;
  struct fraction frac_value;
  CONST char *id_ptr;
  CONST char *braced_ptr;	/* pointer for units, explanations, tables */
  CONST char *sym_ptr;		/* pointer for symbols */
  CONST char *dquote_ptr;       /* for text in "double quotes" */
  struct Name *nptr;
  struct Expr *eptr;
  struct Set *sptr;
  struct VariableList *lptr;
  struct Statement *statptr;
  struct StatementList *slptr;
  struct SelectList *septr;
  struct SwitchList *swptr;
  struct WhenList *wptr;
  void *notesptr;		/* change this once struct Notes is defined */
  struct gl_list_t *listp;
  struct InitProcedure *procptr;
  CONST dim_type *dimp;
  struct TypeDescription *tptr;
  struct UnitDefinition *udefptr;
  dim_type dimen;
  enum ForOrder order;
} ZZ_STYPE;
#ifdef __cplusplus
#  include <stdio.h>
   extern "C" {
     extern void zz_error(char *);
     extern int zz_lex();
   }
#endif	/* __cplusplus */ 
# define ADD_T 257
# define ALIASES_T 258
# define AND_T 259
# define ANY_T 260
# define AREALIKE_T 261
# define ARETHESAME_T 262
# define ARRAY_T 263
# define ATOM_T 264
# define BEQ_T 265
# define BNE_T 266
# define BREAK_T 267
# define CALL_T 268
# define CARD_T 269
# define CASE_T 270
# define CHOICE_T 271
# define CONDITIONAL_T 272
# define CONSTANT_T 273
# define CREATE_T 274
# define DATA_T 275
# define DECREASING_T 276
# define DEFAULT_T 277
# define DEFINITION_T 278
# define DIMENSION_T 279
# define DIMENSIONLESS_T 280
# define DO_T 281
# define ELSE_T 282
# define END_T 283
# define EXTERNAL_T 284
# define FALSE_T 285
# define FOR_T 286
# define FROM_T 287
# define IF_T 288
# define IMPORT_T 289
# define IN_T 290
# define INPUT_T 291
# define INCREASING_T 292
# define INTERACTIVE_T 293
# define INTERSECTION_T 294
# define ISA_T 295
# define _IS_T 296
# define ISREFINEDTO_T 297
# define MAXIMIZE_T 298
# define MAXINTEGER_T 299
# define MAXREAL_T 300
# define METHODS_T 301
# define METHOD_T 302
# define MINIMIZE_T 303
# define MODEL_T 304
# define NOT_T 305
# define NOTES_T 306
# define OF_T 307
# define OR_T 308
# define OTHERWISE_T 309
# define OUTPUT_T 310
# define PATCH_T 311
# define PROD_T 312
# define PROVIDE_T 313
# define REFINES_T 314
# define REPLACE_T 315
# define REQUIRE_T 316
# define RUN_T 317
# define SATISFIED_T 318
# define SELECT_T 319
# define SIZE_T 320
# define SUCHTHAT_T 321
# define SUM_T 322
# define SWITCH_T 323
# define THEN_T 324
# define TRUE_T 325
# define UNION_T 326
# define UNITS_T 327
# define UNIVERSAL_T 328
# define WHEN_T 329
# define WHERE_T 330
# define WILLBE_T 331
# define WILLBETHESAME_T 332
# define WILLNOTBETHESAME_T 333
# define ASSIGN_T 334
# define CASSIGN_T 335
# define DBLCOLON_T 336
# define USE_T 337
# define LEQ_T 338
# define GEQ_T 339
# define NEQ_T 340
# define DOTDOT_T 341
# define WITH_T 342
# define VALUE_T 343
# define WITH_VALUE_T 344
# define REAL_T 345
# define INTEGER_T 346
# define IDENTIFIER_T 347
# define BRACEDTEXT_T 348
# define SYMBOL_T 349
# define DQUOTE_T 350
# define UMINUS_T 351
# define UPLUS_T 352
#define zz_clearin zz_char = -1
#define zz_errok zz_errflag = 0
extern int zz_char;
#ifndef ZZ_MAXDEPTH
#define ZZ_MAXDEPTH 150
#endif

/* __ZZ_SCLASS defines the scoping/storage class for global objects
 * that are NOT renamed by the -p option.  By default these names
 * are going to be 'static' so that multi-definition errors
 * will not occur with multiple parsers.
 * If you want (unsupported) access to internal names you need
 * to define this to be null so it implies 'extern' scope.
 * This should not be used in conjunction with -p.
 */
#ifndef __ZZ_SCLASS
# define __ZZ_SCLASS static
#endif
ZZ_STYPE zz_lval;
__ZZ_SCLASS ZZ_STYPE zz_val;
typedef int zz_tabelem;
# define ZZ_ERRCODE 256

# line 2340 "./ascParse.y"

/*
 * We really need to do something about freeing up the productions
 * that invoke this so we don't leak memory like a seive.
 * for example  z[i IN [1..2]][j IN [process[i]] IS_A mass; eats a ton.
 */
int
zz_error(char *s)
{
  g_untrapped_error++;
  FPRINTF(ASCERR,"%s",s);
  if (Asc_CurrentModule() != NULL) {
    FPRINTF(ASCERR," %s:%lu.\n",
            Asc_ModuleFileName(Asc_CurrentModule()),LineNum());
  } else {
    FPRINTF(ASCERR," at end of input.\n");
  }
  return 0;
}


/*
 *  See the header file scanner.h for a description of this function.
 */
void
Asc_ErrMsgTypeDefnEOF(void)
{
  /*  Check g_type_name to see if we're in the middle of a type
   *  definition.  If NULL no, otherwise yes.
   */
  if ( g_type_name ) {
    FPRINTF(ASCERR,
            "Error:\tEnd of file reached in a type definition.\n"
            "\tIncomplete definition for %s on line %s:%lu\n",
            g_type_name,
            (Asc_CurrentModule()
             ? Asc_ModuleFileName(Asc_CurrentModule())
             : "<UNKNOWN>"),
            LineNum());
  }
}


/*
 *  void ErrMsg_*(void)
 *
 *  The following print error and warning messages to the filehandles
 *  ASCERR and ASCWARN, respectively.
 *  The type of error/warning that will be printed is indicated by the
 *  functions name and the arguments to fprintf.
 */
static void
ErrMsg_Generic(CONST char *string)
{
  struct module_t *mod;

  /* the module may have be already closed */
  mod = Asc_CurrentModule();
  FPRINTF(ASCERR,
          "%son line %s:%lu\n",
          string,
          (mod ? Asc_ModuleFileName(mod) : "<UNKNOWN>"),
          LineNum());
}

#if 0
static void
ErrMsg_NotAllowedDeclarative(CONST char *state_type)
{
  FPRINTF(ASCERR,
          "%sIn the declarative section %s:%lu.\n"
          "  %s statements are not allowed.\n",
	  StatioLabel(3),
          Asc_ModuleFileName(Asc_CurrentModule()),
          LineNum(),
          state_type);
}

static
void ErrMsg_NotAllowedMethod(CONST char *state_type)
{
  FPRINTF(ASCERR,
          "%sIn the method section %s:%lu.\n"
          "  %s statements are not allowed.\n",
          StatioLabel(3),
          Asc_ModuleFileName(Asc_CurrentModule()),
          LineNum(),
          state_type);
}
#endif /* 0 deleteme */

static void
ErrMsg_NullDefPointer(CONST char *object)
{
  FPRINTF(ASCERR,
          "%sRejected \"%s\" at line %s:%lu.\n\n",
          StatioLabel(3),
          object,
          Asc_ModuleFileName(Asc_CurrentModule()),
          LineNum());
}

static void
ErrMsg_ParensBrackets(CONST char *operation)
{
  FPRINTF(ASCERR,
          "  You should be using %s[] not %s()\nat line %s:%lu.\n",
          operation,
          operation,
          Asc_ModuleFileName(Asc_CurrentModule()),
          LineNum());
}


/*
 *  WarnMsg_MismatchEnd(statement, opt_name, end_token, expecting);
 *      const char *statement;
 *      const char *opt_name;
 *      unsigned long end_token;
 *      const char *expecting;
 *
 *  Print a warning message that the token after the END keyword did not
 *  match what we were expecting for the current statement.
 *  Arguments:
 *      statement --the current statement, e.g. ATOM, METHOD, FOR, IF, CASE
 *      opt_name  --the name of the thing we were defining for ATOMs, METHODs,
 *                  etc, or NULL anonymous statements (FOR, IF, CASE, etc)
 *      end_token --the TOKEN_T that we were received instead.  We use the
 *                  TokenAsString to produce a string given a TOKEN_T
 *      expecting --the keyword we were expecting to see after the END; if
 *                  NULL, we were expecting the string given in statement
 */
static void
WarnMsg_MismatchEnd(CONST char *statement, CONST char *opt_name,
		    unsigned long end_token, CONST char *expecting)
{
  FPRINTF(ASCWAR,
          "%s %s %s terminated with\n"
          "  END %s;     ---expecting:  END %s;\n"
          "  at %s:%lu.\n",
          StatioLabel(2),
          statement,
          ((opt_name != NULL) ? opt_name : "statement"),
          TokenAsString(end_token),
          ((expecting != NULL) ? expecting : statement),
          Asc_ModuleFileName(Asc_CurrentModule()),
          LineNum());
}


/*
 *  CONST char *TokenAsString(token);
 *      unsigned long token;
 *
 *  Takes a TOKEN_T (e.g., FOR_T, MODEL_T, END_T, IDENTIFIER_T) and returns
 *  a string representation of it:
 *      e.g.:  TokenAsString(FOR_T) ==> "FOR"
 *
 *  Since this function is only used inside WarnMsg_MismatchEnd, we do a
 *  couple of things specific to that function:  If token is END_T, we
 *  return an empty string, and if it is IDENTIFIER_T, we return the
 *  current value of g_end_identifier, or UNKNOWN if g_end_identifier is
 *  NULL.
 */
static CONST char *
TokenAsString(unsigned long token)
{
  switch( token ) {
  case ATOM_T:
    return "ATOM";
  case CONDITIONAL_T:
    return "CONDITIONAL";
  case FOR_T:
    return "FOR";
  case IF_T:
    return "IF";
  case INTERACTIVE_T:
    return "INTERACTIVE";
  case METHOD_T:
    return "METHOD";
  case METHODS_T:
    return "METHODS";
  case MODEL_T:
    return "MODEL";
  case NOTES_T:
    return "NOTES";
  case PATCH_T:
    return "PATCH";
  case SELECT_T:
    return "SELECT";
  case SWITCH_T:
    return "SWITCH";
  case UNITS_T:
    return "UNITS";
  case WHEN_T:
    return "WHEN";
  case END_T:
    return "";
  case IDENTIFIER_T:
  default:
    if( g_end_identifier != NULL ) {
      return g_end_identifier;
    } else {
      return "UNKNOWN";
    }
  }
}
__ZZ_SCLASS zz_tabelem zz_exca[] ={
-1, 1,
	0, -1,
	264, 56,
	273, 56,
	304, 56,
	-2, 0,
-1, 47,
	270, 72,
	282, 72,
	283, 72,
	301, 72,
	309, 72,
	41, 72,
	-2, 0,
-1, 100,
	258, 180,
	261, 180,
	262, 180,
	295, 180,
	296, 180,
	297, 180,
	331, 180,
	332, 180,
	333, 180,
	44, 180,
	-2, 237,
-1, 437,
	44, 180,
	59, 180,
	58, 180,
	-2, 237,
	};
# define ZZ_NPROD 278
# define ZZ_LAST 1221
__ZZ_SCLASS zz_tabelem zz_act[]={

   149,    70,   121,   293,   513,   209,   468,   298,   303,   429,
    46,   421,   210,    71,   207,   472,   472,   473,   473,    33,
    56,    33,    34,   152,    31,   166,   424,   234,   420,   462,
   424,   398,   372,   347,   254,    48,   280,   204,    71,   167,
   558,   168,    33,   202,   213,   211,   169,   212,   198,   214,
    64,   547,    65,   537,   170,   145,    69,   312,   431,   171,
   139,   514,   224,   223,   225,    57,   139,   464,   562,   427,
   277,   100,   172,   281,   271,   270,   173,   430,   266,   310,
   174,   259,   175,   258,   257,   194,   193,   192,   153,   148,
    68,    67,   213,   211,    66,   212,   215,   214,    52,    45,
   176,    36,   147,   557,   546,    99,   529,   190,   291,   426,
   224,   223,   225,   181,   488,   273,   184,   185,   359,   161,
   160,   311,   205,   550,   295,   554,   221,   365,   235,   195,
   305,   199,   200,   199,   199,   156,   300,   391,   141,   385,
    15,    26,   255,   379,   215,    42,    51,    50,   539,   528,
   179,   182,   183,   552,    43,   290,   265,    60,   143,   267,
   268,   269,    22,   296,   142,   197,   275,   201,   203,   306,
    71,   144,   495,    18,   274,   301,   392,    23,   386,   496,
   418,   446,   380,   356,   357,    44,   180,   186,   187,   559,
   355,    61,   476,   215,   364,    29,   413,    17,   276,    27,
    16,   475,   411,   409,   101,   279,   474,   308,   309,   407,
   405,    30,    28,   213,   211,   403,   212,   327,   214,   520,
   475,   261,   199,   250,   213,   474,   199,   260,   199,   214,
    63,   199,   313,   541,   248,   246,   244,   242,   523,   240,
   521,   329,   330,   331,   332,   333,   334,   335,   336,   337,
   338,   339,   340,   476,   262,   553,   297,   481,   469,   512,
   302,   216,   307,   328,    56,   215,   489,   229,   230,   490,
   491,    56,   476,   467,   251,   414,   215,    59,   444,   363,
   354,   353,   352,   140,    55,   249,   247,   245,   243,   373,
   241,   351,   220,   350,   443,   378,   381,   480,   470,   346,
   384,   387,   345,   377,   199,   390,   393,   344,   342,    57,
   217,   199,   264,   199,   138,   376,    57,   129,   178,   130,
   177,   482,   471,   222,   210,   165,    58,    54,    49,   555,
   542,   507,   423,   422,   199,   199,   423,   422,   368,    33,
   226,   227,   228,    33,    33,   371,   503,   374,   158,   159,
    33,   499,   459,   164,   458,    33,   454,   453,   449,   536,
   448,   394,   416,   388,   382,   127,    25,   432,   472,   397,
   473,   401,   428,   360,   400,   438,   208,   433,   565,   278,
   447,   560,   282,   188,   189,   548,   452,   451,   226,   227,
   228,   151,   457,   450,   437,   285,   199,   456,   543,   455,
   538,   511,   493,   463,   461,   460,   487,   485,   441,   440,
   434,   412,   410,   408,    98,   406,   404,   402,   466,   395,
   199,   138,   256,   389,   129,   383,   130,   362,   436,   563,
   439,   478,   396,   479,   484,   370,   477,   366,   361,   115,
   492,   289,   288,   239,   233,   163,    24,   497,   343,   114,
    41,   425,    40,   502,   397,    39,   157,   498,   506,   500,
   501,    38,   358,   510,   504,   505,    32,    35,   154,   508,
   509,   515,   127,   367,    37,    14,   369,    13,    12,    11,
    10,     9,   515,   519,   522,     8,   516,   517,     7,     6,
     5,   138,     4,     3,   129,   524,   130,   530,   196,     2,
   146,    20,    19,   442,   419,   531,   155,    53,   231,   232,
   532,    47,   199,   199,   533,    62,   206,   294,   534,   304,
   236,   237,   238,   299,   272,   483,   162,   445,    95,    97,
   252,    96,   544,   545,   125,   213,   211,    94,   212,    93,
   214,    92,   127,   136,    91,   137,   525,   527,    90,    89,
    88,    87,    86,   224,   223,   225,    85,    84,    83,   124,
    82,    81,    80,    79,   564,   549,    78,    77,   135,    76,
    75,    74,    72,   120,   122,   113,   556,   526,   435,   128,
   283,   284,   115,   561,   150,   219,   133,   215,   218,   551,
   540,   191,   131,   535,   518,   494,   132,    21,   253,   123,
   134,   417,     1,   314,   315,   316,   317,   318,   319,   320,
   321,   322,   323,   324,   325,     0,     0,   221,     0,   139,
   119,   118,     0,   126,     0,   326,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,    73,     0,   213,
   211,   125,   212,     0,   214,     0,     0,     0,     0,   102,
   136,     0,   137,   111,   348,   349,     0,   224,   223,   225,
     0,     0,     0,     0,     0,   103,   124,   104,     0,   106,
     0,     0,     0,     0,     0,   135,     0,     0,     0,   117,
   120,   122,   375,     0,   116,     0,   128,   112,     0,     0,
     0,   215,     0,   133,     0,     0,     0,     0,   105,   131,
   109,     0,     0,   132,   110,     0,   123,   134,    30,     0,
   107,   125,     0,     0,     0,     0,     0,     0,   108,     0,
   136,   221,   137,     0,     0,     0,   139,   119,   118,     0,
   126,     0,     0,     0,     0,     0,   124,     0,     0,     0,
     0,     0,     0,     0,     0,   135,     0,     0,     0,   117,
   120,   122,   216,     0,   116,     0,   128,     0,   229,   230,
     0,     0,   326,   133,     0,     0,   399,   213,   211,   131,
   212,     0,   214,   132,     0,     0,   123,   134,     0,     0,
   287,     0,     0,   220,     0,   224,   223,   225,     0,   341,
   213,   211,     0,   212,     0,   214,   139,   119,   286,     0,
   126,   217,     0,     0,     0,     0,     0,   465,   224,   223,
   225,     0,     0,     0,   222,     0,     0,     0,     0,   215,
     0,     0,   486,     0,     0,   213,   211,     0,   212,     0,
   214,   226,   227,   228,   415,     0,     0,     0,     0,     0,
     0,     0,   215,   224,   223,   225,     0,     0,     0,   221,
     0,     0,     0,     0,     0,     0,   216,     0,     0,     0,
     0,     0,   229,   230,     0,     0,     0,     0,     0,     0,
     0,     0,   221,     0,     0,     0,     0,   215,   213,   211,
     0,   212,     0,   214,     0,     0,     0,   220,     0,     0,
     0,     0,     0,     0,     0,     0,   224,   223,   225,     0,
     0,   213,   211,     0,   212,   217,   214,   221,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,   222,   224,
   223,   225,   213,   211,     0,   212,     0,   214,     0,     0,
   215,     0,     0,     0,     0,   226,   227,   228,   263,     0,
   224,   223,   225,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,   215,     0,     0,     0,     0,     0,     0,
   221,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,   215,     0,     0,     0,     0,     0,
     0,     0,     0,     0,   216,     0,     0,     0,     0,     0,
   229,   230,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,   216,     0,     0,
     0,     0,     0,   229,   230,   220,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,   217,     0,     0,     0,     0,   220,     0,
     0,     0,   216,     0,     0,     0,   222,     0,   229,   230,
     0,     0,     0,     0,     0,     0,   217,     0,     0,     0,
     0,     0,     0,   226,   227,   228,     0,     0,     0,   222,
     0,     0,     0,   220,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,   226,   227,   228,     0,
     0,   217,     0,     0,     0,   216,     0,     0,     0,     0,
     0,   229,   230,     0,   222,     0,     0,   292,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,   216,     0,
     0,   226,   227,   228,   229,   230,   220,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,   216,
     0,     0,     0,     0,   217,     0,     0,     0,     0,   220,
     0,     0,     0,     0,     0,     0,     0,   222,     0,     0,
     0,     0,     0,     0,     0,     0,     0,   217,     0,     0,
   220,     0,     0,     0,   226,   227,   228,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,   217,     0,
     0,     0,     0,     0,     0,     0,     0,   226,   227,   228,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,   226,   227,
   228 };
__ZZ_SCLASS zz_tabelem zz_pact[]={

 -3000,  -116, -3000, -3000, -3000, -3000, -3000, -3000, -3000, -3000,
 -3000, -3000, -3000, -3000, -3000, -3000,  -326,  -328,  -246, -3000,
 -3000,  -119,  -248, -3000, -3000,   269,  -154,  -155, -3000,  -249,
 -3000,   268,   225, -3000,   267,   218,  -130, -3000, -3000, -3000,
 -3000, -3000,  -253,  -256,  -257, -3000,  -113,   381,  -163, -3000,
  -126,  -132,  -115,  -245, -3000, -3000,  -258,   274, -3000, -3000,
  -259, -3000,  -167, -3000,  -163,  -163,  -194,  -195,   405,  -163,
   266,  -247,   261,   259, -3000, -3000, -3000, -3000, -3000, -3000,
 -3000, -3000, -3000, -3000, -3000, -3000, -3000, -3000, -3000, -3000,
 -3000, -3000, -3000, -3000, -3000, -3000, -3000, -3000, -3000,  -145,
    49, -3000,  -260,  -261,  -262,  -305,   274,     8,  -305,     3,
    -3, -3000,  -335,   332,   -26,   836,   274,   274,   404,  -321,
 -3000, -3000, -3000, -3000, -3000, -3000, -3000,   274,   274,   274,
   274,   403,   199,   197,   196,   195,   194,   183,   274,  -314,
  -113, -3000,  -263,  -264,  -266, -3000, -3000,   166, -3000,   128,
   210,   597, -3000,   253,  -113, -3000,  -269,  -113,  -113,  -113,
  -272,  -273,  -215, -3000,  -113, -3000, -3000, -3000, -3000, -3000,
 -3000, -3000, -3000, -3000, -3000, -3000, -3000, -3000, -3000,  -277,
  -277,    -4,  -274,  -277, -3000, -3000, -3000, -3000,   274,   274,
   451, -3000,   402,   401,  -135,  -228,   783,  -146,  -305, -3000,
 -3000,  -134,  -305,  -140,  -305,  -113,  -270,  -305,  -305, -3000,
 -3000,   274,   274,   274,   274,   274,   274,   274,   274,   274,
   274,   274,   274, -3000, -3000, -3000, -3000, -3000, -3000, -3000,
 -3000,   836,   836,   274, -3000,   124, -3000,    99,    99,  -305,
   274,   274,   274,   274,   274,   274,   274,   274,   274,   274,
   274,   274,   748, -3000, -3000,   249, -3000,   248,   243,   240,
  -315, -3000,   274,   274, -3000,   234,   232,   223,   222,   221,
   -96,   -96,    59,   398,   386,   220,  -180,   397,  -180, -3000,
  -305,  -180, -3000,   836,   836, -3000,   395,    -8,   274,  -305,
   274,  -305, -3000,  -113,  -127,   274,   306,   384,  -113,  -131,
   274,   305,   382,  -113,  -133,   274,   303,   378, -3000, -3000,
  -305,  -305,  -317, -3000,   182,   182,    99,    99,    99,    50,
    50,   171,   880,    50,   859,   859,   725, -3000,   330,   376,
   122,   375,   117,   374,   116,   372,   110,   371,   109,   370,
   103, -3000, -3000, -3000, -3000, -3000, -3000,   216,   493,   836,
 -3000, -3000, -3000, -3000, -3000,   -97,   -14, -3000,  -226, -3000,
  -278, -3000, -3000, -3000,  -267,  -289,   274,  -267,   369, -3000,
   274,  -146,  -305,   368,   367,     2, -3000,  -101, -3000,   274,
   302,   300, -3000,  -146, -3000,   274,   299,   298, -3000,  -134,
 -3000,   274,   296,   294, -3000,  -140,  -305,  -319, -3000, -3000,
  -279, -3000, -3000, -3000, -3000, -3000, -3000, -3000, -3000, -3000,
 -3000, -3000, -3000, -3000, -3000,   274,  -113,   214,   -27, -3000,
 -3000,   159, -3000, -3000,   -10,  -338,   -28,   394,   366, -3000,
   274, -3000,   365, -3000,  -216,   207,   211, -3000,  -113,   361,
 -3000, -3000,  -102, -3000, -3000,  -113, -3000,   293, -3000, -3000,
 -3000,  -113,   288, -3000, -3000, -3000,  -113,   273, -3000, -3000,
 -3000,  -113, -3000,   360,  -321,   836,   200, -3000,  -285, -3000,
 -3000, -3000, -3000, -3000,   -10,   -10,   325,   178,   181,  -285,
 -3000, -3000, -3000,   179, -3000, -3000,   836, -3000,  -305,  -305,
  -142,  -240, -3000,  -146, -3000, -3000, -3000, -3000, -3000, -3000,
 -3000, -3000, -3000, -3000, -3000, -3000, -3000, -3000, -3000, -3000,
 -3000, -3000, -3000, -3000,  -321, -3000,    98,    98, -3000,    13,
 -3000, -3000, -3000, -3000,   359,  -147,   174,   272, -3000,   357,
  -113,  -113, -3000, -3000, -3000, -3000, -3000,  -242, -3000,  -296,
   344,  -305,  -187,  -137, -3000, -3000,   208,  -182, -3000,   271,
 -3000, -3000,  -305,  -243,  -307,   -86, -3000,   340,  -276, -3000,
 -3000, -3000,   389,   274,   337, -3000 };
__ZZ_SCLASS zz_tabelem zz_pgo[]={

     0,   602,   601,     4,     2,   598,     1,     6,   597,   595,
   594,   593,   194,   198,   591,     5,   590,    23,   449,   589,
   204,   391,   588,   585,     9,     0,   584,   583,    57,   578,
   577,   575,   572,   571,   570,   569,   567,   566,   563,   562,
   561,   560,   558,   557,   556,   552,   551,   366,   550,   549,
   548,   544,   541,   539,   537,   531,   529,   528,    10,   527,
   526,   525,   524,     7,   523,     8,   519,     3,   517,   516,
   121,   283,   191,   515,   511,   507,   506,   190,   504,    11,
   503,   502,   501,   500,   499,   493,   492,   490,   489,   488,
   485,   481,   480,   479,   478,   477,   475,   474,   468,   461,
   456,   455,   452,   451,   450,   446,   422 };
__ZZ_SCLASS zz_tabelem zz_r1[]={

     0,     1,     1,    84,    84,    84,    84,    84,    84,    84,
    84,    84,    84,    84,    84,    84,    85,    85,    85,    86,
    86,    86,    87,    97,    98,    88,    81,    99,   100,    89,
    82,    91,   101,     2,     2,     2,     2,     2,    90,   102,
   103,   103,   103,   103,   103,    92,   104,   104,    60,    60,
    62,    62,    61,    61,    95,   105,     8,     8,    94,    93,
    96,    47,    75,    75,    83,    71,   106,    71,    72,    73,
    73,    76,    58,    74,    74,    74,    32,    32,    32,    32,
    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,
    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,
    32,    33,    34,    35,    35,    27,    27,    36,    37,    14,
    14,    13,    13,    12,    12,    24,    24,    38,    39,    40,
    41,    42,    42,    43,    43,    20,    20,    20,    45,    29,
    30,    16,    16,    44,    19,    19,    49,    80,    80,    80,
     9,     9,    50,    50,    48,    46,    51,    59,    59,    52,
    52,    52,    52,    67,    68,    68,    68,    68,    53,    54,
    54,    63,    64,    64,    64,    64,    57,    57,    65,    66,
    66,    66,    66,    55,    56,    69,    69,    70,    70,    28,
    31,    31,    17,    18,    18,    18,     6,     6,     6,     6,
     6,     6,     6,     6,     6,     6,     6,     6,    15,    15,
    25,    25,    26,    26,    26,    26,     3,     3,     4,     4,
     5,     5,    77,    77,    77,    78,    78,    79,    79,    79,
    79,    79,    79,    10,    11,    11,     7,     7,     7,    21,
    21,    21,    21,    21,    21,    21,    21,    21,    21,    21,
    21,    21,    21,    21,    21,    21,    21,    21,    21,    21,
    21,    21,    21,    21,    21,    21,    21,    21,    21,    21,
    21,    21,    21,    21,    21,    21,    21,    21,    21,    21,
    22,    22,    22,    22,    22,    22,    23,    23 };
__ZZ_SCLASS zz_tabelem zz_r2[]={

     0,     0,     4,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     3,     7,     7,     5,     7,
     7,     5,    11,     1,     1,    13,    11,     1,     1,    13,
    11,    13,    15,     1,     7,     5,     5,     5,     5,    17,
     1,     7,     5,     5,     5,    13,    11,    17,     1,     7,
     1,     9,     1,     7,    11,    11,     1,     3,     9,    13,
     5,     7,     1,     5,     9,     1,     1,     7,     3,     1,
     5,    13,     3,     1,     7,     7,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,    11,    11,     7,    25,     1,     9,     9,     7,     3,
     9,     3,     9,     1,     5,     1,     5,     5,     5,     5,
     5,     7,     7,     3,     7,     3,     5,     5,    19,     7,
     7,     1,     9,    19,     1,     5,    17,     1,     3,     3,
     3,     3,     5,     9,    11,     5,    13,     1,     5,     9,
    13,    13,    17,     3,     9,     7,    11,     9,     5,     9,
    13,     3,     9,     7,    11,     9,     9,    13,     3,     9,
     7,    11,     9,     7,     7,     5,     7,     5,     7,     3,
     3,     7,     5,     3,     7,     9,     5,     5,     5,     5,
     5,     5,     5,     5,     5,     5,     5,     3,     1,     3,
     3,     1,     3,     7,     7,    11,     3,     3,     5,     5,
     1,     3,     5,     3,     1,     3,     3,     3,     3,     7,
     7,     7,     7,     5,     3,    11,     1,     3,     3,     3,
     3,     3,     3,     3,     3,     3,     3,     3,     7,     7,
     7,     7,     7,     7,     7,     7,     5,     7,     7,     7,
     7,     7,     5,     5,    13,     9,     9,     9,     9,     9,
     9,     9,     9,     9,     9,     9,     9,     9,     9,     7,
     3,     3,     3,     3,     3,     3,     3,     3 };
__ZZ_SCLASS zz_tabelem zz_chk[]={

 -3000,    -1,   -84,   -85,   -86,   -87,   -88,   -89,   -90,   -91,
   -92,   -93,   -94,   -95,   -96,   256,   316,   313,   289,   -81,
   -82,    -8,   278,   293,  -105,   -47,   257,   315,   328,   311,
   327,   350,   -18,   347,   350,   -18,   347,   -97,   -99,  -101,
  -102,  -104,   264,   273,   304,   347,   -58,   -74,   -58,    59,
   301,   301,   347,   -75,    59,    59,    46,    91,    59,    59,
   287,   -72,   -73,   -72,   -58,   -58,   347,   347,   347,   -58,
    -6,   283,   -32,   256,   -33,   -34,   -35,   -36,   -37,   -38,
   -39,   -40,   -41,   -42,   -43,   -44,   -45,   -46,   -48,   -49,
   -50,   -51,   -52,   -53,   -54,   -57,   -55,   -56,   -47,   -28,
   -17,   -20,   268,   284,   286,   317,   288,   329,   337,   319,
   323,   272,   306,   -31,   -18,   -21,   303,   298,   347,   346,
   299,    -4,   300,   325,   285,   260,   349,    91,   305,    43,
    45,   318,   322,   312,   326,   294,   269,   271,    40,   345,
   -71,   301,   290,   290,   286,    -6,   -83,   347,   347,   -25,
   -26,   -21,   -17,   347,   -98,   -76,   302,  -100,   -71,   -71,
   314,   314,   -60,    40,   -71,    59,   272,   286,   288,   293,
   301,   306,   319,   323,   327,   329,   347,    59,    59,   295,
   331,   258,   296,   297,   261,   262,   332,   333,   334,   335,
    58,   -14,   347,   347,   347,   -17,   -21,   -28,    40,   -17,
   -17,   -28,    40,   -28,    40,   -58,   -69,   349,    44,   -15,
   350,    43,    45,    42,    47,    94,   259,   308,   -22,   -23,
   290,   124,   321,    61,    60,    62,   338,   339,   340,   265,
   266,   -21,   -21,    40,   348,   -25,   -21,   -21,   -21,    40,
    40,    91,    40,    91,    40,    91,    40,    91,    40,    91,
    40,    91,   -21,    -5,   348,    -6,  -106,   347,   347,   347,
    61,    93,    44,   341,    59,    -6,   347,    -6,    -6,    -6,
   347,   347,   -62,   330,   -58,    -6,   -13,   347,   -13,   -17,
    40,   347,   -13,   -21,   -21,   -20,   347,   329,    40,    40,
   290,   336,   324,   -67,   -68,   270,   309,   -28,   -63,   -64,
   270,   309,   -28,   -65,   -66,   270,   309,   -28,    -6,    -6,
   349,   -70,   -28,   -17,   -21,   -21,   -21,   -21,   -21,   -21,
   -21,   -21,   -21,   -21,   -21,   -21,   -21,    93,   -17,   -25,
   -25,   -25,   -25,   -25,   -25,   -25,   -25,   -25,   -25,   -25,
   -25,    41,    59,   -72,    59,    59,    59,   348,   -21,   -21,
    59,    59,    59,    59,    59,   -77,   279,   280,   -77,    59,
   314,    40,    41,    59,   -12,   307,    40,   -12,   -28,   -12,
    40,   -28,    40,   -25,   -28,   -21,   -17,   -58,    -6,   270,
   309,   -25,    58,    41,    -6,   270,   309,   -25,    58,    41,
    -6,   270,   309,   -25,    58,    41,   -70,   -28,   348,    41,
    44,    41,    41,    93,    41,    93,    41,    93,    41,    93,
    41,    93,    41,    93,    59,   341,   -58,    -2,   277,   -78,
    42,   -79,   347,   346,    40,  -103,   335,   347,   -58,   -24,
   344,   347,   -25,   -24,    41,   -29,   -28,   -17,   -67,   -28,
    41,    41,   -80,   292,   276,   -59,   282,   -25,    58,    58,
   -58,   -67,   -25,    58,    58,   -58,   -63,   -25,    58,    58,
   -58,   -65,   348,    -4,   346,   -21,    -6,    59,    -7,   285,
   325,   349,    43,    45,    47,    42,    94,   -79,   -15,    -7,
   325,   285,   349,   -61,    40,    41,   -21,    41,   330,    59,
    58,    59,    -6,    41,    -9,   274,   281,    -6,   -58,    58,
   -58,   -58,    -6,    58,   -58,   -58,    -6,    58,   -58,   -58,
    -6,    41,    59,    -3,   346,    -4,   -79,   -79,   -10,    -7,
    41,    59,    -3,    59,   -58,   -28,   -30,   -28,   291,   346,
   -67,   -58,   -58,   -58,   -58,   -11,   346,    40,    41,   295,
   -16,    59,    58,    41,    -6,    -6,   346,   347,    41,   -17,
   310,   -19,   290,    47,   307,    58,   -17,   346,   347,   275,
    41,   -27,   344,    40,   -25,    41 };
__ZZ_SCLASS zz_tabelem zz_def[]={

     1,    -2,     2,     3,     4,     5,     6,     7,     8,     9,
    10,    11,    12,    13,    14,    15,     0,     0,     0,    23,
    27,     0,     0,    73,    73,     0,     0,     0,    57,     0,
    62,     0,    18,   183,     0,    21,     0,    69,    69,    73,
    38,    73,     0,     0,     0,    73,     0,    -2,    65,    60,
     0,     0,     0,     0,    16,    17,     0,   201,    19,    20,
     0,    24,    68,    28,    65,    65,     0,     0,    48,    65,
     0,   197,     0,     0,    76,    77,    78,    79,    80,    81,
    82,    83,    84,    85,    86,    87,    88,    89,    90,    91,
    92,    93,    94,    95,    96,    97,    98,    99,   100,     0,
    -2,   123,     0,     0,     0,     0,     0,     0,     0,     0,
     0,    73,     0,   179,   198,   125,     0,     0,   183,   229,
   230,   231,   232,   233,   234,   235,   236,   201,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,   210,
     0,    66,     0,     0,     0,    61,    63,     0,   184,     0,
   200,   202,   237,     0,     0,    70,     0,     0,     0,     0,
     0,     0,    50,    73,     0,    58,   186,   187,   188,   189,
   190,   191,   192,   193,   194,   195,   196,    74,    75,     0,
     0,     0,     0,     0,   117,   118,   119,   120,     0,     0,
     0,   145,   109,     0,     0,   142,     0,     0,     0,   180,
   158,     0,     0,     0,     0,     0,     0,     0,     0,   182,
   199,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,   270,   271,   272,   273,   274,   275,   276,
   277,   126,   127,     0,   209,     0,   246,   252,   253,     0,
   201,   201,   201,   201,   201,   201,   201,   201,   201,   201,
   201,   201,     0,   208,   211,     0,    69,     0,     0,     0,
     0,   185,     0,     0,    22,     0,     0,     0,     0,     0,
   214,   214,     0,     0,     0,     0,   113,   111,   113,   103,
     0,   113,   108,   121,   122,   124,   183,     0,   201,     0,
     0,     0,    73,     0,   153,   201,     0,     0,     0,   161,
   201,     0,     0,     0,   168,   201,     0,     0,   173,   174,
     0,   175,     0,   181,   239,   240,   241,   242,   243,   244,
   245,   247,   248,   249,   250,   251,     0,   238,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,   269,    54,    67,    26,    30,    55,     0,   204,   203,
    25,    73,    29,    31,    45,    33,     0,   213,    40,    46,
     0,    73,    49,    59,   115,     0,   201,   115,     0,   107,
     0,     0,     0,     0,     0,   137,   143,   147,   149,   201,
     0,     0,    73,     0,   159,   201,     0,     0,    73,     0,
   166,   201,     0,     0,    73,     0,   176,     0,   177,   268,
     0,   255,   256,   257,   258,   259,   260,   261,   262,   263,
   264,   265,   266,   267,    64,     0,     0,     0,   226,   212,
   215,   216,   217,   218,     0,   198,   226,    52,     0,   101,
     0,   114,     0,   102,     0,     0,     0,    -2,     0,     0,
   110,   144,     0,   138,   139,     0,    73,     0,    73,    73,
   155,     0,     0,    73,    73,   163,     0,     0,    73,    73,
   170,     0,   178,     0,     0,   205,     0,    32,     0,    35,
    36,    37,   227,   228,     0,     0,   226,     0,     0,     0,
    42,    43,    44,     0,    73,    51,   116,   112,     0,     0,
     0,     0,   150,     0,    73,   140,   141,   146,   148,    73,
   157,   154,   151,    73,   165,   162,   160,    73,   172,   169,
   167,   254,    71,    34,   206,   207,   219,   220,   221,     0,
   222,    39,    41,    47,     0,     0,   131,     0,   129,     0,
     0,     0,   156,   164,   171,   223,   224,     0,    53,     0,
     0,     0,     0,   134,   152,   136,     0,     0,   128,     0,
   130,   133,     0,     0,     0,     0,   135,     0,   105,   132,
   225,   104,     0,   201,     0,   106 };
typedef struct { char *t_name; int t_val; } zz_toktype;
#ifndef ZZ_DEBUG
#	define ZZ_DEBUG	0	/* don't allow debugging */
#endif

#if ZZ_DEBUG

__ZZ_SCLASS zz_toktype zz_toks[] =
{
	"ADD_T",	257,
	"ALIASES_T",	258,
	"AND_T",	259,
	"ANY_T",	260,
	"AREALIKE_T",	261,
	"ARETHESAME_T",	262,
	"ARRAY_T",	263,
	"ATOM_T",	264,
	"BEQ_T",	265,
	"BNE_T",	266,
	"BREAK_T",	267,
	"CALL_T",	268,
	"CARD_T",	269,
	"CASE_T",	270,
	"CHOICE_T",	271,
	"CONDITIONAL_T",	272,
	"CONSTANT_T",	273,
	"CREATE_T",	274,
	"DATA_T",	275,
	"DECREASING_T",	276,
	"DEFAULT_T",	277,
	"DEFINITION_T",	278,
	"DIMENSION_T",	279,
	"DIMENSIONLESS_T",	280,
	"DO_T",	281,
	"ELSE_T",	282,
	"END_T",	283,
	"EXTERNAL_T",	284,
	"FALSE_T",	285,
	"FOR_T",	286,
	"FROM_T",	287,
	"IF_T",	288,
	"IMPORT_T",	289,
	"IN_T",	290,
	"INPUT_T",	291,
	"INCREASING_T",	292,
	"INTERACTIVE_T",	293,
	"INTERSECTION_T",	294,
	"ISA_T",	295,
	"_IS_T",	296,
	"ISREFINEDTO_T",	297,
	"MAXIMIZE_T",	298,
	"MAXINTEGER_T",	299,
	"MAXREAL_T",	300,
	"METHODS_T",	301,
	"METHOD_T",	302,
	"MINIMIZE_T",	303,
	"MODEL_T",	304,
	"NOT_T",	305,
	"NOTES_T",	306,
	"OF_T",	307,
	"OR_T",	308,
	"OTHERWISE_T",	309,
	"OUTPUT_T",	310,
	"PATCH_T",	311,
	"PROD_T",	312,
	"PROVIDE_T",	313,
	"REFINES_T",	314,
	"REPLACE_T",	315,
	"REQUIRE_T",	316,
	"RUN_T",	317,
	"SATISFIED_T",	318,
	"SELECT_T",	319,
	"SIZE_T",	320,
	"SUCHTHAT_T",	321,
	"SUM_T",	322,
	"SWITCH_T",	323,
	"THEN_T",	324,
	"TRUE_T",	325,
	"UNION_T",	326,
	"UNITS_T",	327,
	"UNIVERSAL_T",	328,
	"WHEN_T",	329,
	"WHERE_T",	330,
	"WILLBE_T",	331,
	"WILLBETHESAME_T",	332,
	"WILLNOTBETHESAME_T",	333,
	"ASSIGN_T",	334,
	"CASSIGN_T",	335,
	"DBLCOLON_T",	336,
	"USE_T",	337,
	"LEQ_T",	338,
	"GEQ_T",	339,
	"NEQ_T",	340,
	"DOTDOT_T",	341,
	"WITH_T",	342,
	"VALUE_T",	343,
	"WITH_VALUE_T",	344,
	"REAL_T",	345,
	"INTEGER_T",	346,
	"IDENTIFIER_T",	347,
	"BRACEDTEXT_T",	348,
	"SYMBOL_T",	349,
	"DQUOTE_T",	350,
	",",	44,
	"|",	124,
	"<",	60,
	"=",	61,
	">",	62,
	"+",	43,
	"-",	45,
	"/",	47,
	"*",	42,
	"UMINUS_T",	351,
	"UPLUS_T",	352,
	"^",	94,
	"-unknown-",	-1	/* ends search */
};

__ZZ_SCLASS char * zz_reds[] =
{
	"-no such reduction-",
	"definitions : /* empty */",
	"definitions : definitions definition",
	"definition : require_file",
	"definition : provide_module",
	"definition : import",
	"definition : add_method_def",
	"definition : replace_method_def",
	"definition : constant_def",
	"definition : atom_def",
	"definition : model_def",
	"definition : definition_def",
	"definition : interactive",
	"definition : patch_def",
	"definition : units_def",
	"definition : error",
	"require_file : REQUIRE_T DQUOTE_T ';'",
	"require_file : REQUIRE_T name ';'",
	"require_file : REQUIRE_T name",
	"provide_module : PROVIDE_T DQUOTE_T ';'",
	"provide_module : PROVIDE_T name ';'",
	"provide_module : PROVIDE_T name",
	"import : IMPORT_T IDENTIFIER_T FROM_T IDENTIFIER_T ';'",
	"add_method_def : add_method_head",
	"add_method_def : add_method_head proclist",
	"add_method_def : add_method_head proclist end ';'",
	"add_method_head : ADD_T METHODS_T IN_T IDENTIFIER_T ';'",
	"replace_method_def : replace_method_head",
	"replace_method_def : replace_method_head proclist",
	"replace_method_def : replace_method_head proclist end ';'",
	"replace_method_head : REPLACE_T METHODS_T IN_T IDENTIFIER_T ';'",
	"atom_def : universal atom_head fstatements methods end ';'",
	"atom_head : ATOM_T IDENTIFIER_T REFINES_T IDENTIFIER_T dims default_val ';'",
	"default_val : /* empty */",
	"default_val : DEFAULT_T optional_sign number",
	"default_val : DEFAULT_T FALSE_T",
	"default_val : DEFAULT_T TRUE_T",
	"default_val : DEFAULT_T SYMBOL_T",
	"constant_def : universal constant_head",
	"constant_head : CONSTANT_T IDENTIFIER_T REFINES_T IDENTIFIER_T dims constant_val optional_notes ';'",
	"constant_val : /* empty */",
	"constant_val : CASSIGN_T optional_sign number",
	"constant_val : CASSIGN_T TRUE_T",
	"constant_val : CASSIGN_T FALSE_T",
	"constant_val : CASSIGN_T SYMBOL_T",
	"model_def : universal model_head fstatements methods end ';'",
	"model_head : MODEL_T IDENTIFIER_T optional_model_parameters optional_parameter_wheres ';'",
	"model_head : MODEL_T IDENTIFIER_T optional_model_parameters optional_parameter_wheres REFINES_T IDENTIFIER_T optional_parameter_reduction ';'",
	"optional_model_parameters : /* empty */",
	"optional_model_parameters : '(' fstatements ')'",
	"optional_parameter_wheres : /* empty */",
	"optional_parameter_wheres : WHERE_T '(' fstatements ')'",
	"optional_parameter_reduction : /* empty */",
	"optional_parameter_reduction : '(' fstatements ')'",
	"patch_def : patch_head fstatements methods end ';'",
	"patch_head : PATCH_T IDENTIFIER_T FOR_T IDENTIFIER_T ';'",
	"universal : /* empty */",
	"universal : UNIVERSAL_T",
	"interactive : INTERACTIVE_T fstatements end ';'",
	"definition_def : DEFINITION_T IDENTIFIER_T fstatements methods end ';'",
	"units_def : units_statement ';'",
	"units_statement : UNITS_T unitdeflist end",
	"unitdeflist : /* empty */",
	"unitdeflist : unitdeflist unitdef",
	"unitdef : IDENTIFIER_T '=' BRACEDTEXT_T ';'",
	"methods : /* empty */",
	"methods : METHODS_T",
	"methods : METHODS_T proclist",
	"proclist : proclistf",
	"proclistf : /* empty */",
	"proclistf : proclistf procedure",
	"procedure : METHOD_T IDENTIFIER_T ';' fstatements end ';'",
	"fstatements : statements",
	"statements : /* empty */",
	"statements : statements statement ';'",
	"statements : statements error ';'",
	"statement : isa_statement",
	"statement : willbe_statement",
	"statement : aliases_statement",
	"statement : is_statement",
	"statement : isrefinedto_statement",
	"statement : arealike_statement",
	"statement : arethesame_statement",
	"statement : willbethesame_statement",
	"statement : willnotbethesame_statement",
	"statement : assignment_statement",
	"statement : relation_statement",
	"statement : glassbox_statement",
	"statement : blackbox_statement",
	"statement : call_statement",
	"statement : external_statement",
	"statement : for_statement",
	"statement : run_statement",
	"statement : if_statement",
	"statement : when_statement",
	"statement : use_statement",
	"statement : select_statement",
	"statement : switch_statement",
	"statement : conditional_statement",
	"statement : notes_statement",
	"statement : units_statement",
	"isa_statement : fvarlist ISA_T type_identifier optional_of optional_with_value",
	"willbe_statement : fvarlist WILLBE_T type_identifier optional_of optional_with_value",
	"aliases_statement : fvarlist ALIASES_T fname",
	"aliases_statement : fvarlist ALIASES_T '(' fvarlist ')' WHERE_T fvarlist ISA_T IDENTIFIER_T OF_T IDENTIFIER_T optional_set_values",
	"optional_set_values : /* empty */",
	"optional_set_values : WITH_VALUE_T '(' set ')'",
	"is_statement : fvarlist _IS_T IDENTIFIER_T optional_of",
	"isrefinedto_statement : fvarlist ISREFINEDTO_T type_identifier",
	"call_identifier : IDENTIFIER_T",
	"call_identifier : IDENTIFIER_T '(' set ')'",
	"type_identifier : IDENTIFIER_T",
	"type_identifier : IDENTIFIER_T '(' set ')'",
	"optional_of : /* empty */",
	"optional_of : OF_T IDENTIFIER_T",
	"optional_with_value : /* empty */",
	"optional_with_value : WITH_VALUE_T expr",
	"arealike_statement : fvarlist AREALIKE_T",
	"arethesame_statement : fvarlist ARETHESAME_T",
	"willbethesame_statement : fvarlist WILLBETHESAME_T",
	"willnotbethesame_statement : fvarlist WILLNOTBETHESAME_T",
	"assignment_statement : fname ASSIGN_T expr",
	"assignment_statement : fname CASSIGN_T expr",
	"relation_statement : relation",
	"relation_statement : fname ':' relation",
	"relation : expr",
	"relation : MINIMIZE_T expr",
	"relation : MAXIMIZE_T expr",
	"blackbox_statement : fname ':' IDENTIFIER_T '(' input_args ';' output_args data_args ')'",
	"input_args : fvarlist ':' INPUT_T",
	"output_args : fvarlist ':' OUTPUT_T",
	"data_args : /* empty */",
	"data_args : ';' fname ':' DATA_T",
	"glassbox_statement : fname ':' IDENTIFIER_T '(' fvarlist ';' INTEGER_T ')' optional_scope",
	"optional_scope : /* empty */",
	"optional_scope : IN_T fname",
	"for_statement : FOR_T IDENTIFIER_T IN_T expr optional_direction createordo fstatements end",
	"optional_direction : /* empty */",
	"optional_direction : INCREASING_T",
	"optional_direction : DECREASING_T",
	"createordo : CREATE_T",
	"createordo : DO_T",
	"run_statement : RUN_T fname",
	"run_statement : RUN_T fname DBLCOLON_T fname",
	"external_statement : EXTERNAL_T IDENTIFIER_T '(' fvarlist ')'",
	"call_statement : CALL_T call_identifier",
	"if_statement : IF_T expr THEN_T fstatements optional_else end",
	"optional_else : /* empty */",
	"optional_else : ELSE_T fstatements",
	"when_statement : WHEN_T fvarlist whenlist end",
	"when_statement : fname ':' WHEN_T fvarlist whenlist end",
	"when_statement : WHEN_T '(' fvarlist ')' whenlist end",
	"when_statement : fname ':' WHEN_T '(' fvarlist ')' whenlist end",
	"whenlist : whenlistf",
	"whenlistf : CASE_T set ':' fstatements",
	"whenlistf : OTHERWISE_T ':' fstatements",
	"whenlistf : whenlistf CASE_T set ':' fstatements",
	"whenlistf : whenlistf OTHERWISE_T ':' fstatements",
	"use_statement : USE_T fname",
	"select_statement : SELECT_T fvarlist selectlist end",
	"select_statement : SELECT_T '(' fvarlist ')' selectlist end",
	"selectlist : selectlistf",
	"selectlistf : CASE_T set ':' fstatements",
	"selectlistf : OTHERWISE_T ':' fstatements",
	"selectlistf : selectlistf CASE_T set ':' fstatements",
	"selectlistf : selectlistf OTHERWISE_T ':' fstatements",
	"switch_statement : SWITCH_T fvarlist switchlist end",
	"switch_statement : SWITCH_T '(' fvarlist ')' switchlist end",
	"switchlist : switchlistf",
	"switchlistf : CASE_T set ':' fstatements",
	"switchlistf : OTHERWISE_T ':' fstatements",
	"switchlistf : switchlistf CASE_T set ':' fstatements",
	"switchlistf : switchlistf OTHERWISE_T ':' fstatements",
	"conditional_statement : CONDITIONAL_T fstatements end",
	"notes_statement : NOTES_T notes_body end",
	"notes_body : SYMBOL_T noteslist",
	"notes_body : notes_body SYMBOL_T noteslist",
	"noteslist : fvarlist BRACEDTEXT_T",
	"noteslist : noteslist fvarlist BRACEDTEXT_T",
	"fvarlist : varlist",
	"varlist : fname",
	"varlist : varlist ',' fname",
	"fname : name optional_notes",
	"name : IDENTIFIER_T",
	"name : name '.' IDENTIFIER_T",
	"name : name '[' set ']'",
	"end : END_T CONDITIONAL_T",
	"end : END_T FOR_T",
	"end : END_T IF_T",
	"end : END_T INTERACTIVE_T",
	"end : END_T METHODS_T",
	"end : END_T NOTES_T",
	"end : END_T SELECT_T",
	"end : END_T SWITCH_T",
	"end : END_T UNITS_T",
	"end : END_T WHEN_T",
	"end : END_T IDENTIFIER_T",
	"end : END_T",
	"optional_notes : /* empty */",
	"optional_notes : DQUOTE_T",
	"set : setexprlist",
	"set : /* empty */",
	"setexprlist : expr",
	"setexprlist : expr DOTDOT_T expr",
	"setexprlist : setexprlist ',' expr",
	"setexprlist : setexprlist ',' expr DOTDOT_T expr",
	"number : INTEGER_T",
	"number : realnumber",
	"realnumber : REAL_T opunits",
	"realnumber : INTEGER_T BRACEDTEXT_T",
	"opunits : /* empty */",
	"opunits : BRACEDTEXT_T",
	"dims : DIMENSION_T dimensions",
	"dims : DIMENSIONLESS_T",
	"dims : /* empty */",
	"dimensions : '*'",
	"dimensions : dimexpr",
	"dimexpr : IDENTIFIER_T",
	"dimexpr : INTEGER_T",
	"dimexpr : dimexpr '/' dimexpr",
	"dimexpr : dimexpr '*' dimexpr",
	"dimexpr : dimexpr '^' fraction",
	"dimexpr : '(' dimexpr ')'",
	"fraction : optional_sign fractail",
	"fractail : INTEGER_T",
	"fractail : '(' INTEGER_T '/' INTEGER_T ')'",
	"optional_sign : /* empty */",
	"optional_sign : '+'",
	"optional_sign : '-'",
	"expr : INTEGER_T",
	"expr : MAXINTEGER_T",
	"expr : realnumber",
	"expr : MAXREAL_T",
	"expr : TRUE_T",
	"expr : FALSE_T",
	"expr : ANY_T",
	"expr : SYMBOL_T",
	"expr : fname",
	"expr : '[' set ']'",
	"expr : expr '+' expr",
	"expr : expr '-' expr",
	"expr : expr '*' expr",
	"expr : expr '/' expr",
	"expr : expr '^' expr",
	"expr : expr AND_T expr",
	"expr : expr OR_T expr",
	"expr : NOT_T expr",
	"expr : expr relop expr",
	"expr : expr logrelop expr",
	"expr : expr IN_T expr",
	"expr : expr '|' expr",
	"expr : expr SUCHTHAT_T expr",
	"expr : '+' expr",
	"expr : '-' expr",
	"expr : SATISFIED_T '(' fname ',' realnumber ')'",
	"expr : SATISFIED_T '(' fname ')'",
	"expr : SUM_T '(' set ')'",
	"expr : SUM_T '[' set ']'",
	"expr : PROD_T '(' set ')'",
	"expr : PROD_T '[' set ']'",
	"expr : UNION_T '(' set ')'",
	"expr : UNION_T '[' set ']'",
	"expr : INTERSECTION_T '(' set ')'",
	"expr : INTERSECTION_T '[' set ']'",
	"expr : CARD_T '(' set ')'",
	"expr : CARD_T '[' set ']'",
	"expr : CHOICE_T '(' set ')'",
	"expr : CHOICE_T '[' set ']'",
	"expr : IDENTIFIER_T '(' expr ')'",
	"expr : '(' expr ')'",
	"relop : '='",
	"relop : '<'",
	"relop : '>'",
	"relop : LEQ_T",
	"relop : GEQ_T",
	"relop : NEQ_T",
	"logrelop : BEQ_T",
	"logrelop : BNE_T",
};
#endif /* ZZ_DEBUG */
#define ZZ_FLAG  (-3000)
/* @(#) $Revision: 1.1 $ */    

/*
** Skeleton parser driver for yacc output
*/

#if defined(NLS) && !defined(NL_SETN)
#include <msgbuf.h>
#endif

#ifndef nl_msg
#define nl_msg(i,s) (s)
#endif

/*
** yacc user known macros and defines
*/
#define ZZ_ERROR		goto zz_errlab

#ifndef __RUNTIME_ZZ_MAXDEPTH
#define ZZ_ACCEPT	return(0)
#define ZZ_ABORT		return(1)
#else
#define ZZ_ACCEPT	{free_stacks(); return(0);}
#define ZZ_ABORT		{free_stacks(); return(1);}
#endif

#define ZZ_BACKUP( newtoken, newvalue )\
{\
	if ( zz_char >= 0 || ( zz_r2[ zz_tmp ] >> 1 ) != 1 )\
	{\
		zz_error( (nl_msg(30001,"syntax error - cannot backup")) );\
		goto zz_errlab;\
	}\
	zz_char = newtoken;\
	zz_state = *zz_ps;\
	zz_lval = newvalue;\
	goto zz_newstate;\
}
#define ZZ_RECOVERING()	(!!zz_errflag)
#ifndef ZZ_DEBUG
#	define ZZ_DEBUG	1	/* make debugging available */
#endif

/*
** user known globals
*/
int zz_debug;			/* set to 1 to get debugging */

/*
** driver internal defines
*/
/* define for ZZ_FLAG now generated by yacc program. */
/*#define ZZ_FLAG		(FLAGVAL)*/

/*
** global variables used by the parser
*/
# ifndef __RUNTIME_ZZ_MAXDEPTH
__ZZ_SCLASS ZZ_STYPE zz_v[ ZZ_MAXDEPTH ];	/* value stack */
__ZZ_SCLASS int zz_s[ ZZ_MAXDEPTH ];		/* state stack */
# else
__ZZ_SCLASS ZZ_STYPE *zz_v;			/* pointer to malloc'ed value stack */
__ZZ_SCLASS int *zz_s;			/* pointer to malloc'ed stack stack */

#if defined(__STDC__) || defined (__cplusplus)
#include <stdlib.h>
#else
	extern char *malloc();
	extern char *realloc();
	extern void free();
#endif /* __STDC__ or __cplusplus */


static int allocate_stacks(); 
static void free_stacks();
# ifndef ZZ_INCREMENT
# define ZZ_INCREMENT (ZZ_MAXDEPTH/2) + 10
# endif
# endif	/* __RUNTIME_ZZ_MAXDEPTH */
long  zz_maxdepth = ZZ_MAXDEPTH;

__ZZ_SCLASS ZZ_STYPE *zz_pv;			/* top of value stack */
__ZZ_SCLASS int *zz_ps;			/* top of state stack */

__ZZ_SCLASS int zz_state;			/* current state */
__ZZ_SCLASS int zz_tmp;			/* extra var (lasts between blocks) */

int zz_nerrs;			/* number of errors */
__ZZ_SCLASS int zz_errflag;			/* error recovery flag */
int zz_char;			/* current input token number */



/*
** zz_parse - return 0 if worked, 1 if syntax error not recovered from
*/
int
zz_parse()
{
	register ZZ_STYPE *zz_pvt;	/* top of value stack for $vars */

	/*
	** Initialize externals - zz_parse may be called more than once
	*/
# ifdef __RUNTIME_ZZ_MAXDEPTH
	if (allocate_stacks()) ZZ_ABORT;
# endif
	zz_pv = &zz_v[-1];
	zz_ps = &zz_s[-1];
	zz_state = 0;
	zz_tmp = 0;
	zz_nerrs = 0;
	zz_errflag = 0;
	zz_char = -1;

	goto zz_stack;
	{
		register ZZ_STYPE *zz__pv;	/* top of value stack */
		register int *zz__ps;		/* top of state stack */
		register int zz__state;		/* current state */
		register int  zz__n;		/* internal state number info */

		/*
		** get globals into registers.
		** branch to here only if ZZ_BACKUP was called.
		*/
	zz_newstate:
		zz__pv = zz_pv;
		zz__ps = zz_ps;
		zz__state = zz_state;
		goto zz__newstate;

		/*
		** get globals into registers.
		** either we just started, or we just finished a reduction
		*/
	zz_stack:
		zz__pv = zz_pv;
		zz__ps = zz_ps;
		zz__state = zz_state;

		/*
		** top of for (;;) loop while no reductions done
		*/
	zz__stack:
		/*
		** put a state and value onto the stacks
		*/
#if ZZ_DEBUG
		/*
		** if debugging, look up token value in list of value vs.
		** name pairs.  0 and negative (-1) are special values.
		** Note: linear search is used since time is not a real
		** consideration while debugging.
		*/
		if ( zz_debug )
		{
			register int zz__i;

			printf( "State %d, token ", zz__state );
			if ( zz_char == 0 )
				printf( "end-of-file\n" );
			else if ( zz_char < 0 )
				printf( "-none-\n" );
			else
			{
				for ( zz__i = 0; zz_toks[zz__i].t_val >= 0;
					zz__i++ )
				{
					if ( zz_toks[zz__i].t_val == zz_char )
						break;
				}
				printf( "%s\n", zz_toks[zz__i].t_name );
			}
		}
#endif /* ZZ_DEBUG */
		if ( ++zz__ps >= &zz_s[ zz_maxdepth ] )	/* room on stack? */
		{
# ifndef __RUNTIME_ZZ_MAXDEPTH
			zz_error( (nl_msg(30002,"yacc stack overflow")) );
			ZZ_ABORT;
# else
			/* save old stack bases to recalculate pointers */
			ZZ_STYPE * zz_v_old = zz_v;
			int * zz_s_old = zz_s;
			zz_maxdepth += ZZ_INCREMENT;
			zz_s = (int *) realloc(zz_s, zz_maxdepth * sizeof(int));
			zz_v = (ZZ_STYPE *) realloc(zz_v, zz_maxdepth * sizeof(ZZ_STYPE));
			if (zz_s==0 || zz_v==0) {
			    zz_error( (nl_msg(30002,"yacc stack overflow")) );
			    ZZ_ABORT;
			    }
			/* Reset pointers into stack */
			zz__ps = (zz__ps - zz_s_old) + zz_s;
			zz_ps = (zz_ps - zz_s_old) + zz_s;
			zz__pv = (zz__pv - zz_v_old) + zz_v;
			zz_pv = (zz_pv - zz_v_old) + zz_v;
# endif

		}
		*zz__ps = zz__state;
		*++zz__pv = zz_val;

		/*
		** we have a new state - find out what to do
		*/
	zz__newstate:
		if ( ( zz__n = zz_pact[ zz__state ] ) <= ZZ_FLAG )
			goto zz_default;		/* simple state */
#if ZZ_DEBUG
		/*
		** if debugging, need to mark whether new token grabbed
		*/
		zz_tmp = zz_char < 0;
#endif
		if ( ( zz_char < 0 ) && ( ( zz_char = zz_lex() ) < 0 ) )
			zz_char = 0;		/* reached EOF */
#if ZZ_DEBUG
		if ( zz_debug && zz_tmp )
		{
			register int zz__i;

			printf( "Received token " );
			if ( zz_char == 0 )
				printf( "end-of-file\n" );
			else if ( zz_char < 0 )
				printf( "-none-\n" );
			else
			{
				for ( zz__i = 0; zz_toks[zz__i].t_val >= 0;
					zz__i++ )
				{
					if ( zz_toks[zz__i].t_val == zz_char )
						break;
				}
				printf( "%s\n", zz_toks[zz__i].t_name );
			}
		}
#endif /* ZZ_DEBUG */
		if ( ( ( zz__n += zz_char ) < 0 ) || ( zz__n >= ZZ_LAST ) )
			goto zz_default;
		if ( zz_chk[ zz__n = zz_act[ zz__n ] ] == zz_char )	/*valid shift*/
		{
			zz_char = -1;
			zz_val = zz_lval;
			zz__state = zz__n;
			if ( zz_errflag > 0 )
				zz_errflag--;
			goto zz__stack;
		}

	zz_default:
		if ( ( zz__n = zz_def[ zz__state ] ) == -2 )
		{
#if ZZ_DEBUG
			zz_tmp = zz_char < 0;
#endif
			if ( ( zz_char < 0 ) && ( ( zz_char = zz_lex() ) < 0 ) )
				zz_char = 0;		/* reached EOF */
#if ZZ_DEBUG
			if ( zz_debug && zz_tmp )
			{
				register int zz__i;

				printf( "Received token " );
				if ( zz_char == 0 )
					printf( "end-of-file\n" );
				else if ( zz_char < 0 )
					printf( "-none-\n" );
				else
				{
					for ( zz__i = 0;
						zz_toks[zz__i].t_val >= 0;
						zz__i++ )
					{
						if ( zz_toks[zz__i].t_val
							== zz_char )
						{
							break;
						}
					}
					printf( "%s\n", zz_toks[zz__i].t_name );
				}
			}
#endif /* ZZ_DEBUG */
			/*
			** look through exception table
			*/
			{
				register int *zz_xi = zz_exca;

				while ( ( *zz_xi != -1 ) ||
					( zz_xi[1] != zz__state ) )
				{
					zz_xi += 2;
				}
				while ( ( *(zz_xi += 2) >= 0 ) &&
					( *zz_xi != zz_char ) )
					;
				if ( ( zz__n = zz_xi[1] ) < 0 )
					ZZ_ACCEPT;
			}
		}

		/*
		** check for syntax error
		*/
		if ( zz__n == 0 )	/* have an error */
		{
			/* no worry about speed here! */
			switch ( zz_errflag )
			{
			case 0:		/* new error */
				zz_error( (nl_msg(30003,"syntax error")) );
				zz_nerrs++;
				goto skip_init;
			zz_errlab:
				/*
				** get globals into registers.
				** we have a user generated syntax type error
				*/
				zz__pv = zz_pv;
				zz__ps = zz_ps;
				zz__state = zz_state;
				zz_nerrs++;
			skip_init:
			case 1:
			case 2:		/* incompletely recovered error */
					/* try again... */
				zz_errflag = 3;
				/*
				** find state where "error" is a legal
				** shift action
				*/
				while ( zz__ps >= zz_s )
				{
					zz__n = zz_pact[ *zz__ps ] + ZZ_ERRCODE;
					if ( zz__n >= 0 && zz__n < ZZ_LAST &&
						zz_chk[zz_act[zz__n]] == ZZ_ERRCODE)					{
						/*
						** simulate shift of "error"
						*/
						zz__state = zz_act[ zz__n ];
						goto zz__stack;
					}
					/*
					** current state has no shift on
					** "error", pop stack
					*/
#if ZZ_DEBUG
#	define _POP_ "Error recovery pops state %d, uncovers state %d\n"
					if ( zz_debug )
						printf( _POP_, *zz__ps,
							zz__ps[-1] );
#	undef _POP_
#endif
					zz__ps--;
					zz__pv--;
				}
				/*
				** there is no state on stack with "error" as
				** a valid shift.  give up.
				*/
				ZZ_ABORT;
			case 3:		/* no shift yet; eat a token */
#if ZZ_DEBUG
				/*
				** if debugging, look up token in list of
				** pairs.  0 and negative shouldn't occur,
				** but since timing doesn't matter when
				** debugging, it doesn't hurt to leave the
				** tests here.
				*/
				if ( zz_debug )
				{
					register int zz__i;

					printf( "Error recovery discards " );
					if ( zz_char == 0 )
						printf( "token end-of-file\n" );
					else if ( zz_char < 0 )
						printf( "token -none-\n" );
					else
					{
						for ( zz__i = 0;
							zz_toks[zz__i].t_val >= 0;
							zz__i++ )
						{
							if ( zz_toks[zz__i].t_val
								== zz_char )
							{
								break;
							}
						}
						printf( "token %s\n",
							zz_toks[zz__i].t_name );
					}
				}
#endif /* ZZ_DEBUG */
				if ( zz_char == 0 )	/* reached EOF. quit */
					ZZ_ABORT;
				zz_char = -1;
				goto zz__newstate;
			}
		}/* end if ( zz__n == 0 ) */
		/*
		** reduction by production zz__n
		** put stack tops, etc. so things right after switch
		*/
#if ZZ_DEBUG
		/*
		** if debugging, print the string that is the user's
		** specification of the reduction which is just about
		** to be done.
		*/
		if ( zz_debug )
			printf( "Reduce by (%d) \"%s\"\n",
				zz__n, zz_reds[ zz__n ] );
#endif
		zz_tmp = zz__n;			/* value to switch over */
		zz_pvt = zz__pv;			/* $vars top of value stack */
		/*
		** Look in goto table for next state
		** Sorry about using zz__state here as temporary
		** register variable, but why not, if it works...
		** If zz_r2[ zz__n ] doesn't have the low order bit
		** set, then there is no action to be done for
		** this reduction.  So, no saving & unsaving of
		** registers done.  The only difference between the
		** code just after the if and the body of the if is
		** the goto zz__stack in the body.  This way the test
		** can be made before the choice of what to do is needed.
		*/
		{
			/* length of production doubled with extra bit */
			register int zz__len = zz_r2[ zz__n ];

			if ( !( zz__len & 01 ) )
			{
				zz__len >>= 1;
				zz_val = ( zz__pv -= zz__len )[1];	/* $$ = $1 */
				zz__state = zz_pgo[ zz__n = zz_r1[ zz__n ] ] +
					*( zz__ps -= zz__len ) + 1;
				if ( zz__state >= ZZ_LAST ||
					zz_chk[ zz__state =
					zz_act[ zz__state ] ] != -zz__n )
				{
					zz__state = zz_act[ zz_pgo[ zz__n ] ];
				}
				goto zz__stack;
			}
			zz__len >>= 1;
			zz_val = ( zz__pv -= zz__len )[1];	/* $$ = $1 */
			zz__state = zz_pgo[ zz__n = zz_r1[ zz__n ] ] +
				*( zz__ps -= zz__len ) + 1;
			if ( zz__state >= ZZ_LAST ||
				zz_chk[ zz__state = zz_act[ zz__state ] ] != -zz__n )
			{
				zz__state = zz_act[ zz_pgo[ zz__n ] ];
			}
		}
					/* save until reenter driver code */
		zz_state = zz__state;
		zz_ps = zz__ps;
		zz_pv = zz__pv;
	}
	/*
	** code supplied by user is placed in this switch
	*/
	switch( zz_tmp )
	{
		
case 15:
# line 379 "./ascParse.y"
{
	  ErrMsg_Generic("Error in definition ");
	} break;
case 16:
# line 386 "./ascParse.y"
{
	  Asc_ScannerPushBuffer(zz_pvt[-1].dquote_ptr);
	} break;
case 17:
# line 390 "./ascParse.y"
{
	  DestroyName(zz_pvt[-1].nptr);
	  ErrMsg_Generic("Asc-Error: REQUIRE statement syntax is "
	                 "``REQUIRE \"filename\";'' ");
	} break;
case 18:
# line 396 "./ascParse.y"
{
	  DestroyName(zz_pvt[-0].nptr);
	  ErrMsg_Generic("Asc-Error: REQUIRE statement syntax is "
	                 "``REQUIRE \"filename\";'' ");
	} break;
case 19:
# line 405 "./ascParse.y"
{
          Asc_ModuleCreateAlias(Asc_CurrentModule(),zz_pvt[-1].dquote_ptr);
        } break;
case 20:
# line 409 "./ascParse.y"
{
	  DestroyName(zz_pvt[-1].nptr);
	  ErrMsg_Generic("Asc-Error: PROVIDE statement syntax is "
	                 "``PROVIDE \"filename\";'' ");
	} break;
case 21:
# line 415 "./ascParse.y"
{
	  DestroyName(zz_pvt[-0].nptr);
	  ErrMsg_Generic("Asc-Error: PROVIDE statement syntax is "
	                 "``PROVIDE \"filename\";'' ");
	} break;
case 22:
# line 424 "./ascParse.y"
{
	  CONST char *path;
	  path = MakeArchiveLibraryName(zz_pvt[-1].id_ptr);
	  (void)LoadArchiveLibrary(path,zz_pvt[-3].id_ptr);
	} break;
case 23:
# line 433 "./ascParse.y"
{
	} break;
case 24:
# line 436 "./ascParse.y"
{
	} break;
case 25:
# line 439 "./ascParse.y"
{
	  if (zz_pvt[-5].tptr == NULL) {
	    DestroyProcedureList(zz_pvt[-3].listp);
	  } else {
	    if( zz_pvt[-1].int_value != METHODS_T ) {
	      WarnMsg_MismatchEnd("ADD METHODS", NULL, zz_pvt[-1].int_value, "METHODS");
	    }
	    if (AddMethods(zz_pvt[-5].tptr,zz_pvt[-3].listp) != 0) {
	      FPRINTF(ASCERR, "ADD METHODS failed in (%s)\n",GetName(zz_pvt[-5].tptr));
	      DestroyProcedureList(zz_pvt[-3].listp);
	    }
	  }
	} break;
case 26:
# line 456 "./ascParse.y"
{
	  struct TypeDescription *tmptype;
	  tmptype = FindType(zz_pvt[-1].id_ptr);
	  if (tmptype == NULL) {
	    FPRINTF(ASCERR,"ADD METHODS called with undefined type (%s)\n",zz_pvt[-1].id_ptr);
	  }
	  zz_val.tptr = tmptype; /* parent should check for NULL */
	} break;
case 27:
# line 468 "./ascParse.y"
{
	} break;
case 28:
# line 471 "./ascParse.y"
{
	} break;
case 29:
# line 474 "./ascParse.y"
{
	  if (zz_pvt[-5].tptr == NULL) {
	    DestroyProcedureList(zz_pvt[-3].listp);
	  } else {
	    if( zz_pvt[-1].int_value != METHODS_T ) {
	      WarnMsg_MismatchEnd("REPLACE METHODS", NULL, zz_pvt[-1].int_value, "METHODS");
	    }
	    if (ReplaceMethods(zz_pvt[-5].tptr,zz_pvt[-3].listp) != 0) {
	      FPRINTF(ASCERR,"REPLACE METHODS failed in (%s)\n",GetName(zz_pvt[-5].tptr));
	      DestroyProcedureList(zz_pvt[-3].listp);
	    }
	  }
	} break;
case 30:
# line 491 "./ascParse.y"
{
	  struct TypeDescription *tmptype;
	  tmptype = FindType(zz_pvt[-1].id_ptr);
	  if (tmptype == NULL) {
	    FPRINTF(ASCERR,
	            "REPLACE METHODS called with undefined type (%s)\n",zz_pvt[-1].id_ptr);
	  }
	  zz_val.tptr = tmptype; /* parent should check for NULL */
	} break;
case 31:
# line 504 "./ascParse.y"
{
	  struct TypeDescription *def_ptr;

          if(( zz_pvt[-1].int_value != IDENTIFIER_T ) || ( g_end_identifier != g_type_name )) {
	    /* all identifier_t are from symbol table, so ptr match
	     * is sufficient for equality.
	     */
	    WarnMsg_MismatchEnd("ATOM", g_type_name, zz_pvt[-1].int_value, g_type_name);
	    /* assuming char *-IDENTIFIER_T equivalence here */
	  }
	  g_atom_dim_ptr = CheckDimensionsMatch(g_default_dim_ptr,
	                                        g_atom_dim_ptr);
	  if (g_atom_dim_ptr != NULL) {
	    def_ptr = CreateAtomTypeDef(g_type_name,
	                                g_refines_name,
	                                real_type, /* ignored..really */
	                                Asc_CurrentModule(),
	                                zz_pvt[-5].int_value,
	                                zz_pvt[-3].slptr,
	                                zz_pvt[-2].listp,
	                                g_defaulted,
	                                g_default_double,
	                                g_atom_dim_ptr,
	                                g_default_long,
	                                g_default_symbol,
	                                g_untrapped_error);
	    if (def_ptr != NULL) {
	      AddType(def_ptr);
	    } else {
	      /* CreateAtomTypeDef is responsible for freeing (if needed)
	       * all args sent to it event of failure so we don't have to.
	       * In particular $3 $4 should be killed before returning NULL.
	       */
	      ErrMsg_NullDefPointer(g_type_name);
	    }
	  } else {
	    FPRINTF(ASCERR,
	            "Atom dimensions don't match in ATOM %s on line %s:%lu.\n",
	            g_type_name,
	            Asc_ModuleFileName(Asc_CurrentModule()),
	            g_header_linenum);
	    DestroyStatementList(zz_pvt[-3].slptr);
	    DestroyProcedureList(zz_pvt[-2].listp);
	  }
	  g_type_name = g_refines_name = NULL;
	  g_untrapped_error = 0;
	} break;
case 32:
# line 555 "./ascParse.y"
{
	  g_type_name = zz_pvt[-5].id_ptr;
	  g_refines_name = zz_pvt[-3].id_ptr;
	  g_atom_dim_ptr = zz_pvt[-2].dimp;
	  g_default_double = zz_pvt[-1].real_value;
	  g_header_linenum = LineNum();
	} break;
case 33:
# line 566 "./ascParse.y"
{
	  zz_val.real_value = 0.0;
	  g_default_dim_ptr = WildDimension();
	  g_defaulted = 0;
	} break;
case 34:
# line 572 "./ascParse.y"
{
	  zz_val.real_value = zz_pvt[-1].int_value ? -zz_pvt[-0].real_value : zz_pvt[-0].real_value;
	  g_defaulted = 1;
	} break;
case 35:
# line 577 "./ascParse.y"
{
	  zz_val.real_value = 0.0;
	  g_default_dim_ptr = Dimensionless();
	  g_default_long = 0;
	  g_defaulted = 1;
	} break;
case 36:
# line 584 "./ascParse.y"
{
	  zz_val.real_value = 0.0;
	  g_default_dim_ptr = Dimensionless();
	  g_default_long = 1;
	  g_defaulted = 1;
	} break;
case 37:
# line 591 "./ascParse.y"
{
	  zz_val.real_value = 0.0;
	  g_default_dim_ptr = Dimensionless();
	  g_default_symbol = zz_pvt[-0].sym_ptr;
	  g_defaulted = 0;
	} break;
case 38:
# line 601 "./ascParse.y"
{
	  struct TypeDescription *def_ptr;
	  if (g_defaulted) {
	    g_atom_dim_ptr = CheckDimensionsMatch(g_default_dim_ptr,
	                                          g_atom_dim_ptr);
	  }
	  if (g_atom_dim_ptr != NULL) {
	    def_ptr = CreateConstantTypeDef(g_type_name,
	                                    g_refines_name,
	                                    Asc_CurrentModule(),
	                                    zz_pvt[-1].int_value,
	                                    g_defaulted,
	                                    g_default_double,
	                                    g_default_long,
	                                    g_default_symbol,
	                                    g_atom_dim_ptr,
	                                    g_untrapped_error);
	    if (def_ptr != NULL) {
	      AddType(def_ptr);
	    } else {
	      ErrMsg_NullDefPointer(g_type_name);
	    }
	  } else {
	    FPRINTF(ASCERR,
	            "Constant dimensions don't match in CONSTANT %s"
	            " on line %s:%lu.\n",
	            g_type_name,
	            Asc_ModuleFileName(Asc_CurrentModule()),
	            g_header_linenum);
	  }
	  g_type_name = g_refines_name = NULL;
	  g_untrapped_error = 0;
	} break;
case 39:
# line 639 "./ascParse.y"
{
	  g_type_name = zz_pvt[-6].id_ptr;
	  g_refines_name = zz_pvt[-4].id_ptr;
	  g_atom_dim_ptr = zz_pvt[-3].dimp;
	  switch (g_constant_type) {
	  case DOUBLECONSTANT:
	    g_default_double = zz_pvt[-2].real_value;
	    break;
	  case LONGCONSTANT:
	    g_default_long = zz_pvt[-2].real_value;
	    break;
	  case BOOLEANCONSTANT:
	    g_default_long = zz_pvt[-2].int_value;
	    break;
	  case SYMBOLCONSTANT:
	    g_default_symbol = zz_pvt[-2].sym_ptr;
	    break;
	  default:
	    ErrMsg_Generic("Error: wierd constant type assign encountered ");
	    break; /* better not be reached. */
	  }
	  g_type_notes = zz_pvt[-1].dquote_ptr;
	  g_header_linenum = LineNum();
	} break;
case 40:
# line 667 "./ascParse.y"
{
	  zz_val.real_value = 0.0;
	  g_default_dim_ptr = WildDimension();
	  g_defaulted = 0;
	} break;
case 41:
# line 673 "./ascParse.y"
{
	  zz_val.real_value = zz_pvt[-1].int_value ? -zz_pvt[-0].real_value : zz_pvt[-0].real_value;
	  g_defaulted = 1;
	} break;
case 42:
# line 678 "./ascParse.y"
{
	  zz_val.int_value = 1;
	  g_defaulted = 1;
	  g_default_dim_ptr = Dimensionless();
	  g_constant_type = BOOLEANCONSTANT;
	} break;
case 43:
# line 685 "./ascParse.y"
{
	  zz_val.int_value = 0;
	  g_defaulted = 1;
	  g_default_dim_ptr = Dimensionless();
	  g_constant_type = BOOLEANCONSTANT;
	} break;
case 44:
# line 692 "./ascParse.y"
{
	  zz_val.sym_ptr = zz_pvt[-0].sym_ptr;
	  g_defaulted = 1;
	  g_default_dim_ptr = Dimensionless();
	  g_constant_type = SYMBOLCONSTANT;
	} break;
case 45:
# line 702 "./ascParse.y"
{
	  struct TypeDescription *def_ptr;
	  if(( zz_pvt[-1].int_value != IDENTIFIER_T ) || ( g_end_identifier != g_type_name )) {
	    /* all identifier_t are from symbol table, so ptr match
	     * is sufficient for equality.
	     */
	    WarnMsg_MismatchEnd("MODEL", g_type_name, zz_pvt[-1].int_value, g_type_name);
	    /* assuming char *-IDENTIFIER_T equivalence here */
	  }
	  def_ptr = CreateModelTypeDef(g_type_name,
	                               g_refines_name,
	                               Asc_CurrentModule(),
	                               zz_pvt[-5].int_value,
	                               zz_pvt[-3].slptr,
	                               zz_pvt[-2].listp,
	                               g_model_parameters,
	                               g_parameter_reduction,
	                               g_parameter_wheres,
	                               g_untrapped_error);
	  if (def_ptr != NULL) {
	    AddType(def_ptr);
	  } else {
	    /* CreateModelTypeDef is responsible for freeing (if needed)
	     * all args sent to it so we don't have to here.
	     * in particular $3 $4 g_model_parameters, g_parameter_reduction,
	     * and g_parameter_wheres.
	     */
	    ErrMsg_NullDefPointer(g_type_name);
	  }
	  g_type_name = g_refines_name = NULL;
	  g_model_parameters =
	    g_parameter_reduction =
	    g_parameter_wheres = NULL;
	  g_untrapped_error = 0;
	} break;
case 46:
# line 742 "./ascParse.y"
{
	  g_type_name = zz_pvt[-3].id_ptr;
	  g_model_parameters = zz_pvt[-2].slptr;
	  g_parameter_wheres = zz_pvt[-1].slptr;
	  g_refines_name = NULL;
	  g_header_linenum = LineNum();
	} break;
case 47:
# line 751 "./ascParse.y"
{
	  g_type_name = zz_pvt[-6].id_ptr;
	  g_model_parameters = zz_pvt[-5].slptr;
	  g_parameter_wheres = zz_pvt[-4].slptr;
	  g_refines_name = zz_pvt[-2].id_ptr;
	  g_parameter_reduction = zz_pvt[-1].slptr;
	  g_header_linenum = LineNum();
	} break;
case 48:
# line 763 "./ascParse.y"
{
	  zz_val.slptr = NULL;
	} break;
case 49:
# line 767 "./ascParse.y"
{
	  zz_val.slptr = zz_pvt[-1].slptr; /* this could be much more sophisticated */
	} break;
case 50:
# line 774 "./ascParse.y"
{
	  zz_val.slptr = NULL;
	} break;
case 51:
# line 778 "./ascParse.y"
{
	  zz_val.slptr = zz_pvt[-1].slptr; /* this could be much more sophisticated */
	} break;
case 52:
# line 785 "./ascParse.y"
{
	  zz_val.slptr = NULL;
	} break;
case 53:
# line 789 "./ascParse.y"
{
	  zz_val.slptr = zz_pvt[-1].slptr; /* this could be much more sophisticated */
	} break;
case 54:
# line 796 "./ascParse.y"
{
	  struct TypeDescription *def_ptr;
	  if ((zz_pvt[-1].int_value != IDENTIFIER_T ) || ( g_end_identifier != g_type_name )) {
	    /* all identifier_t are from symbol table, so ptr match
	     * is sufficient for equality.
	     */
	    WarnMsg_MismatchEnd("PATCH", g_type_name, zz_pvt[-1].int_value, g_type_name);
	    /* assuming char *-IDENTIFIER_T equivalence here */
	  }
	  def_ptr = CreatePatchTypeDef(g_type_name,
	                               g_refines_name,
	                               NULL,
	                               Asc_CurrentModule(),
	                               zz_pvt[-3].slptr,
	                               zz_pvt[-2].listp,
	                               g_untrapped_error);
	  g_untrapped_error = 0;
	  if (def_ptr != NULL) {
	    AddType(def_ptr);
	  } else {
	    /* CreatePatchTypeDef is responsible for freeing (if needed)
	     * all args sent to it so we don't have to here.
	     * in particular $2 $3
	     */
	    ErrMsg_NullDefPointer(g_type_name);
	  }
	  g_type_name = g_refines_name = NULL;
	} break;
case 55:
# line 828 "./ascParse.y"
{
	  /*
	   * A patch definition looks just like a model def.
	   * with the original name <=> refine name.
	   */
	  g_type_name = zz_pvt[-3].id_ptr;
	  g_refines_name = zz_pvt[-1].id_ptr;
	  g_header_linenum = LineNum();
	} break;
case 56:
# line 841 "./ascParse.y"
{
	  zz_val.int_value = 0;
	} break;
case 57:
# line 845 "./ascParse.y"
{
	  zz_val.int_value = 1;
	} break;
case 58:
# line 852 "./ascParse.y"
{
	  if( zz_pvt[-1].int_value != INTERACTIVE_T ) {
	    WarnMsg_MismatchEnd("INTERACTIVE", NULL, zz_pvt[-1].int_value, NULL);
	  }
	  g_statementlist_ptr = zz_pvt[-2].slptr;
	} break;
case 59:
# line 862 "./ascParse.y"
{
	  struct TypeDescription *def_ptr;

	  if(( zz_pvt[-1].int_value != IDENTIFIER_T ) || ( g_end_identifier != zz_pvt[-4].id_ptr )) {
	    WarnMsg_MismatchEnd("DEFINITION", zz_pvt[-4].id_ptr, zz_pvt[-1].int_value, zz_pvt[-4].id_ptr);
	  }
	  if( strcmp(zz_pvt[-4].id_ptr,BASE_REL_NAME) == 0 ) {
	    def_ptr = CreateRelationTypeDef(Asc_CurrentModule(),zz_pvt[-4].id_ptr,zz_pvt[-3].slptr,zz_pvt[-2].listp);
	  }
	  else if( strcmp(zz_pvt[-4].id_ptr,BASE_LOGREL_NAME) == 0 ) {
	    def_ptr = CreateLogRelTypeDef(Asc_CurrentModule(),zz_pvt[-4].id_ptr,zz_pvt[-3].slptr,zz_pvt[-2].listp);
	  }
	  else {
	    ErrMsg_Generic("Bad type passed to DEFINITION statement ");
	    def_ptr = NULL;
	  }
	  if ( def_ptr != NULL ) {
	    AddType(def_ptr);
	  } else {
	    ErrMsg_NullDefPointer(zz_pvt[-4].id_ptr);
	  }
	  g_untrapped_error = 0;
	} break;
case 60:
# line 889 "./ascParse.y"
{ /* nothing to do. just cruft to fix ; problem */ } break;
case 61:
# line 894 "./ascParse.y"
{
          struct UnitDefinition *ud;
          unsigned long c,len;

	  if( zz_pvt[-0].int_value != UNITS_T ) {
	    WarnMsg_MismatchEnd("UNITS", NULL, zz_pvt[-0].int_value, NULL);
	  }
          len = gl_length(zz_pvt[-1].listp);
          for (c=1; c <= len; c++) {
            ud = (struct UnitDefinition *)gl_fetch(zz_pvt[-1].listp,c);
            ProcessUnitDef(ud);
            DestroyUnitDef(ud);
          }
          gl_destroy(zz_pvt[-1].listp);
          zz_val.statptr = NULL;
	} break;
case 62:
# line 913 "./ascParse.y"
{
	  zz_val.listp = gl_create(100L);
	} break;
case 63:
# line 917 "./ascParse.y"
{
	  gl_append_ptr(zz_pvt[-1].listp,(char *)zz_pvt[-0].udefptr);
	  zz_val.listp = zz_pvt[-1].listp;
	} break;
case 64:
# line 925 "./ascParse.y"
{
	  zz_val.udefptr = CreateUnitDef(zz_pvt[-3].id_ptr,zz_pvt[-1].braced_ptr,Asc_ModuleFileName(Asc_CurrentModule()),
                             LineNum());
	} break;
case 65:
# line 934 "./ascParse.y"
{
	  zz_val.listp = NULL;
	} break;
case 66:
# line 938 "./ascParse.y"
{ /* To get rid of this, we will need a global proclist
           * that accumulates procs until a MODEL production is
           * completed. If any other sort of production is started,
           * and proclist is not NULL, it should be discarded.
           */
	} break;
case 67:
# line 945 "./ascParse.y"
{
	  zz_val.listp = zz_pvt[-0].listp;
	} break;
case 68:
# line 952 "./ascParse.y"
{
	  zz_val.listp = zz_pvt[-0].listp;
	  gl_sort(zz_val.listp,(CmpFunc)CmpProcs);
	} break;
case 69:
# line 959 "./ascParse.y"
{
	  zz_val.listp = gl_create(7L);
	} break;
case 70:
# line 963 "./ascParse.y"
{
	  gl_append_ptr(zz_pvt[-1].listp,(char *)zz_pvt[-0].procptr);
	  zz_val.listp = zz_pvt[-1].listp;
	} break;
case 71:
# line 971 "./ascParse.y"
{
	  if ((zz_pvt[-1].int_value != IDENTIFIER_T) || (zz_pvt[-4].id_ptr != g_end_identifier)) {
	    /* all identifier_t are from symbol table, so ptr match
	     * is sufficient for equality.
	     */
	    WarnMsg_MismatchEnd("METHOD", zz_pvt[-4].id_ptr, zz_pvt[-1].int_value, zz_pvt[-4].id_ptr);
	    /* assuming char *-IDENTIFIER_T equivalence here */
	  }
	  zz_val.procptr = CreateProcedure(zz_pvt[-4].id_ptr,zz_pvt[-2].slptr);
	} break;
case 72:
# line 985 "./ascParse.y"
{
	  zz_val.slptr = CreateStatementList(zz_pvt[-0].listp);
	} break;
case 73:
# line 992 "./ascParse.y"
{
	  zz_val.listp = gl_create(7L);
	} break;
case 74:
# line 996 "./ascParse.y"
{
	  /* this is appending to a gllist of statements, not yet slist. */
	  if (zz_pvt[-1].statptr != NULL) {
	    gl_append_ptr(zz_pvt[-2].listp,(char *)zz_pvt[-1].statptr);
	  }
	  zz_val.listp = zz_pvt[-2].listp;
	} break;
case 75:
# line 1004 "./ascParse.y"
{
	  ErrMsg_Generic("Error in statement input ");
	  zz_val.listp = zz_pvt[-2].listp;
	} break;
case 101:
# line 1040 "./ascParse.y"
{
	  struct TypeDescription *tmptype;
	  tmptype = FindType(zz_pvt[-2].id_ptr);
	  if (zz_pvt[-0].eptr != NULL) {
	    ErrMsg_Generic("Error: WITH VALUE clause not allowed in IS_A ");
	    g_untrapped_error++;
	    DestroyVariableList(zz_pvt[-4].lptr);
	    DestroySetList(g_typeargs);
	    DestroyExprList(zz_pvt[-0].eptr);
	    zz_val.statptr = NULL;
	  } else {
	    if (tmptype != NULL) {
	      if ((GetBaseType(tmptype) != model_type) &&
	          (g_typeargs != NULL)) {
	        FPRINTF(ASCERR,
	                "%sThe IS_A on line %s:%lu\n"
	                "  has arguments to the nonmodel type %s.\n",
	                StatioLabel(3),
	                Asc_ModuleFileName(Asc_CurrentModule()),
	                LineNum(),
	                zz_pvt[-2].id_ptr);
	        DestroyVariableList(zz_pvt[-4].lptr);
	        DestroySetList(g_typeargs);
	        DestroyExprList(zz_pvt[-0].eptr);
	        g_untrapped_error++;
	        zz_val.statptr = NULL;
	      } else {
	        zz_val.statptr = CreateISA(zz_pvt[-4].lptr,zz_pvt[-2].id_ptr,g_typeargs,zz_pvt[-1].id_ptr);
	      }
	    } else {
	      FPRINTF(ASCERR,"%sThe IS_A on line %s:%lu\n",
	              StatioLabel(3),Asc_ModuleFileName(Asc_CurrentModule()),
	              LineNum());
	      FPRINTF(ASCERR,"  uses the undefined type %s.\n",zz_pvt[-2].id_ptr);
	      DestroyVariableList(zz_pvt[-4].lptr);
	      DestroySetList(g_typeargs);
	      DestroyExprList(zz_pvt[-0].eptr);
	      g_untrapped_error++;
	      zz_val.statptr = NULL;
	    }
	  }
	  g_typeargs = NULL;

	} break;
case 102:
# line 1088 "./ascParse.y"
{
	  struct TypeDescription *tmptype;
	  tmptype = FindType(zz_pvt[-2].id_ptr);
	  if (tmptype != NULL) {
	    if ((GetBaseType(tmptype) != model_type) &&
	        (g_typeargs != NULL)) {
	      FPRINTF(ASCERR,"%sThe WILL_BE on line %s:%lu\n",
	              StatioLabel(3),Asc_ModuleFileName(Asc_CurrentModule()),
	              LineNum());
	      FPRINTF(ASCERR,
	              "  has arguments to the nonmodel type %s.\n",zz_pvt[-2].id_ptr);
	      DestroyVariableList(zz_pvt[-4].lptr);
	      DestroySetList(g_typeargs);
	      DestroyExprList(zz_pvt[-0].eptr);
	      g_untrapped_error++;
	      zz_val.statptr = NULL;
	    } else {
	      zz_val.statptr = CreateWILLBE(zz_pvt[-4].lptr,zz_pvt[-2].id_ptr,g_typeargs,zz_pvt[-1].id_ptr,zz_pvt[-0].eptr);
	    }
	  } else {
	    DestroyVariableList(zz_pvt[-4].lptr);
	    DestroySetList(g_typeargs);
	    DestroyExprList(zz_pvt[-0].eptr);
	    g_untrapped_error++;
	    zz_val.statptr = NULL;
	    FPRINTF(ASCERR,"%sThe WILL_BE on line %s:%lu\n",StatioLabel(3),
	            Asc_ModuleFileName(Asc_CurrentModule()),LineNum());
	    FPRINTF(ASCERR,"	uses the undefined type %s.\n",zz_pvt[-2].id_ptr);
	  }
	  g_typeargs = NULL;
	} break;
case 103:
# line 1123 "./ascParse.y"
{
	  zz_val.statptr = CreateALIASES(zz_pvt[-2].lptr,zz_pvt[-0].nptr);
	} break;
case 104:
# line 1128 "./ascParse.y"
{
	  int carray_err;
	  carray_err = 0;
	  if (VariableListLength(zz_pvt[-11].lptr) != 1L) {
	    carray_err = 1;
	    FPRINTF(ASCERR,
	            "%sCompound ALIASES allows only 1 LHS name. Found:\n",
	            StatioLabel(3));
	    WriteVariableList(ASCERR,zz_pvt[-11].lptr);
	    FPRINTF(ASCERR,"\n");
	  }
	  if (VariableListLength(zz_pvt[-5].lptr) != 1L) {
	    carray_err = 1;
	    FPRINTF(ASCERR,
	            "%sCompound ALIASES/IS_A allows only "
	            "1 LHS name. Found:\n",StatioLabel(3));
	    WriteVariableList(ASCERR,zz_pvt[-5].lptr);
	    FPRINTF(ASCERR,"\n");
	  }
	  /* verify $9 == "set" */
	  if (!carray_err && strcmp(zz_pvt[-3].id_ptr,BASE_SET_NAME) != 0) {
	    carray_err = 1;
	    FPRINTF(ASCERR,"%sCompound ALIASES statement requires IS_A %s. ",
	            StatioLabel(3),BASE_SET_NAME);
	    FPRINTF(ASCERR,"Found %s.\n",zz_pvt[-3].id_ptr);
	  }
	  /* verify set type */
	  if ((!carray_err) &&
	      (strcmp(zz_pvt[-1].id_ptr,BASE_CON_SYMBOL_NAME) != 0) &&
	      (strcmp(zz_pvt[-1].id_ptr,BASE_CON_INTEGER_NAME) != 0)) {
	    carray_err = 1;
	    FPRINTF(ASCERR,
	            "%sCompound ALIASES IS_A statement requires %s or %s.\n",
	            StatioLabel(3),BASE_CON_INTEGER_NAME,
	            BASE_CON_SYMBOL_NAME);
	    FPRINTF(ASCERR,"	Found %s.\n",zz_pvt[-1].id_ptr);
	  }
	  if (carray_err) {
	    DestroyVariableList(zz_pvt[-11].lptr);
	    DestroyVariableList(zz_pvt[-8].lptr);
	    DestroyVariableList(zz_pvt[-5].lptr);
	    DestroySetList(zz_pvt[-0].sptr);
	    g_untrapped_error++;
	    zz_val.statptr = NULL;
	  } else {
	    int intset;
	    intset = (strcmp(zz_pvt[-1].id_ptr,BASE_CON_INTEGER_NAME) == 0);
	    zz_val.statptr = CreateARR(zz_pvt[-11].lptr,zz_pvt[-8].lptr,zz_pvt[-5].lptr,intset,zz_pvt[-0].sptr);
	  }
	} break;
case 105:
# line 1182 "./ascParse.y"
{
	  zz_val.sptr = NULL;
	} break;
case 106:
# line 1186 "./ascParse.y"
{
	  zz_val.sptr = zz_pvt[-1].sptr;
	} break;
case 107:
# line 1193 "./ascParse.y"
{
	  if (FindType(zz_pvt[-1].id_ptr)) {
	    zz_val.statptr = CreateREF(zz_pvt[-3].lptr,zz_pvt[-1].id_ptr,zz_pvt[-0].id_ptr,1);
	  } else {
	    zz_val.statptr = CreateREF(zz_pvt[-3].lptr,zz_pvt[-1].id_ptr,zz_pvt[-0].id_ptr,1);
	    FPRINTF(ASCERR,"%s The _IS_ on line %s:%lu\n",
	            StatioLabel(2),
	            Asc_ModuleFileName(Asc_CurrentModule()),LineNum());
	    FPRINTF(ASCERR,"uses the unbuilt prototype %s.\n",zz_pvt[-1].id_ptr);
	  }
	} break;
case 108:
# line 1208 "./ascParse.y"
{
	  struct TypeDescription *tmptype;
	  tmptype = FindType(zz_pvt[-0].id_ptr);
	  if (tmptype != NULL) {
	    if ((GetBaseType(tmptype) != model_type) && 
                (g_typeargs != NULL)) {
	      FPRINTF(ASCERR,"%sThe IS_REFINED_TO on line %s:%lu\n",
	              StatioLabel(3),
	              Asc_ModuleFileName(Asc_CurrentModule()),LineNum());
	      FPRINTF(ASCERR,
	              "  has arguments to the nonmodel type %s.\n",zz_pvt[-0].id_ptr);
	      DestroyVariableList(zz_pvt[-2].lptr);
	      DestroySetList(g_typeargs);
	      g_untrapped_error++;
	      zz_val.statptr = NULL;
	    } else {
	      zz_val.statptr = CreateIRT(zz_pvt[-2].lptr,zz_pvt[-0].id_ptr,g_typeargs);
	    }
	  } else {
	    FPRINTF(ASCERR,"%sThe IS_REFINED_TO on line %s:%lu\n",
	            StatioLabel(3),
	            Asc_ModuleFileName(Asc_CurrentModule()),LineNum());
	    FPRINTF(ASCERR,"	uses the undefined type %s.\n",zz_pvt[-0].id_ptr);
	    DestroyVariableList(zz_pvt[-2].lptr);
	    DestroySetList(g_typeargs);
	    g_untrapped_error++;
	    zz_val.statptr = NULL;
	  }
	  g_typeargs = NULL;
	} break;
case 109:
# line 1242 "./ascParse.y"
{
	  zz_val.id_ptr = zz_pvt[-0].id_ptr;
	  g_callargs = NULL;
	} break;
case 110:
# line 1247 "./ascParse.y"
{
	  zz_val.id_ptr = zz_pvt[-3].id_ptr;
	  g_callargs = zz_pvt[-1].sptr;
	} break;
case 111:
# line 1255 "./ascParse.y"
{
	  zz_val.id_ptr = zz_pvt[-0].id_ptr;
	  g_typeargs = NULL;
	} break;
case 112:
# line 1260 "./ascParse.y"
{
	  zz_val.id_ptr = zz_pvt[-3].id_ptr;
	  g_typeargs = zz_pvt[-1].sptr;
	} break;
case 113:
# line 1268 "./ascParse.y"
{
	  zz_val.id_ptr = NULL;
	} break;
case 114:
# line 1272 "./ascParse.y"
{
	  zz_val.id_ptr = zz_pvt[-0].id_ptr;
	} break;
case 115:
# line 1279 "./ascParse.y"
{
	  zz_val.eptr = NULL;
	} break;
case 116:
# line 1283 "./ascParse.y"
{
	  zz_val.eptr = zz_pvt[-0].eptr;
	} break;
case 117:
# line 1290 "./ascParse.y"
{
	  zz_val.statptr = CreateAA(zz_pvt[-1].lptr);
	} break;
case 118:
# line 1297 "./ascParse.y"
{
	  zz_val.statptr = CreateATS(zz_pvt[-1].lptr);
	} break;
case 119:
# line 1304 "./ascParse.y"
{
	  zz_val.statptr = CreateWBTS(zz_pvt[-1].lptr);
	} break;
case 120:
# line 1311 "./ascParse.y"
{
	  zz_val.statptr = CreateWNBTS(zz_pvt[-1].lptr);
	} break;
case 121:
# line 1318 "./ascParse.y"
{
	  zz_val.statptr = CreateASSIGN(zz_pvt[-2].nptr,zz_pvt[-0].eptr);
	} break;
case 122:
# line 1322 "./ascParse.y"
{
	  zz_val.statptr = CreateCASSIGN(zz_pvt[-2].nptr,zz_pvt[-0].eptr);
	} break;
case 123:
# line 1329 "./ascParse.y"
{
	  if (IsRelation(zz_pvt[-0].eptr)) {
	    if (g_parse_relns == 0) {
	      DestroyExprList(zz_pvt[-0].eptr);
	      zz_val.statptr = NULL;
	    } else {
	      zz_val.statptr = CreateREL(NULL,zz_pvt[-0].eptr);
	    }
	  } else {
	    zz_val.statptr = CreateLOGREL(NULL,zz_pvt[-0].eptr);
	  }
	} break;
case 124:
# line 1342 "./ascParse.y"
{
	  if (IsRelation(zz_pvt[-0].eptr)) {
	    if (g_parse_relns == 0) {
	      DestroyExprList(zz_pvt[-0].eptr);
	      DestroyName(zz_pvt[-2].nptr);
	      zz_val.statptr = NULL;
	    } else {
	      zz_val.statptr = CreateREL(zz_pvt[-2].nptr,zz_pvt[-0].eptr);
	    }
	  } else {
	    zz_val.statptr = CreateLOGREL(zz_pvt[-2].nptr,zz_pvt[-0].eptr);
	  }
	} break;
case 125:
# line 1359 "./ascParse.y"
{
	  zz_val.eptr = zz_pvt[-0].eptr;
	  if (NumberOfRelOps(zz_pvt[-0].eptr) < 1) {
	    /* want at least 1. restriction to exactly 1 is in typelint */
	    ErrMsg_Generic("Error: Missing punctuation (,;:) or else\n"
	                  "expression contains the wrong number of relation\n"
	                  "operators (=, ==, <, >, <=, >=, !=)"
	                  " preceeding or ");
	    g_untrapped_error++;
	  }
	} break;
case 126:
# line 1371 "./ascParse.y"
{
	  zz_val.eptr = JoinExprLists(zz_pvt[-0].eptr,CreateOpExpr(e_minimize));
	  if (NumberOfRelOps(zz_pvt[-0].eptr) > 0) {
	    ErrMsg_Generic("Error: Objective function contains relation\n"
	                  "operators (=, ==, <, >, <=, >=, !=) ");
	    g_untrapped_error++;
	  }
	} break;
case 127:
# line 1380 "./ascParse.y"
{
	  zz_val.eptr = JoinExprLists(zz_pvt[-0].eptr,CreateOpExpr(e_maximize));
	  if (NumberOfRelOps(zz_pvt[-0].eptr)>0) {
	    ErrMsg_Generic("Error: Objective function contains relation\n"
	                  "operators (=, ==, <, >, <=, >=, !=) ");
	    g_untrapped_error++;
	  }
	} break;
case 128:
# line 1392 "./ascParse.y"
{
	  /*
	   * This is the blackbox declarative external relation.
	   */
	  struct VariableList *vl;
	  vl = JoinVariableLists(zz_pvt[-4].lptr,zz_pvt[-2].lptr);
	  zz_val.statptr = CreateEXTERN(2,zz_pvt[-8].nptr,zz_pvt[-6].id_ptr,vl,zz_pvt[-1].nptr,NULL);
	} break;
case 129:
# line 1404 "./ascParse.y"
{
	  zz_val.lptr = zz_pvt[-2].lptr;
	} break;
case 130:
# line 1411 "./ascParse.y"
{
	  zz_val.lptr = zz_pvt[-2].lptr;
	} break;
case 131:
# line 1418 "./ascParse.y"
{
	  zz_val.nptr = NULL;
	} break;
case 132:
# line 1422 "./ascParse.y"
{
	  zz_val.nptr = zz_pvt[-2].nptr;
	} break;
case 133:
# line 1429 "./ascParse.y"
{
	  /*
	   * This is the glassbox declarative external relation.
	   * This now allows a scope for placement of the relations
	   */
	  struct VariableList *vl = zz_pvt[-4].lptr;
	  struct Name *nptr;
	  char tmp[32]; CONST char *str;
	  sprintf(tmp,"%ld",zz_pvt[-2].int_value);
	  str = AddSymbol(tmp);
	  nptr = CreateIdName(str);
	  zz_val.statptr = CreateEXTERN(1,zz_pvt[-8].nptr,zz_pvt[-6].id_ptr,vl,nptr,zz_pvt[-0].nptr);
	} break;
case 134:
# line 1446 "./ascParse.y"
{
	  zz_val.nptr = NULL;
	} break;
case 135:
# line 1450 "./ascParse.y"
{
	  zz_val.nptr = zz_pvt[-0].nptr;
	} break;
case 136:
# line 1458 "./ascParse.y"
{
	  if( zz_pvt[-0].int_value != FOR_T ) {
	    WarnMsg_MismatchEnd("FOR", zz_pvt[-6].id_ptr, zz_pvt[-0].int_value, NULL);
	  }
          if (zz_pvt[-2].int_value == 0 && zz_pvt[-3].order != f_random) {
            /* create cannot have an order in declarative FOR */
	    ErrMsg_Generic("Error: FOR loops only accept DECREASING or\n"
                           "INCREASING in the method section ");
	    g_untrapped_error++;
          }
          if (zz_pvt[-2].int_value == 1 && zz_pvt[-3].order == f_random) {
            /* all FOR/DO default to increasing */
	    zz_val.statptr = CreateFOR(zz_pvt[-6].id_ptr,zz_pvt[-4].eptr,zz_pvt[-1].slptr,f_increasing);
          } else {
	    zz_val.statptr = CreateFOR(zz_pvt[-6].id_ptr,zz_pvt[-4].eptr,zz_pvt[-1].slptr,zz_pvt[-3].order);
          }
	} break;
case 137:
# line 1479 "./ascParse.y"
{
	  zz_val.order = f_random;
	} break;
case 138:
# line 1483 "./ascParse.y"
{
	  zz_val.order = f_increasing;
	} break;
case 139:
# line 1487 "./ascParse.y"
{
	  zz_val.order = f_decreasing;
	} break;
case 140:
# line 1494 "./ascParse.y"
{
          zz_val.int_value = 0; /* declarative FOR */
	} break;
case 141:
# line 1498 "./ascParse.y"
{
          zz_val.int_value = 1; /* method FOR */
	} break;
case 142:
# line 1505 "./ascParse.y"
{
	  zz_val.statptr = CreateRUN(zz_pvt[-0].nptr,NULL);
	} break;
case 143:
# line 1509 "./ascParse.y"
{
	  zz_val.statptr = CreateRUN(zz_pvt[-0].nptr,zz_pvt[-2].nptr);	  /* type :: name */
	} break;
case 144:
# line 1516 "./ascParse.y"
{
	  /*
	   * This is procedural external code.
	   */
	  zz_val.statptr = CreateEXTERN(0,NULL,zz_pvt[-3].id_ptr,zz_pvt[-1].lptr,NULL,NULL);
	} break;
case 145:
# line 1526 "./ascParse.y"
{
	  /*
	   * This is proper procedural external method code.
	   */
	  zz_val.statptr = CreateCALL(zz_pvt[-0].id_ptr,g_callargs);
          g_callargs = NULL;
	} break;
case 146:
# line 1537 "./ascParse.y"
{
	  if( zz_pvt[-0].int_value != IF_T ) {
	    WarnMsg_MismatchEnd("IF", NULL, zz_pvt[-0].int_value, NULL);
	  }
	  zz_val.statptr = CreateIF(zz_pvt[-4].eptr,zz_pvt[-2].slptr,zz_pvt[-1].slptr);
	} break;
case 147:
# line 1546 "./ascParse.y"
{
	  zz_val.slptr = NULL;
	} break;
case 148:
# line 1550 "./ascParse.y"
{
	  zz_val.slptr = zz_pvt[-0].slptr;
	} break;
case 149:
# line 1557 "./ascParse.y"
{
	  if( zz_pvt[-0].int_value != WHEN_T ) {
	    WarnMsg_MismatchEnd("WHEN", NULL, zz_pvt[-0].int_value, NULL);
	  }
	  ErrMsg_Generic("Error: () missing in WHEN statement ");
	  DestroyWhenList(zz_pvt[-1].wptr);
	  DestroyVariableList(zz_pvt[-2].lptr);
	  g_untrapped_error++;
	  zz_val.statptr = NULL;
	} break;
case 150:
# line 1568 "./ascParse.y"
{
	  if( zz_pvt[-0].int_value != WHEN_T ) {
	    WarnMsg_MismatchEnd("WHEN", NULL, zz_pvt[-0].int_value, NULL);
	  }
	  ErrMsg_Generic("Error: () missing in WHEN statement ");
	  DestroyWhenList(zz_pvt[-1].wptr);
	  DestroyVariableList(zz_pvt[-2].lptr);
	  DestroyName(zz_pvt[-5].nptr);
	  g_untrapped_error++;
	  zz_val.statptr = NULL;
	} break;
case 151:
# line 1580 "./ascParse.y"
{
	  if( zz_pvt[-0].int_value != WHEN_T ) {
	    WarnMsg_MismatchEnd("WHEN", NULL, zz_pvt[-0].int_value, NULL);
	  }
	  zz_val.statptr = CreateWHEN(NULL,zz_pvt[-3].lptr,zz_pvt[-1].wptr);
	} break;
case 152:
# line 1587 "./ascParse.y"
{
	  if( zz_pvt[-0].int_value != WHEN_T ) {
	    WarnMsg_MismatchEnd("WHEN", NULL, zz_pvt[-0].int_value, NULL);
	  }
	  zz_val.statptr = CreateWHEN(zz_pvt[-7].nptr,zz_pvt[-3].lptr,zz_pvt[-1].wptr);
	} break;
case 153:
# line 1597 "./ascParse.y"
{
	  zz_val.wptr = ReverseWhenCases(zz_pvt[-0].wptr);
	} break;
case 154:
# line 1604 "./ascParse.y"
{
	  zz_val.wptr = CreateWhen(zz_pvt[-2].sptr,zz_pvt[-0].slptr);
	} break;
case 155:
# line 1608 "./ascParse.y"
{
	  zz_val.wptr = CreateWhen(NULL,zz_pvt[-0].slptr);
	} break;
case 156:
# line 1612 "./ascParse.y"
{
	  zz_val.wptr = LinkWhenCases(CreateWhen(zz_pvt[-2].sptr,zz_pvt[-0].slptr),zz_pvt[-4].wptr);
	} break;
case 157:
# line 1616 "./ascParse.y"
{
	  zz_val.wptr = LinkWhenCases(CreateWhen(NULL,zz_pvt[-0].slptr),zz_pvt[-3].wptr);
	} break;
case 158:
# line 1623 "./ascParse.y"
{
	  zz_val.statptr = CreateFNAME(zz_pvt[-0].nptr);
	} break;
case 159:
# line 1630 "./ascParse.y"
{
	  if( zz_pvt[-0].int_value != SELECT_T ) {
	    WarnMsg_MismatchEnd("SELECT", NULL, zz_pvt[-0].int_value, NULL);
	  }
	  ErrMsg_Generic("Error: () missing in SELECT statement ");
	  DestroySelectList(zz_pvt[-1].septr);
	  DestroyVariableList(zz_pvt[-2].lptr);
	  g_untrapped_error++;
	  zz_val.statptr = NULL;
	} break;
case 160:
# line 1641 "./ascParse.y"
{
	  if( zz_pvt[-0].int_value != SELECT_T ) {
	    WarnMsg_MismatchEnd("SELECT", NULL, zz_pvt[-0].int_value, NULL);
	  }
	  zz_val.statptr = CreateSELECT(zz_pvt[-3].lptr,zz_pvt[-1].septr);
	} break;
case 161:
# line 1651 "./ascParse.y"
{
	  zz_val.septr = ReverseSelectCases(zz_pvt[-0].septr);
	} break;
case 162:
# line 1658 "./ascParse.y"
{
	  zz_val.septr = CreateSelect(zz_pvt[-2].sptr,zz_pvt[-0].slptr);
	} break;
case 163:
# line 1662 "./ascParse.y"
{
	  zz_val.septr = CreateSelect(NULL,zz_pvt[-0].slptr);
	} break;
case 164:
# line 1666 "./ascParse.y"
{
	  zz_val.septr = LinkSelectCases(CreateSelect(zz_pvt[-2].sptr,zz_pvt[-0].slptr),zz_pvt[-4].septr);
	} break;
case 165:
# line 1670 "./ascParse.y"
{
	  zz_val.septr = LinkSelectCases(CreateSelect(NULL,zz_pvt[-0].slptr),zz_pvt[-3].septr);
	} break;
case 166:
# line 1677 "./ascParse.y"
{
	  if( zz_pvt[-0].int_value != SWITCH_T ) {
	    WarnMsg_MismatchEnd("SWITCH", NULL, zz_pvt[-0].int_value, NULL);
	  }
	  ErrMsg_Generic("Error: () missing in SWITCH statement ");
	  DestroySwitchList(zz_pvt[-1].swptr);
	  DestroyVariableList(zz_pvt[-2].lptr);
	  g_untrapped_error++;
	  zz_val.statptr = NULL;
	} break;
case 167:
# line 1688 "./ascParse.y"
{
	  if( zz_pvt[-0].int_value != SWITCH_T ) {
	    WarnMsg_MismatchEnd("SWITCH", NULL, zz_pvt[-0].int_value, NULL);
	  }
	  zz_val.statptr = CreateSWITCH(zz_pvt[-3].lptr,zz_pvt[-1].swptr);
	} break;
case 168:
# line 1698 "./ascParse.y"
{
	  zz_val.swptr = ReverseSwitchCases(zz_pvt[-0].swptr);
	} break;
case 169:
# line 1705 "./ascParse.y"
{
	  zz_val.swptr = CreateSwitch(zz_pvt[-2].sptr,zz_pvt[-0].slptr);
	} break;
case 170:
# line 1709 "./ascParse.y"
{
	  zz_val.swptr = CreateSwitch(NULL,zz_pvt[-0].slptr);
	} break;
case 171:
# line 1713 "./ascParse.y"
{
	  zz_val.swptr = LinkSwitchCases(CreateSwitch(zz_pvt[-2].sptr,zz_pvt[-0].slptr),zz_pvt[-4].swptr);
	} break;
case 172:
# line 1717 "./ascParse.y"
{
	  zz_val.swptr = LinkSwitchCases(CreateSwitch(NULL,zz_pvt[-0].slptr),zz_pvt[-3].swptr);
	} break;
case 173:
# line 1724 "./ascParse.y"
{
	  if( zz_pvt[-0].int_value != CONDITIONAL_T ) {
	    WarnMsg_MismatchEnd("CONDITIONAL", NULL, zz_pvt[-0].int_value, NULL);
	  }
	  zz_val.statptr = CreateCOND(zz_pvt[-1].slptr);
	} break;
case 174:
# line 1734 "./ascParse.y"
{
	  /*  All processing of notes takes place in the notes_body.
	   *  Notes should NOT be added to the statement list.
	   */
	  if( zz_pvt[-0].int_value != NOTES_T ) {
	    WarnMsg_MismatchEnd("NOTES", NULL, zz_pvt[-0].int_value, NULL);
	  }
	  zz_val.statptr = NULL;
	} break;
case 175:
# line 1747 "./ascParse.y"
{
	  /*  At this point we have the "language", the names of the
	   *  objects we are explaining, and the explanation/notes itself.
	   *  We need to *do*something* with all this data; right now,
	   *  we just drop it on the floor.
	   */
	  DEBUG_NOTES(zz_pvt[-1].sym_ptr);
	  zz_val.notesptr = NULL;
	} break;
case 176:
# line 1757 "./ascParse.y"
{
	  DEBUG_NOTES(zz_pvt[-1].sym_ptr);
	  zz_val.notesptr = NULL;
	} break;
case 177:
# line 1765 "./ascParse.y"
{
	  DEBUG_NOTES(zz_pvt[-0].braced_ptr);
	  DestroyVariableList(zz_pvt[-1].lptr);
	  zz_val.notesptr = NULL;
	} break;
case 178:
# line 1771 "./ascParse.y"
{
	  DEBUG_NOTES(zz_pvt[-0].braced_ptr);
	  DestroyVariableList(zz_pvt[-1].lptr);
	  zz_val.notesptr = NULL;
	} break;
case 179:
# line 1780 "./ascParse.y"
{
	  /*
	   * Reversing the variable list is now essential to deal with
	   * external procedures and other things where order is important.
	   */
	  zz_val.lptr = ReverseVariableList(zz_pvt[-0].lptr);
	} break;
case 180:
# line 1791 "./ascParse.y"
{
	  zz_val.lptr = CreateVariableNode(zz_pvt[-0].nptr);
	} break;
case 181:
# line 1795 "./ascParse.y"
{
	  zz_val.lptr = CreateVariableNode(zz_pvt[-0].nptr);
	  LinkVariableNodes(zz_val.lptr,zz_pvt[-2].lptr);
	} break;
case 182:
# line 1803 "./ascParse.y"
{
	  zz_val.nptr = ReverseName(zz_pvt[-1].nptr);
	} break;
case 183:
# line 1810 "./ascParse.y"
{
	  zz_val.nptr = CreateIdName(zz_pvt[-0].id_ptr);
	} break;
case 184:
# line 1814 "./ascParse.y"
{
	  zz_val.nptr = CreateIdName(zz_pvt[-0].id_ptr);
	  LinkNames(zz_val.nptr,zz_pvt[-2].nptr);
	} break;
case 185:
# line 1819 "./ascParse.y"
{
	  if (zz_pvt[-1].sptr == NULL) {
	    FPRINTF(ASCERR,"%ssyntax error at %lu\n",StatioLabel(3),LineNum());
	    FPRINTF(ASCERR,"  Empty set in name definition\n");
	    FPRINTF(ASCERR,"  Name: ");
	    WriteName(ASCERR,zz_pvt[-3].nptr);
	    FPRINTF(ASCERR,"[]\n");
	    g_untrapped_error++;
	  } else {
	    zz_val.nptr = CreateSetName(zz_pvt[-1].sptr);
	    LinkNames(zz_val.nptr,zz_pvt[-3].nptr);
	  }
	} break;
case 186:
# line 1836 "./ascParse.y"
{
          g_end_identifier = NULL;
          zz_val.int_value = CONDITIONAL_T;
        } break;
case 187:
# line 1841 "./ascParse.y"
{
          g_end_identifier = NULL;
          zz_val.int_value = FOR_T;
        } break;
case 188:
# line 1846 "./ascParse.y"
{
          g_end_identifier = NULL;
          zz_val.int_value = IF_T;
        } break;
case 189:
# line 1851 "./ascParse.y"
{
          g_end_identifier = NULL;
          zz_val.int_value = INTERACTIVE_T;
        } break;
case 190:
# line 1856 "./ascParse.y"
{
          g_end_identifier = NULL;
          zz_val.int_value = METHODS_T;
        } break;
case 191:
# line 1861 "./ascParse.y"
{
          g_end_identifier = NULL;
          zz_val.int_value = NOTES_T;
        } break;
case 192:
# line 1866 "./ascParse.y"
{
          g_end_identifier = NULL;
          zz_val.int_value = SELECT_T;
        } break;
case 193:
# line 1871 "./ascParse.y"
{
          g_end_identifier = NULL;
          zz_val.int_value = SWITCH_T;
        } break;
case 194:
# line 1876 "./ascParse.y"
{
          g_end_identifier = NULL;
          zz_val.int_value = UNITS_T;
        } break;
case 195:
# line 1881 "./ascParse.y"
{
          g_end_identifier = NULL;
          zz_val.int_value = WHEN_T;
        } break;
case 196:
# line 1886 "./ascParse.y"
{
          g_end_identifier = zz_pvt[-0].id_ptr;
          zz_val.int_value = IDENTIFIER_T;
        } break;
case 197:
# line 1891 "./ascParse.y"
{
          g_end_identifier = NULL;
          zz_val.int_value = END_T;
        } break;
case 198:
# line 1899 "./ascParse.y"
{
	  zz_val.dquote_ptr = NULL;
	} break;
case 199:
# line 1903 "./ascParse.y"
{
	  DEBUG_NOTES(zz_pvt[-0].dquote_ptr);
	  zz_val.dquote_ptr = NULL;
	} break;
case 200:
# line 1911 "./ascParse.y"
{
	  zz_val.sptr = ReverseSetList(zz_pvt[-0].sptr);
	} break;
case 201:
# line 1915 "./ascParse.y"
{
	  zz_val.sptr = NULL;
	} break;
case 202:
# line 1922 "./ascParse.y"
{
	  zz_val.sptr = CreateSingleSet(zz_pvt[-0].eptr);
	} break;
case 203:
# line 1926 "./ascParse.y"
{
	  zz_val.sptr = CreateRangeSet(zz_pvt[-2].eptr,zz_pvt[-0].eptr);
	} break;
case 204:
# line 1930 "./ascParse.y"
{
	  zz_val.sptr = CreateSingleSet(zz_pvt[-0].eptr);
	  LinkSets(zz_val.sptr,zz_pvt[-2].sptr);
	} break;
case 205:
# line 1935 "./ascParse.y"
{
	  zz_val.sptr = CreateRangeSet(zz_pvt[-2].eptr,zz_pvt[-0].eptr);
	  LinkSets(zz_val.sptr,zz_pvt[-4].sptr);
	} break;
case 206:
# line 1943 "./ascParse.y"
{
	  zz_val.real_value = zz_pvt[-0].int_value;
	  g_constant_type = LONGCONSTANT;
	  g_default_dim_ptr = Dimensionless();
	} break;
case 207:
# line 1949 "./ascParse.y"
{
	  zz_val.real_value = zz_pvt[-0].real_value;
	  g_constant_type = DOUBLECONSTANT;
	  g_default_dim_ptr = g_dim_ptr;
	} break;
case 208:
# line 1958 "./ascParse.y"
{
	  zz_val.real_value = zz_pvt[-1].real_value*zz_pvt[-0].real_value;
	} break;
case 209:
# line 1962 "./ascParse.y"
{
	  unsigned long pos;
	  int error_code;
	  g_units_ptr = FindOrDefineUnits(zz_pvt[-0].braced_ptr,&pos,&error_code);
	  if (g_units_ptr != NULL) {
	    zz_val.real_value = (double)zz_pvt[-1].int_value*UnitsConvFactor(g_units_ptr);
	    g_dim_ptr = UnitsDimensions(g_units_ptr);
	  } else {
            char **errv;
	    zz_val.real_value = (double)zz_pvt[-1].int_value;
	    g_dim_ptr = WildDimension();
	    FPRINTF(ASCERR,"%sUndefined units {%s} %s:%lu.\n",StatioLabel(3),
	            zz_pvt[-0].braced_ptr,Asc_ModuleFileName(Asc_CurrentModule()),LineNum());
            errv = UnitsExplainError(zz_pvt[-0].braced_ptr,error_code,pos);
	    FPRINTF(ASCERR,"  %s\n  %s\n  %s\n",errv[0],errv[1],errv[2]);
	    g_untrapped_error++;
	  }
	} break;
case 210:
# line 1984 "./ascParse.y"
{
	  g_dim_ptr = Dimensionless();
	  zz_val.real_value = 1.0;
	} break;
case 211:
# line 1989 "./ascParse.y"
{
	  unsigned long pos;
	  int error_code;
	  g_units_ptr = FindOrDefineUnits(zz_pvt[-0].braced_ptr,&pos,&error_code);
	  if (g_units_ptr != NULL) {
	    zz_val.real_value = UnitsConvFactor(g_units_ptr);
	    g_dim_ptr = UnitsDimensions(g_units_ptr);
	  } else {
            char **errv;
	    zz_val.real_value = 1.0;
	    g_dim_ptr = WildDimension();
	    FPRINTF(ASCERR,"%sUndefined units {%s} %s:%lu.\n",StatioLabel(3),
	            zz_pvt[-0].braced_ptr,Asc_ModuleFileName(Asc_CurrentModule()),LineNum());
            errv = UnitsExplainError(zz_pvt[-0].braced_ptr,error_code,pos);
	    FPRINTF(ASCERR,"  %s\n  %s\n  %s\n",errv[0],errv[1],errv[2]);
	    g_untrapped_error++;
	  }
	} break;
case 212:
# line 2011 "./ascParse.y"
{
	  zz_val.dimp = zz_pvt[-0].dimp;
	} break;
case 213:
# line 2015 "./ascParse.y"
{
	  zz_val.dimp = Dimensionless();
	} break;
case 214:
# line 2019 "./ascParse.y"
{
	  zz_val.dimp = WildDimension();
	} break;
case 215:
# line 2026 "./ascParse.y"
{
	  zz_val.dimp = WildDimension();
	} break;
case 216:
# line 2030 "./ascParse.y"
{
	  zz_val.dimp = FindOrAddDimen(&(zz_pvt[-0].dimen));
	} break;
case 217:
# line 2037 "./ascParse.y"
{
	  ParseDim(&(zz_val.dimen),zz_pvt[-0].id_ptr);
	} break;
case 218:
# line 2041 "./ascParse.y"
{
	  ClearDimensions(&(zz_val.dimen));
	} break;
case 219:
# line 2045 "./ascParse.y"
{
	  zz_val.dimen = SubDimensions(&(zz_pvt[-2].dimen),&(zz_pvt[-0].dimen));
	} break;
case 220:
# line 2049 "./ascParse.y"
{
	  zz_val.dimen = AddDimensions(&(zz_pvt[-2].dimen),&(zz_pvt[-0].dimen));
	} break;
case 221:
# line 2053 "./ascParse.y"
{
	  zz_val.dimen = ScaleDimensions(&(zz_pvt[-2].dimen),zz_pvt[-0].frac_value);
	} break;
case 222:
# line 2057 "./ascParse.y"
{
	  CopyDimensions(&(zz_pvt[-1].dimen),&(zz_val.dimen));
	} break;
case 223:
# line 2064 "./ascParse.y"
{
	  zz_val.frac_value = zz_pvt[-1].int_value ? NegateF(zz_pvt[-0].frac_value) : zz_pvt[-0].frac_value;
	} break;
case 224:
# line 2071 "./ascParse.y"
{
	  zz_val.frac_value = CreateFraction((short)zz_pvt[-0].int_value,(short)1);
	} break;
case 225:
# line 2075 "./ascParse.y"
{
	  zz_val.frac_value = CreateFraction((short)zz_pvt[-3].int_value,(short)zz_pvt[-1].int_value);
	} break;
case 226:
# line 2082 "./ascParse.y"
{
	  zz_val.int_value = 0;
	} break;
case 227:
# line 2086 "./ascParse.y"
{
	  zz_val.int_value = 0;
	} break;
case 228:
# line 2090 "./ascParse.y"
{
	  zz_val.int_value = 1;
	} break;
case 229:
# line 2097 "./ascParse.y"
{
	  zz_val.eptr = CreateIntExpr(zz_pvt[-0].int_value);
	} break;
case 230:
# line 2101 "./ascParse.y"
{
	  zz_val.eptr = CreateIntExpr(LONG_MAX-1);
	} break;
case 231:
# line 2105 "./ascParse.y"
{
	  zz_val.eptr = CreateRealExpr(zz_pvt[-0].real_value,g_dim_ptr);
	} break;
case 232:
# line 2109 "./ascParse.y"
{
	  zz_val.eptr = CreateRealExpr(DBL_MAX/(1+1e-15),Dimensionless());
	} break;
case 233:
# line 2113 "./ascParse.y"
{
	  zz_val.eptr = CreateTrueExpr();
	} break;
case 234:
# line 2117 "./ascParse.y"
{
	  zz_val.eptr = CreateFalseExpr();
	} break;
case 235:
# line 2121 "./ascParse.y"
{
	  zz_val.eptr = CreateAnyExpr();
	} break;
case 236:
# line 2125 "./ascParse.y"
{
	  zz_val.eptr = CreateSymbolExpr(zz_pvt[-0].sym_ptr);
	} break;
case 237:
# line 2129 "./ascParse.y"
{
	  zz_val.eptr = CreateVarExpr(zz_pvt[-0].nptr);
	} break;
case 238:
# line 2133 "./ascParse.y"
{
	  zz_val.eptr = CreateSetExpr(zz_pvt[-1].sptr);
	} break;
case 239:
# line 2137 "./ascParse.y"
{
	  zz_pvt[-0].eptr = JoinExprLists(zz_pvt[-0].eptr,CreateOpExpr(e_plus));
	  zz_val.eptr = JoinExprLists(zz_pvt[-2].eptr,zz_pvt[-0].eptr);
	} break;
case 240:
# line 2142 "./ascParse.y"
{
	  zz_pvt[-0].eptr = JoinExprLists(zz_pvt[-0].eptr,CreateOpExpr(e_minus));
	  zz_val.eptr = JoinExprLists(zz_pvt[-2].eptr,zz_pvt[-0].eptr);
	} break;
case 241:
# line 2147 "./ascParse.y"
{
	  zz_pvt[-0].eptr = JoinExprLists(zz_pvt[-0].eptr,CreateOpExpr(e_times));
	  zz_val.eptr = JoinExprLists(zz_pvt[-2].eptr,zz_pvt[-0].eptr);
	} break;
case 242:
# line 2152 "./ascParse.y"
{
	  zz_pvt[-0].eptr = JoinExprLists(zz_pvt[-0].eptr,CreateOpExpr(e_divide));
	  zz_val.eptr = JoinExprLists(zz_pvt[-2].eptr,zz_pvt[-0].eptr);
	} break;
case 243:
# line 2157 "./ascParse.y"
{
	  zz_pvt[-0].eptr = JoinExprLists(zz_pvt[-0].eptr,CreateOpExpr(e_power));
	  zz_val.eptr = JoinExprLists(zz_pvt[-2].eptr,zz_pvt[-0].eptr);
	} break;
case 244:
# line 2162 "./ascParse.y"
{
	  zz_pvt[-0].eptr = JoinExprLists(zz_pvt[-0].eptr,CreateOpExpr(e_and));
	  zz_val.eptr = JoinExprLists(zz_pvt[-2].eptr,zz_pvt[-0].eptr);
	} break;
case 245:
# line 2167 "./ascParse.y"
{
	  zz_pvt[-0].eptr = JoinExprLists(zz_pvt[-0].eptr,CreateOpExpr(e_or));
	  zz_val.eptr = JoinExprLists(zz_pvt[-2].eptr,zz_pvt[-0].eptr);
	} break;
case 246:
# line 2172 "./ascParse.y"
{
	  zz_val.eptr = JoinExprLists(zz_pvt[-0].eptr,CreateOpExpr(e_not));
	} break;
case 247:
# line 2176 "./ascParse.y"
{
	  zz_pvt[-0].eptr = JoinExprLists(zz_pvt[-0].eptr,zz_pvt[-1].eptr);
	  zz_val.eptr = JoinExprLists(zz_pvt[-2].eptr,zz_pvt[-0].eptr);
	} break;
case 248:
# line 2181 "./ascParse.y"
{
	  zz_pvt[-0].eptr = JoinExprLists(zz_pvt[-0].eptr,zz_pvt[-1].eptr);
	  zz_val.eptr = JoinExprLists(zz_pvt[-2].eptr,zz_pvt[-0].eptr);
	} break;
case 249:
# line 2186 "./ascParse.y"
{
	  zz_pvt[-0].eptr = JoinExprLists(zz_pvt[-0].eptr,CreateOpExpr(e_in));
	  zz_val.eptr = JoinExprLists(zz_pvt[-2].eptr,zz_pvt[-0].eptr);
	} break;
case 250:
# line 2191 "./ascParse.y"
{
	  zz_pvt[-0].eptr = JoinExprLists(zz_pvt[-0].eptr,CreateOpExpr(e_st));
	  zz_val.eptr = JoinExprLists(zz_pvt[-2].eptr,zz_pvt[-0].eptr);
	} break;
case 251:
# line 2196 "./ascParse.y"
{
	  zz_pvt[-0].eptr = JoinExprLists(zz_pvt[-0].eptr,CreateOpExpr(e_st));
	  zz_val.eptr = JoinExprLists(zz_pvt[-2].eptr,zz_pvt[-0].eptr);
	} break;
case 252:
# line 2201 "./ascParse.y"
{
	  zz_val.eptr = zz_pvt[-0].eptr;
	} break;
case 253:
# line 2205 "./ascParse.y"
{
	  zz_val.eptr = JoinExprLists(zz_pvt[-0].eptr,CreateOpExpr(e_uminus));
	} break;
case 254:
# line 2209 "./ascParse.y"
{
	  zz_val.eptr = CreateSatisfiedExpr(zz_pvt[-3].nptr,zz_pvt[-1].real_value,g_dim_ptr);
	} break;
case 255:
# line 2213 "./ascParse.y"
{
	  zz_val.eptr = CreateSatisfiedExpr(zz_pvt[-1].nptr,DBL_MAX,NULL);
	} break;
case 256:
# line 2217 "./ascParse.y"
{
	  DestroySetList(zz_pvt[-1].sptr);
	  zz_val.eptr = NULL;
	  ErrMsg_ParensBrackets("SUM");
	  g_untrapped_error++;
	} break;
case 257:
# line 2224 "./ascParse.y"
{
	  zz_val.eptr = CreateBuiltin(e_sum,zz_pvt[-1].sptr);
	} break;
case 258:
# line 2228 "./ascParse.y"
{
	  DestroySetList(zz_pvt[-1].sptr);
	  zz_val.eptr = NULL;
	  ErrMsg_ParensBrackets("PROD");
	  g_untrapped_error++;
	} break;
case 259:
# line 2235 "./ascParse.y"
{
	  zz_val.eptr = CreateBuiltin(e_prod,zz_pvt[-1].sptr);
	} break;
case 260:
# line 2239 "./ascParse.y"
{
	  DestroySetList(zz_pvt[-1].sptr);
	  zz_val.eptr = NULL;
	  ErrMsg_ParensBrackets("UNION");
	  g_untrapped_error++;
	} break;
case 261:
# line 2246 "./ascParse.y"
{
	  zz_val.eptr = CreateBuiltin(e_union,zz_pvt[-1].sptr);
	} break;
case 262:
# line 2250 "./ascParse.y"
{
	  DestroySetList(zz_pvt[-1].sptr);
	  zz_val.eptr = NULL;
	  ErrMsg_ParensBrackets("INTERSECTION");
	  g_untrapped_error++;
	} break;
case 263:
# line 2257 "./ascParse.y"
{
	  zz_val.eptr = CreateBuiltin(e_inter,zz_pvt[-1].sptr);
	} break;
case 264:
# line 2261 "./ascParse.y"
{
	  DestroySetList(zz_pvt[-1].sptr);
	  zz_val.eptr = NULL;
	  ErrMsg_ParensBrackets("CARD");
	  g_untrapped_error++;
	} break;
case 265:
# line 2268 "./ascParse.y"
{
	  zz_val.eptr = CreateBuiltin(e_card,zz_pvt[-1].sptr);
	} break;
case 266:
# line 2272 "./ascParse.y"
{
	  DestroySetList(zz_pvt[-1].sptr);
	  zz_val.eptr = NULL;
	  ErrMsg_ParensBrackets("CHOICE");
	  g_untrapped_error++;
	} break;
case 267:
# line 2279 "./ascParse.y"
{
	  zz_val.eptr = CreateBuiltin(e_choice,zz_pvt[-1].sptr);
	} break;
case 268:
# line 2283 "./ascParse.y"
{
	  CONST struct Func *fptr;
	  if ((fptr = LookupFunc(zz_pvt[-3].id_ptr))) {
	    zz_val.eptr = JoinExprLists(zz_pvt[-1].eptr,CreateFuncExpr(fptr));
	  } else {
	    zz_val.eptr = NULL;
	    FPRINTF(ASCERR,"%sFunction %s is not defined.\n",
	            StatioLabel(3),zz_pvt[-3].id_ptr);
	    FPRINTF(ASCERR,"  File name: %s:%lu\n",
	            Asc_ModuleFileName(Asc_CurrentModule()),
	            LineNum());
	    g_untrapped_error++;
	  }
	} break;
case 269:
# line 2298 "./ascParse.y"
{
	  zz_val.eptr = zz_pvt[-1].eptr;
	} break;
case 270:
# line 2305 "./ascParse.y"
{
	  zz_val.eptr = CreateOpExpr(e_equal);
	} break;
case 271:
# line 2309 "./ascParse.y"
{
	  zz_val.eptr = CreateOpExpr(e_less);
	} break;
case 272:
# line 2313 "./ascParse.y"
{
	  zz_val.eptr = CreateOpExpr(e_greater);
	} break;
case 273:
# line 2317 "./ascParse.y"
{
	  zz_val.eptr = CreateOpExpr(e_lesseq);
	} break;
case 274:
# line 2321 "./ascParse.y"
{
	  zz_val.eptr = CreateOpExpr(e_greatereq);
	} break;
case 275:
# line 2325 "./ascParse.y"
{
	  zz_val.eptr = CreateOpExpr(e_notequal);
	} break;
case 276:
# line 2332 "./ascParse.y"
{
	  zz_val.eptr = CreateOpExpr(e_boolean_eq);
	} break;
case 277:
# line 2336 "./ascParse.y"
{
	  zz_val.eptr = CreateOpExpr(e_boolean_neq);
	} break;
	}
	goto zz_stack;		/* reset registers in driver code */
}

# ifdef __RUNTIME_ZZ_MAXDEPTH

static int allocate_stacks() {
	/* allocate the zz_s and zz_v stacks */
	zz_s = (int *) malloc(zz_maxdepth * sizeof(int));
	zz_v = (ZZ_STYPE *) malloc(zz_maxdepth * sizeof(ZZ_STYPE));

	if (zz_s==0 || zz_v==0) {
	   zz_error( (nl_msg(30004,"unable to allocate space for yacc stacks")) );
	   return(1);
	   }
	else return(0);

}


static void free_stacks() {
	if (zz_s!=0) free((char *) zz_s);
	if (zz_v!=0) free((char *) zz_v);
}

# endif  /* defined(__RUNTIME_ZZ_MAXDEPTH) */

