/*
 *  Initialization Routines
 *  by Tom Epperly
 *  Created: 3/24/1990
 *  Version: $Revision: 1.9 $
 *  Version control file: $RCSfile: initialize.h,v $
 *  Date last modified: $Date: 97/07/18 12:30:01 $
 *  Last modified by: $Author: mthomas $
 *
 *  This file is part of the Ascend Language Interpreter.
 *
 *  Copyright (C) 1990, 1993, 1994 Thomas Guthrie Epperly
 *
 *  The Ascend Language Interpreter is free software; you can redistribute
 *  it and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  The Ascend Language Interpreter is distributed in hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the program; if not, write to the Free Software Foundation,
 *  Inc., 675 Mass Ave, Cambridge, MA 02139 USA.  Check the file named
 *  COPYING.
 */

#ifndef __INITIALIZE_H_SEEN__
#define __INITIALIZE_H_SEEN__


/*
 *  When #including initialize.h, make sure these files are #included first:
 *         #include "instance_enum.h"
 *         #include "fractions.h"
 *         #include "compiler.h"
 *         #include "dimen.h"
 *         #include "types.h"
 */


/*
 * the following enum is for internal consumption only.
 */
enum Proc_enum {
  Proc_all_ok, Proc_bad_name, Proc_illegal_name_use,
  Proc_name_not_found, Proc_type_not_found, Proc_illegal_type_use,
  Proc_proc_not_found, Proc_instance_not_found, Proc_stack_exceeded,
  Proc_stack_exceeded_this_frame, Proc_unknown_error
};

extern void SetProcStackLimit(unsigned long);
/*
 *  void SetProcStackLimit(l);
 *  Sets the procedure stack limit = l.
 *  The stack limit starts out at INITSTACKLIMIT.
 *  The limit exists to prevent infinite loops from running
 *  the machine out of C automatic variable space.
 */
#define INITSTACKLIMIT 20

extern unsigned long GetProcStackLimit(void);
/*
 *  int GetProcStackLimit(l);
 *  Gets the procedure stack limit currently set.
 */

extern enum Proc_enum Initialize(struct Instance *,struct Name *);
/*
 *  enum Proc_enum Initialize(context,name)
 *  struct Instance *context;
 *  struct Name *name;
 *  This procedure will execute the initialization code indicated by name
 *  with respect to the context instance.
 *  Will return Proc_all_ok if all went well; otherwise it will return
 *  one of the error codes above.
 */

extern enum Proc_enum ClassAccessInitialize(struct Instance *, struct Name *,
         struct Name *);
/*
 *  enum Proc_enum ClassAccessInitialize(context,class,procname);
 *  struct Instance *context;
 *  struct Name *class;
 *  struct Name *name;
 *  Will attempt to run the initialization procedure given by using so-called
 *  class access, i.e., deals with syntax such as RUN Mytype::values.
 *  The context instance will be used.
 *  Will return Proc_all_ok if all went well; otherwise it will return
 *  one of the error codes above.
 *
 */

extern struct InitProcedure *FindProcedure(CONST struct Instance *,
                                           CONST char *);
/*
 *  struct InitProcedure *FindProcedure(i,procname)
 *  This will return the pointer to the named procedure if it exists in i.
 *  Returns null if named not found.
 */

#endif /* __INITIALIZE_H_SEEN__ */
