/*
 *  Procedure Data Structure
 *  by Tom Epperly
 *  Created: 1/10/90
 *  Version: $Revision: 1.9 $
 *  Version control file: $RCSfile: proc.h,v $
 *  Date last modified: $Date: 97/07/25 15:13:28 $
 *  Last modified by: $Author: rv2a $
 *
 *  This file is part of the Ascend Language Interpreter.
 *
 *  Copyright (C) 1990, 1993, 1994 Thomas Guthrie Epperly
 *
 *  The Ascend Language Interpreter is free software; you can redistribute
 *  it and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  The Ascend Language Interpreter is distributed in hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with the program; if not, write to the Free Software Foundation, Inc., 675
 *  Mass Ave, Cambridge, MA 02139 USA.  Check the file named COPYING.
 */

/*
 *  When #including proc.h, make sure these files are #included first:
 *         #include "compiler.h"
 */



#ifndef __PROC_H_SEEN__
#define __PROC_H_SEEN__
/* requires
# #include"compiler.h"
# #include"list.h"
# #include"slist.h"
*/

struct InitProcedure {
  CONST char *name;		/* procedure's name */
  struct StatementList *slist;	/* statement list */
};

extern struct InitProcedure *CreateProcedure(CONST char *,
          struct StatementList *);
/*
 *  struct InitProcedure *CreateProcedure(name,stats)
 *  const char *name;
 *  struct StatementList *stats;
 *  Create a procedure data structure with name and stats.
 */

extern struct InitProcedure *CopyProcedure(struct InitProcedure *);
/*
 *  struct InitProcedure *CopyProcedure(p)
 *  struct InitProcedure *p;
 *  Make a copy of procedure p, but this copy should not be modified.
 *  If you want to change the structure, use CopyProcToModify.  Use
 *  DestroyProcedure when you are done with it.
 */

extern struct InitProcedure *CopyProcToModify(struct InitProcedure *);
/*
 *  struct InitProcedure *CopyProcToModify(p)
 *  struct InitProcedure *p;
 *  Make a copy, but this copy can be changed.
 */

extern struct gl_list_t *MergeProcedureLists(struct gl_list_t *,
                                              struct gl_list_t *);
/*
 *  plr = MergeProcedureLists(pl1,pl2);
 *  struct gl_list_t *plr, *pl1, *pl2;
 *  pl2 may be NULL.
 *  Merges the lists of (struct InitProcedure *) given into a third list,
 *  plr.
 *  pl2, if it exists, is destroyed.
 *  pl1 is assumed to belong to someone else and left alone.
 *  The copying is done by reference as much as possible.
 *  The result is sorted with CmpProcs.
 */

extern void DestroyProcedure(struct InitProcedure *);
/*
 *  void DestroyProcedure(p)
 *  struct InitProcedure *p;
 *  Destroy this reference to p.  This won't necessary destroy all the parts
 *  of p.
 */

extern void DestroyProcedureList(struct gl_list_t *);
/*
 *  void DestroyProcedureList(pl)
 *  struct gl_list_t *pl contain pointers to
 *  struct InitProcedure *p
 *  Destroy this reference to p.  This won't necessary destroy all the parts
 *  of each p unless the parts have no other references. The gl_list is
 *  destroyed as are all the p in it.
 *  Handles NULL input gracefully.
 */

extern int CompareProcedureLists(struct gl_list_t *,struct gl_list_t *,
                                 unsigned long int *);
/*
 *  CompareProcedureLists(pl1,pl2,&n)
 *  struct gl_list_t *pl1, *pl2 contain pointers to
 *  struct InitProcedure *p.
 *  unsigned long n;
 *  Returns 0 if pl1,pl2 semantically equivalent, 1 if not.
 *  If return is 1, n contains the index of the first different
 *  procedure.
 */  

#ifdef NDEBUG
#define ProcName(p) ((p)->name)
#else
#define ProcName(p) ProcNameF(p)
#endif
extern CONST char *ProcNameF(CONST struct InitProcedure *);
/*
 *  macro ProcName(p)
 *  const char *ProcNameF(p)
 *  const struct InitProcedure *p;
 *  Return the name part of a procedure structure.
 */

#ifdef NDEBUG
#define ProcStatementList(p) ((p)->slist)
#else
#define ProcStatementList(p) ProcStatementListF(p)
#endif
extern struct StatementList *ProcStatementListF(CONST struct InitProcedure *);
/*
 *  struct StatementList *ProcStatementListF(p)
 *  const struct InitProcedure *p;
 *  Return the statement list part of the procedure structure.
 */

extern int CmpProcs(CONST struct InitProcedure *,CONST struct InitProcedure *);
/*
 *  int CmpProcs(p1,p2)
 *  const struct InitProcedure *p1,*p2;
 *  Compare the two procedures to provide an ordering for them.
 */
#endif /* __PROC_H_SEEN__ */
