/*
 *  Relation Output Routines
 *  by Tom Epperly
 *  Created: 2/8/90
 *  Version: $Revision: 1.7 $
 *  Version control file: $RCSfile: relation_io.h,v $
 *  Date last modified: $Date: 97/07/18 12:33:36 $
 *  Last modified by: $Author: mthomas $
 *
 *  This file is part of the Ascend Language Interpreter.
 *
 *  Copyright (C) 1990, 1993, 1994 Thomas Guthrie Epperly
 *
 *  The Ascend Language Interpreter is free software; you can redistribute
 *  it and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  The Ascend Language Interpreter is distributed in hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with the program; if not, write to the Free Software Foundation, Inc., 675
 *  Mass Ave, Cambridge, MA 02139 USA.  Check the file named COPYING.
 */

/*
 *  When #including relation_io.h, make sure these files are #included first:
 *         #include "instance_enum.h"
 *         #include "fractions.h"
 *         #include "compiler.h"
 *         #include "dimen.h"
 *         #include "types.h"
 *         #include "relation_type.h"
 */


#ifndef __RELATION_IO_H_SEEN__
#define __RELATION_IO_H_SEEN__
/* requires
# #include<stdio.h>
# #include"compiler.h"
# #include"exprs.h"
# #include"instance_enum.h"
# #include"relation.h"
*/

extern int NeedParen(enum Expr_enum,enum Expr_enum,int);
/*
 *  int NeedParen(parent,child,rhs);
 *  enum Expr_enum parent_op;
 *  enum Expr_enum child_op;
 *  int rhs;
 *  Given a unary or binary expression, will determine whether, the
 *  child expression needs parentheses. "rhs" tells if we are looking
 *  at the left or right side of a binary token.
 *  E.g x ^ (2/y), would need parens exactly as shown to give the
 *  correct precedence.
 *  This would be called as : NeedParen(e_power,e_divide,right);
 *  and would return nonzero.
 */

extern void WriteRelation(FILE *,CONST struct Instance *,
     CONST struct Instance *);
/*
 *  WriteRelation(f,relinst,ref);
 *  FILE *f;
 *  const struct Instance *relinst;
 *  const struct Instance *ref;
 *  Write the relation in infix to the file indicated.
 *  This function comfortably handles all the relation types.
 */

extern char *WriteRelationString(CONST struct Instance *,
     CONST struct Instance *);
/*
 *  s = WriteRelationString(relinst,ref);
 *  char *f;
 *  const struct Instance *relinst;
 *  const struct Instance *ref;
 *  Write the relation in infix to a char.
 *  This function comfortably handles all the relation types.
 */

extern void WriteRelationPostfix(FILE *,CONST struct Instance *,
     CONST struct Instance *);
/*
 *  void WriteRelationPostfix(f,relinst,ref)
 *  FILE *f;
 *  const struct Instance *relinst;
 *  const struct Instance *ref;
 *  Write the relation in postfix to the file indicated.
 *  This function at the moment is only applicable to token relations.
 */

extern char *WriteRelationPostfixString(CONST struct Instance *,
     CONST struct Instance *);
/*
 *  void WriteRelationPostfixString(relinst,ref)
 *  char *f;
 *  const struct Instance *relinst;
 *  const struct Instance *ref;
 *  Write the relation in postfix to the file indicated.
 *  This function at the moment is only applicable to token relations.
 */

extern void Infix_WriteRelation(FILE *,CONST struct Instance *,
    CONST struct Instance *);
/*
 *  void Infix_WriteRelation(f,r,ref)
 *  FILE *f;
 *  const struct Instance *relinst;
 *  const struct Instance *ref;
 *  Write the relation in infixe to the file indicated.
 *  This function uses the "new" infix representation of relations.
 *  This function at the moment is only applicable to token relations.
 */

extern void WriteRelationsInTree(FILE *,struct Instance *);
/*
 *  Search for relations in an instance and write them to the
 *  file given in various formats.
 */

extern void SaveRelationVariables(FILE *fp, CONST struct relation *r);
/*
 *  Given a relation will save its variable list in the ASCEND condensed
 *  format.
 */

extern void SaveGlassBoxRelation(FILE *fp, CONST struct Instance *relinst);
/*
 *  Given a glassbox relation will save it in the ASCEND condensed
 *  format.
 */

extern void SaveTokenRelation(FILE *fp, CONST struct Instance *relinst);
/*
 *  Given a token relation will save it in the ASCEND condensed
 *  format.
 */

extern void SaveReln2GlassBox(FILE *fp, CONST struct Instance *relinst,
         char *prefix, unsigned long index);
/*
 *  Given a relation will save it and perform conversion to the glassbox
 *  format.
 */

int ConversionIsValid(enum Expr_enum old, enum Expr_enum new);
/*
 *  Gicen a relation format will return TRUE if conversion to another
 *  format is valid. It assumes that new is a valid relation type.
 */

#endif /* __RELATION_IO_H_SEEN__ */

