/*
 *  BrowserMethod.h
 *  by Kirk Abbott and Ben Allan
 *  Created: 1/94
 *  Version: $Revision: 1.7 $
 *  Version control file: $RCSfile: BrowserMethod.h,v $
 *  Date last modified: $Date: 97/07/20 20:58:33 $
 *  Last modified by: $Author: ballan $
 *
 *  This file is part of the ASCEND Tcl/Tk interface
 *
 *  Copyright 1997, Carnegie Mellon University
 *
 *  The ASCEND Tcl/Tk interface is free software; you can redistribute
 *  it and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  The ASCEND Tcl/Tk interface is distributed in hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the program; if not, write to the Free Software Foundation,
 *  Inc., 675 Mass Ave, Cambridge, MA 02139 USA.  Check the file named
 *  COPYING.  COPYING is found in ../compiler.
 */

#ifndef Browser_procedure_io_module
#define Browser_procedure_io_module

/*
 *  To include this header, you must include the following:
 *      #include "tcl.h"
 *      #include "BrowserMethod.h"
 */


extern int Asc_BrowProcedureListCmd(ClientData cdata,Tcl_Interp *interp,
                                    int argc, char *argv[]);
/*
 *  BrowWriteProcedureCmd
 *  usage: bproc [qlfdid]
 *  If no qlfdid given
 *  writes the procedure list of the current browser instance.
 *  else writes the procedure list of the given qlfdid, if exists.
 *  returns "1" on normal failures, tclerror on wierdness.
 */

extern int Asc_BrowRunInitializeCmd(ClientData cdata,Tcl_Interp *interp,
                                    int argc, char *argv[]);
/*
 *  Asc_BrowRunInitializeCmd
 *  usage: runproc [qlfdid] name
 *  Runs the named model procedure in Browser current instance,or if
 *  qlfdid given, in the qlfdid instance.
 */

extern int Asc_BrowRunAssignmentCmd(ClientData cdata,Tcl_Interp *interp,
                                    int argc, char *argv[]);
/*
 *  Asc_BrowRunAssignmentCmd
 *  usage: brow_assign [-search] value [units]  {browser inst assignment}
 *  Checks for Boolean, Integers and Reals.
 *  A null inst returns a TCL_ERROR.
 *  Bad data or setting an already set integer gets a TCL_ERROR w/message.
 *  If units are present and type is Real, will attempt to use units in
 *  assignment. If units dimensionally incompatible, returns an error.
 *  If units are missing or "*" will assign the value directly without
 *  conversion and without changing the dimensionality of the real.
 *  If value is "UNDEFINED" returns without doing anything, regardless
 *  of units.
 */

extern int Asc_BrowRunAssignQlfdidCmd2(ClientData cdata,Tcl_Interp *interp,
                                       int argc, char *argv[]);
/*
 *  Asc_BrowRunAssignQlfdidCmd2
 *  Usage : qassgn2 qlfdid value [units] {qualified id assignment}
 *  See the notes for Asc_BrowRunAssignment. The same applies only this
 *  function requires a qulaified id. An errors will return a TCL_ERROR;
 */

extern int Asc_BrowRunAssignQlfdidCmd3(ClientData cdata,Tcl_Interp *interp,
                                       int argc, char *argv[]);
/*
 *  Asc_BrowRunAssignQlfdidCmd3
 *  Usage : qassgn3 qlfdid value [units] [-relative] {qualified id assignment}
 *  Yet another variant to try to get some more speed. This version
 *  uses Asc_QlfdidSearch3, which is the mininal implementation of
 *  searching for a name.
 *  If -relative arguement provided then the assignment is expected to
 *  be relative to the results of the last call to qlfdid (tcl call).
 */

extern int Asc_BrowWriteProcedure(ClientData cdata,Tcl_Interp *interp,
                                  int argc, char *argv[]);
/*
 *  bgetproc procname pathname [search]
 *  write the statements of a named procedure
 *  (assumed to be in g_curinst unless search appears)
 *  to a full file pathname given.
 */

#endif  /* Browser_procedure_io_module loaded */



