/*
 *  Qlfdid.h
 *  by Kirk Abbott and Ben Allan
 *  Created: 1/94
 *  Version: $Revision: 1.8 $
 *  Version control file: $RCSfile: Qlfdid.h,v $
 *  Date last modified: $Date: 97/07/18 12:23:32 $
 *  Last modified by: $Author: mthomas $
 *
 *  This file is part of the ASCEND Tcl/Tk interface
 *
 *  Copyright 1997, Carnegie Mellon University
 *
 *  The ASCEND Tcl/Tk interface is free software; you can redistribute
 *  it and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  The ASCEND Tcl/Tk interface is distributed in hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the program; if not, write to the Free Software Foundation,
 *  Inc., 675 Mass Ave, Cambridge, MA 02139 USA.  Check the file named
 *  COPYING.  COPYING is found in ../compiler.
 */

#ifndef Qlfdid_module_loaded
#define Qlfdid_module_loaded

/*
 *  To include this header, you must include the following:
 *      #include "tcl.h"
 *      #include "compiler/list.h"
 *      #include "compiler/instance_enum.h"
 *      #include "interface/Qlfdid.h"
 */

struct SearchEntry {
  char *name;                      /* name of the instance */
  struct Instance *i;              /* pointer to the instance */
};


/*
 *  Result of the last qlfdid search of any kind in c or tcl.
 */
extern struct Instance *g_search_inst;

/*
 *  Result of the last qlfdid search of any kind in tcl.
 *  (C calls to qlfdid routines do not change this.)
 * 'qlfdid' and 'probe qlfdid' are known to set it.
 */
extern struct Instance *g_relative_inst;


extern char *Asc_MakeInitString(int);
/*
 *  Asc_MakeInitString(int len);
 *  Will make a string of the specified length of a predefine default
 *  length if garbage is given. Will initialize the string to the mt string.
 *  Will return a pointer to the string;
 */

extern void Asc_ReInitString(char *);
/*
 *  Asc_ReInitString;
 *  char *str;
 *  We will reinitialize a string of non-zero length to the mt string.
 *  If an ivalid string will leave it untouched.
 */

extern struct SearchEntry *Asc_SearchEntryCreate(char *,struct Instance *);
/*
 *  Asc_SearchEntryCreate(name,i);
 *  char *name;
 *  struct Instance *i;
 *  Comments :
 *  Accepts a string and a pointer to an Instance and will create a
 *  and return a SearchEntry. Will save a copy of the string. Will always
 *  try to make space for the string.
 */

extern struct Instance *Asc_SearchEntryInstance(struct SearchEntry *);
/*
 *  Asc_SearchEntryInstance(se);
 *  struct SearchEntry *se;
 *  Comments : Returns a the instance pointer from a search entry;
 */

extern char *Asc_SearchEntryName(struct SearchEntry *);
/*
 *  Asc_SearchEntryName(se)
 *  struct SearchEntry *se;
 *  Comments : Returns the name of a search entry;
 */

extern void Asc_SearchEntryDestroy(struct SearchEntry *se);
/*
 *  Asc_SearchEntryDestroy(se);
 *  struct SearchEntry *se;
 *  Comments :
 *  Will delete the memory associated with its string, set its pointers
 *  to NULL, and free the memory associated with a SearchEntry.
 */

extern void Asc_SearchListDestroy(struct gl_list_t *search_list);
/*
 *  Asc_SearchListDestroy(search_list);
 *  struct gl_list_t *search_list;
 *  Comments :
 *  Frees up the memory that was allocated for the search_list which is
 *  a list of SearchEntry(s).
 */

extern struct gl_list_t *Asc_BrowQlfdidSearch(char *, char *);
/*
 *  Asc_BrowQlfdidSearch(str,temp)
 *  char *str, char *temp;
 *  Comments :
 *  Will accept two strings, both of which will be destroyed by the
 *  operation. Searches for an instance of the qualified name. Will
 *  return of a list (of SearchEntries), with each list element being
 *  a pointer to a level in the instance tree of the qualified id and
 *  'g_search_inst' looking at the last part of the instance name.
 *  Will return NULL, whenever the qualified instance name cannot be found.
 *  If it returns NULL, the list does not exist; don't try to destroy it.
 */

extern int Asc_QlfdidSearch2(char *);
/*
 *  Asc_QlfdidSearch2(str);
 *  char *str;
 *  This function is perhaps the entry point suited for most users.
 *  It uses the function Asc_BrowQlfdidSearch, and will leave the g_search_inst
 *  looking at the found instance, or NULL. It attempts to avoid the
 *  overhead of creating search entries, for cases where the user only
 *  wants to find the instance associated with the given qualfied id.
 *  However because it still uses Asc_BrowQlfdidSearch, it incurs the cost
 *  of creating the path to the instance. For cases where this path is
 *  of no interest, but just the final instance use Asc_QlfdidSearch3 below.
 *  Returns 0 if found, 1 otherwise.
 */

extern int Asc_BrowQlfdidSearchCmd(ClientData cdata, Tcl_Interp *interp,
                                   int argc, char *argv[]);
/*
 *  Asc_BrowQlfdidSearchCmd
 *  Registered as : \"qlfdid name\"
 *  Comments :
 *  Will take a fully qualified ascend name and will search for the instance
 *  with that name. Will leave g_search_inst looking at the named instance
 *  if successful and will return a parsed string. Otherwise should return
 *  a TCL_ERROR with the message "Orphaned part", where part is the portion
 *  that could not be found.
 */

extern int Asc_QlfdidSearch3(CONST char *, int);
/*
 *  Asc_QlfdidSearch3(str,relative);
 *  const char *str;
 *  int relative;
 *  This function is still yet another entry point to the search routines.
 *  It uses the function BrowQlfdidSearch3, and will leave the g_search_inst
 *  looking at the found instance, or NULL. It attempts to avoid the
 *  overhead of creating search entries, for cases where the user only
 *  wants to find the instance associated with the given qualfied id.
 *  Returns 0 if found, 1 otherwise. This is perhaps the fastest
 *  version of the search codes.
 *
 *  If relative = 1 will start search at g_relative_inst whose value
 *  should be set by a call to qlfdid (tcl) or Asc_BrowQlfdidSearchCmd (c).
 *  Note that if the relative option is used str should be relative to
 *  g_relative_inst NOT to the simulation root.
 */

#endif /*Qlfdid_module_loaded*/



