/*
 *  Utility functions for Ascend
 *  Version: $Revision: 1.1 $
 *  Version control file: $RCSfile: old_utils.c,v $
 *  Date last modified: $Date: 97/07/18 11:49:47 $
 *  Last modified by: $Author: mthomas $
 *  Part of Ascend
 *
 *  This file is part of the Ascend Programming System.
 *
 *  Copyright (C) 1990 Thomas Guthrie Epperly, Karl Michael Westerberg
 *  Copyright (C) 1993 Joseph James Zaher
 *  Copyright (C) 1993, 1994 Benjamin Andrew Allan, Joseph James Zaher
 *
 *  The Ascend Programming System is free software; you can redistribute
 *  it and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  ASCEND is distributed in hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the program; if not, write to the Free Software Foundation,
 *  Inc., 675 Mass Ave, Cambridge, MA 02139 USA.  Check the file named
 *  COPYING.
 *
 *  This module defines the dimensionality checking and some other auxillaries
 *  for Ascend.
 */

#include <ctype.h>
#include <math.h>
#include "utilities/ascConfig.h"
#include "utilities/ascMalloc.h"
#include "general/list.h"
#include "general/dstring.h"
#include "compiler/compiler.h"
#include "compiler/fractions.h"
#include "compiler/dimen.h"
#include "compiler/functype.h"
#include "compiler/types.h"
#include "compiler/sets.h"
#include "compiler/instance_enum.h"
#include "compiler/instance_name.h"
#include "compiler/atomvalue.h"
#include "compiler/parentchild.h"
#include "compiler/symtab.h"
#include "compiler/instance_io.h"
#include "compiler/safe.h"
#include "compiler/interval.h"
#include "compiler/func.h"
#include "compiler/extfunc.h"
#include "compiler/extcall.h"
#include "compiler/setinstval.h"
#include "compiler/exprs.h"
#include "compiler/value_type.h"
#include "compiler/find.h"
#include "compiler/relation_type.h"
#include "compiler/relation.h"
#include "compiler/relation_util.h"
#include "interface/old_utils.h"

/*int g_check_dimensions_noisy=1;*/
#define GCDN g_check_dimensions_noisy

static struct InstanceName *copy_name(iname)
struct InstanceName *iname;
/*
 *  This function copies an InstanceName structure but
 *  re-uses the original string pointers from names of type
 *  StrArrayIndex and StrName.
 */
{
   struct InstanceName *copy;
   /* commenting out unused variables:    char *s,*t;  */
   copy = (struct InstanceName *)ascmalloc( sizeof(struct InstanceName) );
   switch( InstanceNameType(*iname) ) {
      case IntArrayIndex:
         SetInstanceNameType(*copy,IntArrayIndex);
         SetInstanceNameIntIndex(*copy,InstanceIntIndex(*iname));
         break;

      case StrArrayIndex:
         SetInstanceNameType(*copy,StrArrayIndex);
         SetInstanceNameStrIndex(*copy,InstanceStrIndex(*iname));
         break;

      case StrName:
         SetInstanceNameType(*copy,StrName);
         SetInstanceNameStrPtr(*copy,InstanceNameStr(*iname));
         break;
   }

   return(copy);
}

static struct gl_list_t *short_instance_namelist(inst,ref)
struct Instance *inst,*ref;
{
   struct gl_list_t *namelist = NULL;
   struct gl_list_t *path;
   path = ShortestPath(inst,ref,0,UINT_MAX);
   if( path != NULL ) {
      struct Instance *parent;
      unsigned long c;
      namelist = gl_create((long)(gl_length(path)-1));
      parent = gl_fetch(path, gl_length(path));
      for( c = gl_length(path)-1; c >= 1; c-- ) {
  struct Instance *child;
  struct InstanceName name;
  child = gl_fetch(path,c);
  name = ParentsName(parent,child);
  gl_append_ptr(namelist,(POINTER)copy_name(&name));
  parent = child;
      }
      gl_destroy(path);
   }
   return namelist;
}

static struct gl_list_t *any_instance_namelist(inst,ref)
struct Instance *inst,*ref;
{
   unsigned long ndx,nparents;

   nparents = NumberParents(inst);
   if( (inst==ref) || (ref==NULL && nparents==0) )
      return(gl_create(5L));

   for( ndx=1L; ndx <= nparents; ++ndx ) {
      struct gl_list_t *namelist;
      struct Instance *parent;
      parent = InstanceParent(inst,ndx);
      namelist = any_instance_namelist(parent,ref);
      if( namelist != NULL ) {
  struct InstanceName name;
  name = ParentsName(parent,inst);
  gl_append_ptr(namelist,(POINTER)copy_name(&name));
  return(namelist);
      }
   }
   return(NULL);
}

#define ROOM_FOR_LONG   21     /* a 64-bit long, that is */
static int size_required_for_qualified_name(namelist)
struct gl_list_t *namelist;
{
   int memreq;
   unsigned long ndx;

   memreq=0;
   for( ndx=gl_length(namelist) ; ndx > 0 ; --ndx ) {
      struct InstanceName *iname;
      iname = (struct InstanceName *)gl_fetch(namelist,ndx);
      switch( InstanceNameType(*iname) ) {
         case StrArrayIndex:  /* 4 characters for "['']" */
     memreq += strlen(InstanceStrIndex(*iname)) + 4;
     break;
  case StrName:        /* 4 characters for "['']" */
     memreq += strlen(InstanceNameStr(*iname)) + 4;
     break;
  case IntArrayIndex:
  default:
     memreq += ROOM_FOR_LONG;
     break;
      }
   }
   return(memreq+1);
}

static char *write_iname(s,iname,ndx)
char *s;
struct InstanceName *iname;
int ndx;
{
   switch( InstanceNameType(*iname) ) {
      case IntArrayIndex: {
         char buf[ROOM_FOR_LONG];
         sprintf(buf,"%ld",InstanceIntIndex(*iname));
  s += strlen(strcpy(s,"["));
  s += strlen(strcpy(s,buf));
  s += strlen(strcpy(s,"]"));
  break;
      }
      case StrArrayIndex:
  s += strlen(strcpy(s,"['"));
  s += strlen(strcpy(s,InstanceStrIndex(*iname)));
  s += strlen(strcpy(s,"']"));
  break;
      case StrName:
  if( ndx > 1 ) s += strlen(strcpy(s,"."));
  s += strlen(strcpy(s,InstanceNameStr(*iname)));
  break;
      default:   /* We've alotted ROOM_FOR_LONG characters */
         s += strlen(strcpy(s,".<error>"));
  break;
   }
   return(s);
}
#undef ROOM_FOR_LONG


static char *qualified_name(namelist)
struct gl_list_t *namelist;
{
   char *s;
   char *name;
   unsigned long ndx,len;
   int sizereq;

   sizereq = size_required_for_qualified_name(namelist);
   s = name = (char *)ascmalloc(sizereq);
   len = gl_length(namelist);
   for( s=name,ndx=1 ; ndx <= len ; ++ndx ) {
      struct InstanceName *iname;
      iname = (struct InstanceName *)gl_fetch(namelist,ndx);
      s = write_iname(s, iname,(int) ndx);
   }
   return(name);
}

char *asc_make_short_instance_name(inst, ref)
struct Instance *inst;
struct Instance *ref;
/*
 *  A string will be created with sufficient length and
 *  returned storing the qualified name of instance inst
 *  with respect to the ancestor instance ref which utilizes
 *  the fewest intermediate instance names.  If NULL is
 *  passed in for ref, the full qualified name of the
 *  instance as known by ref is returned.  Otherwise, if
 *  the subtree inst..ref is found to not exist, "?" is
 *  returned.  The string should be destroyed when it is no
 *  longer in use.
 */
{
   struct gl_list_t *namelist;
   char *name;
   namelist = short_instance_namelist(inst,ref);
   if( namelist == NULL || gl_length(namelist) == 0L ) {
      name = (char *)ascmalloc(2*sizeof(char));
      strcpy(name,"?");
   } else
      name = qualified_name(namelist);
   if( namelist )
      gl_free_and_destroy(namelist);
   return name;
}

char *asc_make_any_instance_name(inst, ref)
struct Instance *inst;
struct Instance *ref;
/*
 *  The same holds here as above except any one of the alias
 *  names for the inst may be returned and not necessarily
 *  that which utilizes the shortest path between inst and ref.
 *  This will be expected to perform faster.
 */
{
   struct gl_list_t *namelist;
   char *name;
   namelist = any_instance_namelist(inst,ref);
   if( namelist == NULL || gl_length(namelist) == 0L ) {
      name = (char *)ascmalloc(2*sizeof(char));
      strcpy(name,"?");
   } else
      name = qualified_name(namelist);
   if( namelist )
      gl_free_and_destroy(namelist);
   return name;
}


static struct Instance *get_child(inst, iname)
struct Instance *inst;
struct InstanceName *iname;
{
   unsigned long ndx;
   ndx = ChildSearch(inst,iname);
   if( ndx ) return InstanceChild(inst,ndx);
   else return NULL;
}

static struct Instance *next_instance(nptr, inst)
char **nptr;
struct Instance *inst;
/*
 *  Returns the next instance given by the child of inst whose
 *  instance name matches that of the beginning of the string
 *  *nptr.  *nptr is updated to point to the next child name in
 *  the qualified name.  Returns NULL when finished or error.
 */
{
   char save;
   char *start;
   struct InstanceName iname;
   struct Instance *next;
   long index;

   switch( isalpha(**nptr) ? 'a' : **nptr ) {
      case '.':
         ++*nptr;
         if( !(isalpha(**nptr) || **nptr == '_') )
     return(NULL);
  /* FALL THROUGH OTHERWISE */

      case 'a':   /* represents all letters */
      case '_':
         start = *nptr;
         while( isalnum(**nptr) || **nptr == '_' ) ++*nptr;
         save = **nptr;
         **nptr = '\0';
         SetInstanceNameType(iname,StrName);
         SetInstanceNameStrPtr(iname,start);
  next = get_child(inst,&iname);
  **nptr = save;
         return(next);

      case '[':
         if( *++*nptr == '\'' ) {
            start = ++*nptr;
            while( **nptr != '\'' && **nptr != '\0' ) ++*nptr;
            if( **nptr != '\'' ) {
               *nptr = start;
               return(NULL);
            }
     save = **nptr;
            **nptr = '\0';
            SetInstanceNameType(iname,StrArrayIndex);
            SetInstanceNameStrIndex(iname,start);
     next = get_child(inst,&iname);
     **nptr = save;
            if( *++*nptr == ']' ) ++*nptr;
            return(next);
         }
         /* Otherwise, it must be a number */

         start = *nptr;
         while( **nptr != ']' && **nptr != '\0' ) ++*nptr;
         if( **nptr != ']' ) {
            *nptr = start;
            return(NULL);
         }
         ++*nptr;

         if( sscanf(start,"%ld",&index) < 1 )
            return(NULL);

         SetInstanceNameType(iname,IntArrayIndex);
         SetInstanceNameIntIndex(iname,index);
  next = get_child(inst,&iname);
         return(next);

      default:
         return(NULL);
   }
}

struct Instance *asc_get_instance(name, ref)
char *name;
struct Instance *ref;
{
   while( ref != NULL && *name != '\0' )
      ref = next_instance(&name,ref);
   return(ref);
}


struct Instance *asc_get_child_by_name(inst, name)
struct Instance *inst;
char *name;
{
   struct InstanceName childname;

   childname.t = StrName;
   childname.u.name = name;
   return( get_child(inst,&childname) );
}


static char *write_char(str,slim,c)
char *str,*slim;
int c;
{
   if( str < slim ) *(str++) = c;
   return(str);
}

static char *write_string(str,slim,s)
char *str,*slim;
char *s;
{
   for( ; *s != '\0' ; ++s )
      str = write_char(str,slim,*s);
   return(str);
}

#define SIZE_INCREMENT 16
#define ROOM_FOR_INT  11    /* a 32-bit int, that is */
char *asc_make_dimensions(dim)
dim_type *dim;
{
   boolean first;
   char *dimens;
   char *str, *slim;
   int size = 0;

   if( IsWild(dim) ) {
      dimens = (char *)ascmalloc(2*sizeof(char));
      sprintf(dimens,"*");
      return( dimens );
   }

   str = slim = dimens = NULL;
   while( str==slim ) {
      int n;
      size += SIZE_INCREMENT;
      if( dimens != NULL ) ascfree(dimens);
      str = dimens = (char *)ascmalloc(size*sizeof(char));
      slim = str + size;
      first = TRUE;
      for( n=0 ; n<NUM_DIMENS ; ++n ) {
  struct fraction frac;
  frac = GetDimFraction(*dim,n);
  if( Numerator(frac) == 0 )
     continue;

  if( !first ) str = write_string(str, slim, " * ");
  str = write_string(str, slim, DimName(n));
  if( Denominator(frac) == 1 ) {
     if( Numerator(frac) != 1 ) {
               char buf[ROOM_FOR_INT];
        sprintf(buf,"%d",(int)Numerator(frac));
        str = write_char(str, slim, '^');
        str = write_string(str, slim, buf);
     }
  } else {
            char buf[ROOM_FOR_INT];
     str = write_string(str, slim, "^(");
     sprintf(buf,"%d",(int)Numerator(frac));
     str = write_string(str, slim, buf);
     str = write_char(str, slim, '/');
     sprintf(buf,"%d",(int)Denominator(frac));
     str = write_string(str, slim, buf);
     str = write_char(str, slim, ')');
  }
  first = FALSE;
      }
   }
   *str = '\0';
   return(dimens);
}
#undef ROOM_FOR_INT
#undef SIZE_INCREMENT

#ifdef THIS_MAY_BE_UNUSED_CODE
/* commenting out unused functions  mthomas.96.09.20 */
/* dim checking stuff invokable at any time. */
static double frac_to_real(frac)
struct fraction frac;
{
   return( (double)Numerator(frac) / (double)Denominator(frac) );
}
#endif

#define START 10000  /* largest power of 10 held by a short */
static struct fraction real_to_frac(real)
double real;
{
   short  num, den;
   for( den=START; den>1 && fabs(real)*den>SHRT_MAX; den /= 10 ) ;
   num = (short)(fabs(real)*den + 0.5);
   if( real < 0.0 ) num = -num;
   return( CreateFraction(num,den) );
}
#undef START


static int nargs(type)
enum Expr_enum type;
{
   switch(type) {
   case e_int:
   case e_real:
   case e_var:
   case e_zero:
     return(0);

   case e_func:
   case e_uminus:
      return(1);

   case e_plus:
   case e_minus:
   case e_times:
   case e_divide:
   case e_power:
   case e_ipower:
      return(2);

   default:
      FPRINTF(stderr,"Unknown relation term type.\n");
      return(0);
   }
}

struct dimnode {
   dim_type d;
   enum Expr_enum type;
   short int_const;
   double real_const;
   struct fraction power;
};

static int IsZero(struct dimnode *node)
{
   if( node->type==e_real && node->real_const==0.0 )
      return TRUE;
   return FALSE;
}


static void apply_term_dimensions(struct relation *rel,
                                  struct relation_term *rt,
                                  struct dimnode *first,
                                  struct dimnode *second,
                                  boolean *con,             /* consistent ? */
                                  boolean *wild)            /* wild ? */
{
  enum Expr_enum type;

  switch(type=RelationTermType(rt)) {
  case e_int:
    CopyDimensions(Dimensionless(),&(first->d));
    first->int_const = (short)TermInteger(rt);
    first->type = type;
    break;

  case e_zero:
    CopyDimensions(Dimensionless(),&(first->d));
    first->real_const = TermReal(rt);
    first->type = type;
    break;

  case e_real:
    CopyDimensions(TermDimensions(rt),&(first->d));
    first->real_const = TermReal(rt);
    first->type = type;
    break;

  case e_var: {
    struct Instance *var = RelationVariable(rel,TermVarNumber(rt));
    CopyDimensions(RealAtomDims(var),&(first->d));
    first->type = type;
    break;
  }
  case e_func: {
    enum Func_enum id = FuncId(TermFunc(rt));
    switch( id ) {
    case F_SQR:
      /* no checking, just simple scaling */
      first->d = ScaleDimensions(&(first->d),CreateFraction(2,1));
      break;

    case F_CUBE:
      /* no checking, just simple scaling */
      first->d = ScaleDimensions(&(first->d),CreateFraction(3,1));
      break;

    case F_SQRT:
      /* no checking, just simple scaling */
      first->d = ScaleDimensions(&(first->d),CreateFraction(1,2));
      break;

    case F_CBRT:
      /* no checking, just simple scaling */
      first->d = ScaleDimensions(&(first->d),CreateFraction(1,3));
      break;

    case F_HOLD:
    case F_ABS:
      /*  For abs, there is nothing to do, so just break out.
       *  assuming first->d is o.k.
       */
      break;

    case F_EXP:
    case F_LN:
    case F_LNM:
    case F_LOG:
    case F_ERF:
    case F_SINH:
    case F_COSH:
    case F_TANH:
    case F_ARCSINH:
    case F_ARCCOSH:
    case F_ARCTANH:
      /*
       *  first must now be dimensionless.  It will
       *  end up dimensionless as well.
       */
      if( IsWild(&(first->d)) && !IsZero(first) ) {
        char *name = (char *)FuncName(TermFunc(rt));
        if( !*wild ) *wild = TRUE;
        if (GCDN) {
          FPRINTF(stderr,"ERROR:  Relation has wild dimensions\n");
          FPRINTF(stderr,"        in function %s.\n",name);
        }
      } else if( !IsWild(&(first->d)) &&
                 CmpDimen(&(first->d),Dimensionless()) ) {
        char *name = (char *)FuncName(TermFunc(rt));
        char *dimstring = asc_make_dimensions(&(first->d));
        if( *con ) *con = FALSE;
        if (GCDN) {
          FPRINTF(stderr,"ERROR:  Function %s called\n",name);
          FPRINTF(stderr,"        with dimensions %s.\n",dimstring);
        }
        ascfree(dimstring);
      }
      CopyDimensions(Dimensionless(),&(first->d));
      break;

    case F_SIN:
    case F_COS:
    case F_TAN: {
      /*
       *  first must now be of dimension D_PLANE_ANGLE.
       *  It will then be made dimensionless.
       */
      if( IsWild(&(first->d)) && !IsZero(first) ) {
        char *name = (char *)FuncName(TermFunc(rt));
        if( !*wild ) *wild = TRUE;
        if (GCDN) {
          FPRINTF(stderr,"ERROR:  Relation has wild dimensions\n");
          FPRINTF(stderr,"        in function %s.\n",name);
        }
      } else if( !IsWild(&(first->d)) &&
                 CmpDimen(&(first->d),TrigDimension()) ) {
        char *dimstring = asc_make_dimensions(&(first->d));
        char *name = (char *)FuncName(TermFunc(rt));
        if( *con ) *con = FALSE;
        if (GCDN) {
          FPRINTF(stderr,"ERROR:  Function %s called with\n",name);
          FPRINTF(stderr,"        dimensions %s.\n",dimstring);
        }
        ascfree(dimstring);
      }
      CopyDimensions(Dimensionless(),&(first->d));
      break;
    }

    case F_ARCSIN:
    case F_ARCCOS:
    case F_ARCTAN:
      /*
       *  first must now be dimensionless.  It will
       *  end up with dimension D_PLANE_ANGLE
       */
      if( IsWild(&(first->d)) && !IsZero(first) ) {
        char *name = (char *)FuncName(TermFunc(rt));
        if( !*wild ) *wild = TRUE;
        if (GCDN) {
          FPRINTF(stderr,"ERROR:  Relation has wild dimensions\n");
          FPRINTF(stderr,"        in function %s.\n",name);
        }
      } else if( !IsWild(&(first->d)) &&
                 CmpDimen(&(first->d),Dimensionless()) ) {
        char *dimstring = asc_make_dimensions(&(first->d));
        char *name =(char *) FuncName(TermFunc(rt));
        if( *con ) *con = FALSE;
        if (GCDN) {
          FPRINTF(stderr,"ERROR:  Function %s called with\n",name);
          FPRINTF(stderr,"        dimensions %s.\n",dimstring);
        }
        ascfree(dimstring);
      }
      CopyDimensions(TrigDimension(),&(first->d));
      break;
    }
    first->type = type;
    break;
  }

  case e_uminus:
    first->type = type;
    break;

  case e_times:
    first->d = AddDimensions(&(first->d),&(second->d));
    first->type = type;
    break;

  case e_divide:
    first->d = SubDimensions(&(first->d),&(second->d));
    first->type = type;
    break;

  case e_power:
  case e_ipower:
    if( IsWild(&(second->d)) && !IsZero(second) ) {
      if( !*wild ) *wild = TRUE;
      if (GCDN) {
        FPRINTF(stderr,"ERROR:  Relation has wild dimensions\n");
        FPRINTF(stderr,"        in exponent.\n");
      }
    } else if( !IsWild(&(second->d)) &&
               CmpDimen(&(second->d),Dimensionless()) ) {
      char *dimstring = asc_make_dimensions(&(second->d));
      if( *con ) *con = FALSE;
      if (GCDN) {
        FPRINTF(stderr,"ERROR:  Exponent has dimensions %s.\n",
                dimstring);
      }
      ascfree(dimstring);
    }
    CopyDimensions(Dimensionless(),&(second->d));
    switch( second->type ) {
    case e_int:
      if( !IsWild(&(first->d)) &&
          CmpDimen(&(first->d),Dimensionless()) ) {
        struct fraction power;
        power = CreateFraction(second->int_const,1);
        first->d = ScaleDimensions(&(first->d),power);
      }
      break;

    case e_real:
      if( !IsWild(&(first->d)) &&
          CmpDimen(&(first->d),Dimensionless()) ) {
        struct fraction power;
        power = real_to_frac(second->real_const);
        first->d = ScaleDimensions(&(first->d),power);
      }
      break;

    default:
      if( IsWild(&(first->d)) && !IsZero(first) ) {
        if( !*wild ) *wild = TRUE;
        if (GCDN) {
          FPRINTF(stderr,"ERROR:  Relation has wild dimensions\n");
          FPRINTF(stderr,"        raised to a non-constant power.\n");
        }
      } else if( !IsWild(&(first->d)) &&
                 CmpDimen(&(first->d),Dimensionless()) ) {
        char *dimstring = asc_make_dimensions(&(first->d));
        if( *con ) *con = FALSE;
        if (GCDN) {
          FPRINTF(stderr,
                  "ERROR:  Dimensions %s are\n",dimstring);
          FPRINTF(stderr,
                  "        raised to a non-constant power.\n");
        }
        ascfree(dimstring);
      }
      CopyDimensions(Dimensionless(),&(first->d));
      break;

    }
    first->type = type;
    break;

  case e_plus:
  case e_minus:
    if( IsWild(&(first->d)) && IsZero(first) ) {
      /* first wild zero */
      CopyDimensions(&(second->d),&(first->d));
      first->type = second->type;
      if( second->type==e_int )
        first->int_const = second->int_const;
      if( second->type==e_real )
        first->real_const = second->real_const;
    } else if( IsWild(&(first->d)) && !IsZero(first) ) {
      /* first wild non-zero */
      if( IsWild(&(second->d)) && !IsZero(second) ) {
        /* second wild non-zero */
        if( !*wild ) *wild = TRUE;
        if (GCDN) {
          FPRINTF(stderr,"ERROR:  %s has wild dimensions on\n",
                  type==e_plus ? "Addition":"Subtraction");
          FPRINTF(stderr,"        left and right hand sides.\n");
        }
        first->type = type;
      } else if( !IsWild(&(second->d)) ) {
        /* second not wild */
        if( !*wild ) *wild = TRUE;
        if (GCDN) {
          FPRINTF(stderr,"ERROR:  %s has wild dimensions on\n",
                  type==e_plus ? "Addition":"Subtraction");
          FPRINTF(stderr,"        left hand side.\n");
        }
        CopyDimensions(&(second->d),&(first->d));
        first->type = type;
      }
    } else if( !IsWild(&(first->d)) ) {
      /* first not wild */
      if( IsWild(&(second->d)) && !IsZero(second) ) {
        /* second wild non-zero */
        if( !*wild ) *wild = TRUE;
        if (GCDN) {
          FPRINTF(stderr,"ERROR:  %s has wild dimensions on\n",
                  type==e_plus ? "Addition":"Subtraction");
          FPRINTF(stderr,"        right hand side.\n");
        }
        first->type = type;
      } else if ( !IsWild(&(second->d)) ) {
        /* second not wild */
        if( CmpDimen(&(first->d),&(second->d)) ) {
          char *dimfirst = asc_make_dimensions(&(first->d));
          char *dimsecond = asc_make_dimensions(&(second->d));
          if( *con ) *con = FALSE;
          if (GCDN) {
            FPRINTF(stderr,"ERROR:  %s has dimensions %s on left\n",
                    type==e_plus ? "Addition":"Subtraction",
                    dimfirst);
            FPRINTF(stderr,"        and dimensions %s on right.\n",
                    dimsecond);
          }
          ascfree(dimfirst);
          ascfree(dimsecond);
        }
        first->type = type;
      }
    }
    break;

  default:
    FPRINTF(stderr,"ERROR:  Unknown relation term type.\n");
    if( *con ) *con = FALSE;
    first->type = type;
    break;
  }
}


int asc_check_dimensions(rel,dimens)
struct relation *rel;
dim_type *dimens;
{
   struct dimnode *stack, *sp;
   boolean consistent = TRUE;
   boolean wild = FALSE;
   unsigned long c, len;

   if ( !IsWild(RelationDim(rel)) ) { /* don't do this twice */
     CopyDimensions(RelationDim(rel),dimens);
     return 2;
   }
   sp = stack = (struct dimnode *)
      ascmalloc(RelationDepth(rel)*sizeof(struct dimnode));
   switch( RelationRelop(rel) ) {
      case e_less:
      case e_lesseq:
      case e_greater:
      case e_greatereq:
      case e_equal:
      case e_notequal:
         /* Working on the left-hand-side */
         len = RelationLength(rel,TRUE);
         for( c = 1; c <= len; c++ ) {
            struct relation_term *rt;
            rt = (struct relation_term *)RelationTerm(rel,c,TRUE);
            sp += 1-nargs(RelationTermType(rt));
            apply_term_dimensions(rel,rt,sp-1,sp,&consistent,&wild);
         } /* stack[0].d contains the dimensions of the lhs expression */

         /* Now working on the right-hand_side */
         len = RelationLength(rel,FALSE);
         for( c = 1; c <= len; c++ ) {
            struct relation_term *rt;
            rt = (struct relation_term *) RelationTerm(rel,c,FALSE);
            sp += 1-nargs(RelationTermType(rt));
            apply_term_dimensions(rel,rt,sp-1,sp,&consistent,&wild);
         } /* stack[1].d contains the dimensions of the rhs expression */

         if( IsWild(&(stack[0].d)) || IsWild(&(stack[1].d)) ) {
     if( IsWild(&(stack[0].d)) && !IsZero(&(stack[0])) ) {
        if( !wild ) wild = TRUE;
               if (GCDN) {
          FPRINTF(stderr,"ERROR:  Relation has wild dimensions\n");
          FPRINTF(stderr,"        on left hand side.\n");
               }
     }
     if( IsWild(&(stack[1].d)) && !IsZero(&(stack[1])) ) {
        if( !wild ) wild = TRUE;
               if (GCDN) {
          FPRINTF(stderr,"ERROR:  Relation has wild dimensions\n");
          FPRINTF(stderr,"        on right hand side.\n");
               }
     }

         } else if( CmpDimen(&(stack[0].d),&(stack[1].d)) ) {
            char *dimfirst = asc_make_dimensions(&(stack[0].d));
            char *dimsecond = asc_make_dimensions(&(stack[1].d));
            if( consistent ) consistent = FALSE;
            if (GCDN) {
              FPRINTF(stderr,"ERROR:  Relation has dimensions %s on left\n",
   	    dimfirst);
              FPRINTF(stderr,"        and dimensions %s on right.\n",
      dimsecond);
            }
            ascfree(dimfirst);
            ascfree(dimsecond);
         }
         break;

      case e_maximize:
      case e_minimize:
         /* Working on the left-hand-side */
         len = RelationLength(rel,TRUE);
         for( c = 1; c <= len; c++ ) {
            struct relation_term *rt;
            rt = (struct relation_term *) RelationTerm(rel,c,TRUE);
            sp += 1-nargs(RelationTermType(rt));
            apply_term_dimensions(rel,rt,sp-1,sp,&consistent,&wild);
         } /* stack[0].d contains the dimensions of the lhs expression */

  if( IsWild(&(stack[0].d)) && !IsZero(&(stack[0])) ) {
     if( !wild ) wild = TRUE;
            if (GCDN) {
       FPRINTF(stderr,"ERROR:  Objective has wild dimensions.\n");
            }
  }
         break;

      default:
         FPRINTF(stderr,"ERROR:  Unknown relation type.\n");
         if( consistent ) consistent = FALSE;
         break;
   }
   CopyDimensions(&(stack[0].d),dimens);
   ascfree(stack);
   return( consistent && !wild );
}
