/*
 *  Signal handling protocol definitions for ASCEND
 *  May 27, 1997
 *  By Benjamin Andrew Allan
 *  Version: $Revision: 1.3 $
 *  Version control file: $RCSfile: ascSignal.h,v $
 *  Date last modified: $Date: 97/07/18 12:04:17 $
 *  Last modified by: $Author: mthomas $
 *  Part of Ascend
 *
 *  This file is part of the Ascend Programming System.
 *
 *  Copyright (C) 1997 Benjamin Andrew Allan
 *
 *  The Ascend Programming System is free software; you can redistribute
 *  it and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  ASCEND is distributed in hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the program; if not, write to the Free Software Foundation,
 *  Inc., 675 Mass Ave, Cambridge, MA 02139 USA.  Check the file named
 *  COPYING.
 *
 */
#include <signal.h>
#include <setjmp.h>
#include "utilities/ascConfig.h"
#ifdef __WIN32__
#include <process.h>
#else
#include <unistd.h>
#endif
#include "general/list.h"

#ifndef lint
static CONST char ascSignalRCS[] = "$Id: ascSignal.h,v 1.3 97/07/18 12:04:17 mthomas Exp $";
#endif

#define TRAPPTR(a) void (*(a))(int)
/*
 * This macro defines a pointer to a signal handler named a.
 * Substitute your variable name and
 * TRAPPTR(yourname); is a function pointer declaration.
 * typedef instead?
 */

#define TPTYPE void (*)(int)
/*
 * This macro can be used like  tp = (TPTYPE)foo;
 * or in function prototypes to indicate a signal handling
 * function pointer is the argument.
 */

#define MAX_TRAP_DEPTH 40L
/*
 * This is the maximum number of traps that can be nested.
 * with push and pop below.
 */

extern jmp_buf *g_fpe_env;
extern jmp_buf *g_seg_env;
extern jmp_buf *g_int_env;
extern jmp_buf *g_foreign_code_call_env;
/*
 * not currently in use, but should be when we get to a unified
 * standard for signal handling.
 */

extern int Asc_SignalInit(void);
/* int err = Asc_SignalInit();
 * Returns 0 if successful, 1 if out of memory, 2 otherwyse.
 * Does not establish any traps, just the stacks for
 * maintaining them. Cannot be called twice successfully.
 * Currently we support stacks for the signals:
 * SIGFPE, SIGINT, SIGSEGV.
 */

extern void Asc_SignalDestroy(void);
/* Asc_SignalDestroy();
 * Clears and destroys the stacks of signal handlers.
 */

extern void Asc_SignalRecover(void);
/* This function reinstalls the most recent signal handlers this module
 * has been informed of. This should be called after every
 * trapped exception and at any other time when the status of
 * exception handlers may have become not well defined.
 * The most recently pushed handler is installed for each supported
 * signal.
 */

extern int Asc_SignalHandlerPush(int, TPTYPE);
/* 
 * err =  Asc_SignalHandlerPush(signum, tp)
 * Adds a handler to the stack of signal handlers for the given signal.
 * There is a maximum stack limit, so returns 1 if limit exceeded.
 * Returns -1 if stack of signal requested does not exist.
 * Pushing a NULL handler tp does NOT change anything at all.
 * On a successful return, the handler has been installed and will
 * remain installed until a Asc_SignalHandlerPop or another push.
 * (Remain installed part is TRUE only if Asc_SignalRecover is used
 * properly after every longjmp.)
 */

extern int Asc_SignalHandlerPop(int, TPTYPE);
/*
 * err = Asc_SignalHandlerPop(signum, tp);
 * Pops the last trap off the stack of signum trap functions and
 * installs the new last trap. Sideeffects: reinstalls all other
 * currently pushed traps, also.
 * If called with tp != NULL, checks that the trap popped is the
 * same as tp and returns nonzero if it is not.
 * If stack is empty, returns nonzero.
 */
